/******************************************************************************
 * The contents of this file are subject to the   Compiere License  Version 1.1
 * ("License"); You may not use this file except in compliance with the License
 * You may obtain a copy of the License at http://www.compiere.org/license.html
 * Software distributed under the License is distributed on an  "AS IS"  basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 * The Original Code is Compiere ERP & CRM Smart Business Solution. The Initial
 * Developer of the Original Code is Jorg Janke. Portions created by Jorg Janke
 * are Copyright (C) 1999-2005 Jorg Janke.
 * All parts are Copyright (C) 1999-2005 ComPiere, Inc.  All Rights Reserved.
 * Contributor(s): ______________________________________.
 *****************************************************************************/
package org.compiere.impexp;

import java.sql.*;
import java.util.*;
import java.util.logging.*;
import org.compiere.model.*;
import org.compiere.util.*;


/**
 *	Import Format Model 
 *	
 *  @author Jorg Janke
 *  @version $Id: MImpFormat.java,v 1.1 2006/09/25 14:19:37 hans Exp $
 */
public class MImpFormat extends X_AD_ImpFormat
{

	public MImpFormat (X_I_ImpFormat_Row imp)
	{
		this (imp.getCtx(), imp.getAD_ImpFormat_ID(), imp.get_TrxName());
		setClientOrg(imp);
		setUpdatedBy(imp.getUpdatedBy());
		setAD_Org_ID(imp.getAD_Org_ID());
		setName(imp.getImpFormatName());
		setDescription(imp.getDescription());
		setFormatType(imp.getFormatType());
		setAD_Table_ID(imp.getAD_Table_ID());
	}	//	MPrintFormat
	/**
	 * 	Standard Constructor
	 *	@param ctx context
	 *	@param AD_ImpFormat_ID id
	 */
	public MImpFormat (Properties ctx, int AD_ImpFormat_ID, String trxName)
	{
		super (ctx, AD_ImpFormat_ID, trxName);
	}	//	MImpFormat

	/**
	 * 	Load Constructor
	 *	@param ctx context
	 *	@param rs result set
	 */
	public MImpFormat (Properties ctx, ResultSet rs, String trxName)
	{
		super(ctx, rs, trxName);
	}	//	MImpFormat
	
	/**
	 * 	Get (all) Rows
	 *	@return array of Rows
	 */
	public MImpFormatRow[] getRows()
	{
		ArrayList<MImpFormatRow> list = new ArrayList<MImpFormatRow>();
		String sql = "SELECT * FROM AD_ImpFormat_Row "
			+ "WHERE AD_ImpFormat_ID=? "
			+ "ORDER BY SeqNo";
		PreparedStatement pstmt = null;
		try
		{
			pstmt = DB.prepareStatement (sql, get_TrxName());
			pstmt.setInt (1, getAD_ImpFormat_ID());
			ResultSet rs = pstmt.executeQuery ();
			while (rs.next ())
				list.add(new MImpFormatRow (getCtx(), rs, get_TrxName()));
			rs.close ();
			pstmt.close ();
			pstmt = null;
		}
		catch (Exception e)
		{
			log.log(Level.SEVERE, "getRows", e);
		}
		try
		{
			if (pstmt != null)
				pstmt.close ();
			pstmt = null;
		}
		catch (Exception e)
		{
			pstmt = null;
		}
		MImpFormatRow[] retValue = new MImpFormatRow[list.size ()];
		list.toArray (retValue);
		return retValue;
	}	//	getRows
	
	/**
	 * 	Get (all) Rows
	 *	@return array of Rows
	 */
	public static int getAD_ImportFormat_ID(String formatName)
	{
		int AD_ImportFormat_ID = 0;
		String sql = "SELECT AD_ImpFormat_ID FROM AD_ImpFormat "
			+ "WHERE name=? ";
		PreparedStatement pstmt = null;
		try
		{
			pstmt = DB.prepareStatement (sql, null);
			pstmt.setString( 1, formatName);
			ResultSet rs = pstmt.executeQuery ();
			while (rs.next ())
				AD_ImportFormat_ID = rs.getInt( 1 );
			rs.close ();
			pstmt.close ();
			pstmt = null;
		}
		catch (Exception e)
		{
			try
			{
				if (pstmt != null)
					pstmt.close ();
				pstmt = null;
			}
			catch (Exception ee)
			{
				pstmt = null;
			}
		}
		
		return AD_ImportFormat_ID;
	}	//	getRows
	
}	//	MImpFormat
