/******************************************************************************
 * The contents of this file are subject to the   Compiere License  Version 1.1
 * ("License"); You may not use this file except in compliance with the License
 * You may obtain a copy of the License at http://www.compiere.org/license.html
 * Software distributed under the License is distributed on an  "AS IS"  basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 * The Original Code is Compiere ERP & CRM Smart Business Solution. The Initial
 * Developer of the Original Code is Jorg Janke. Portions created by Jorg Janke
 * are Copyright (C) 1999-2005 Jorg Janke.
 * All parts are Copyright (C) 1999-2005 ComPiere, Inc.  All Rights Reserved.
 * Contributor(s): ______________________________________.
 *****************************************************************************/
package org.compiere.model;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Properties;
import java.util.logging.Level;

import org.compiere.util.DB;
import org.compiere.util.Env;
import org.compiere.util.Msg;

/**
 *	BPartner Callouts.
 *	
 *  @author Jirimuto
 *  @version $Id: CalloutBPartner.java,v 1.2 2006/10/12 02:15:04 jrmt Exp $
 */
public class CalloutBPartner extends CalloutEngine
{
	/**	Debug Steps			*/
	private boolean steps = false;

	/**
	 *  @param ctx      Context
	 *  @param WindowNo current Window No
	 *  @param mTab     Model Tab
	 *  @param mField   Model Field
	 *  @param value    The new value
	 *  @return Error message or ""
	 */
	public String Address (Properties ctx, int WindowNo, GridTab mTab, GridField mField, Object value)
	{
		Integer C_Location_ID = (Integer)value;
		if (C_Location_ID == null || C_Location_ID.intValue() == 0)
			return "";
		setCalloutActive(ctx,true);
		
		String addressKana = (String)mTab.getValue("AddressKana");
		if( addressKana == null || addressKana.trim().length() == 0 ){
			MLocation m_location = new MLocation(Env.getCtx(), C_Location_ID, null);
			
			PreparedStatement pstmt = null;
			StringBuffer sql = new StringBuffer ("SELECT X_ADDRESS, X_REGION_NAME_KANA, X_CITY_NAME_KANA, X_ADDRESS_KANA" 
			    + " FROM X_POSTAL"
				+ " WHERE IsActive='Y'" 
				+ " AND C_COUNTRY_ID=" + m_location.getC_Country_ID()
				+ " AND POSTAL='" + m_location.getPostal()+m_location.getPostal_Add()+"'"
				+ " AND C_REGION_ID=" + m_location.getRegion().getC_Region_ID()
				+ " AND X_CITY_NAME='" + m_location.getCity()+"'");
			try
			{
				pstmt = DB.prepareStatement (sql.toString(), null);
				ResultSet rs = pstmt.executeQuery ();
				while ( rs.next () )
				{
					String address1 = rs.getString( 1 );
					if( m_location.getAddress1()!= null && m_location.getAddress1().startsWith( address1 ) ){
						String AddressKana = rs.getString( 2 ) + " " + rs.getString( 3 ) + " " + rs.getString( 4 );
						mTab.setValue("AddressKana", AddressKana);
						break;
					}
				}
				rs.close ();
				pstmt.close ();
				pstmt = null;
			}
			catch (Exception e)
			{
				log.log(Level.SEVERE, sql.toString(), e);
				try
				{
					if (pstmt != null)
						pstmt.close ();
				}
				catch (Exception pe)
				{
				}
				pstmt = null;
			}
		}
		setCalloutActive(ctx,false);

//		String sameAddressBPartner = getSameAddressBPartner( C_Location_ID.intValue() );
//		if( sameAddressBPartner != null ){
//			mTab.fireDataStatusEEvent("sameAddressBPartnerExist",
//					Msg.translate(Env.getCtx(), "sameAddressBPartnerExist") +" : " + sameAddressBPartner,	false );
//		}
		
		return "";
	}	//	address
	
	public String getSameAddressBPartner( int C_Location_ID ){
		
		String result = null;
		MLocation m_location = new MLocation(Env.getCtx(), C_Location_ID, null );
		
		PreparedStatement pstmt = null;
		StringBuffer sql = new StringBuffer ("SELECT C_BPARTNER.* FROM C_BPARTNER, C_BPARTNER_LOCATION, C_LOCATION"
			+ " WHERE C_BPARTNER.IsActive='Y' AND C_BPARTNER_LOCATION.IsActive='Y' AND C_LOCATION.IsActive='Y' " 
			+ " AND C_LOCATION.C_LOCATION_ID = C_BPARTNER_LOCATION.C_LOCATION_ID"
			+ " AND C_BPARTNER_LOCATION.C_BPARTNER_ID = C_BPARTNER.C_BPARTNER_ID"
			+ " AND C_LOCATION.C_COUNTRY_ID=" + m_location.getC_Country_ID()
			+ " AND C_LOCATION.POSTAL='" + m_location.getPostal()+"'" 
			+ " AND C_LOCATION.POSTAL_ADD='" + m_location.getPostal_Add()+"'"
			+ " AND C_LOCATION.C_REGION_ID=" + m_location.getRegion().getC_Region_ID()
			+ " AND C_LOCATION.CITY='" + m_location.getCity()+"'"
			+ " AND C_LOCATION.ADDRESS1='" + m_location.getAddress1()+"'"
			+ " AND C_LOCATION.C_LOCATION_ID<>" + m_location.getC_Location_ID());
		try
		{
			pstmt = DB.prepareStatement (sql.toString(), null);
			ResultSet rs = pstmt.executeQuery ();
			if( rs.next() ){
				MBPartner bpartner = new MBPartner(Env.getCtx(), rs, null );
				result = bpartner.getName();
			}
			rs.close ();
			pstmt.close ();
			pstmt = null;
		}
		catch (Exception e)
		{
			log.log(Level.SEVERE, sql.toString(), e);
			try
			{
				if (pstmt != null)
					pstmt.close ();
			}
			catch (Exception pe)
			{
			}
			pstmt = null;
		}
	
		return result;
	}
		
	
}	//	CalloutBPartner

