/******************************************************************************
 * The contents of this file are subject to the   Compiere License  Version 1.1
 * ("License"); You may not use this file except in compliance with the License
 * You may obtain a copy of the License at http://www.compiere.org/license.html
 * Software distributed under the License is distributed on an  "AS IS"  basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 * The Original Code is Compiere ERP & CRM Smart Business Solution. The Initial
 * Developer of the Original Code is Jorg Janke. Portions created by Jorg Janke
 * are Copyright (C) 1999-2005 Jorg Janke.
 * All parts are Copyright (C) 1999-2005 ComPiere, Inc.  All Rights Reserved.
 * Contributor(s): ______________________________________.
 *****************************************************************************/
package org.compiere.process;

import java.math.BigDecimal;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Timestamp;
import java.util.logging.Level;

import org.compiere.model.X_I_Movement;
import org.compiere.model.MMovement;
import org.compiere.model.MMovementLine;
import org.compiere.util.DB;
import org.compiere.util.Env;
import org.compiere.util.TimeUtil;

/**
 *	Import Movement f?��om I_Movement
 *
 * 	@author 	Jorg Janke
 * 	@version 	$Id: ImportMovement.java,v 1.1 2006/09/25 14:19:25 hans Exp $
 */
public class ImportMovement extends SvrProcess
{
	/**	Client to be imported to		*/
	private int				p_AD_Client_ID = 0;
	/**	Organization to be imported to	*/
	private int				p_AD_Org_ID = 0;
	/**	Locator to be imported to		*/
	private int				p_M_Locator_ID = 0;
	/**	LocatorTo to be imported to		*/
	private int				p_M_LocatorTo_ID = 0;
	/**	Default Date					*/
	private Timestamp		p_MovementDate = null;
	/**	Delete old Imported				*/
	private boolean			p_DeleteOldImported = false;

	/**
	 *  Prepare - e.g., get Parameters.
	 */
	protected void prepare()
	{
		ProcessInfoParameter[] para = getParameter();
		for (int i = 0; i < para.length; i++)
		{
			String name = para[i].getParameterName();
			if (para[i].getParameter() == null)
				;
			else if (name.equals("AD_Client_ID"))
				p_AD_Client_ID = ((BigDecimal)para[i].getParameter()).intValue();
			else if (name.equals("AD_Org_ID"))
				p_AD_Org_ID = ((BigDecimal)para[i].getParameter()).intValue();
			else if (name.equals("M_Locator_ID"))
				p_M_Locator_ID = ((BigDecimal)para[i].getParameter()).intValue();
			else if (name.equals("M_LocatorTo_ID"))
				p_M_LocatorTo_ID = ((BigDecimal)para[i].getParameter()).intValue();
			else if (name.equals("MovementDate"))
				p_MovementDate = (Timestamp)para[i].getParameter();
			else if (name.equals("DeleteOldImported"))
				p_DeleteOldImported = "Y".equals(para[i].getParameter());
			else
				log.log(Level.SEVERE, "Unknown Parameter: " + name);
		}
	}	//	prepare


	/**
	 *  Perrform process.
	 *  @return Message
	 *  @throws Exception
	 */
	protected String doIt() throws java.lang.Exception
	{
		log.info("M_Locator_ID=" + p_M_Locator_ID +"M_LocatorTo_ID=" + p_M_LocatorTo_ID + "MovementDate=" + p_MovementDate);
		//
		StringBuffer sql = null;
		int no = 0;
		String clientCheck = " AND AD_Client_ID=" + p_AD_Client_ID;

		//	****	Prepare	****

		//	Delete Old Imported
		if (p_DeleteOldImported)
		{
			sql = new StringBuffer ("DELETE I_Movement "
				  + "WHERE I_IsImported='Y'").append (clientCheck);
			no = DB.executeUpdate (sql.toString (), get_TrxName());
			log.fine("Delete Old Impored =" + no);
		}

		//	Set Client, Org, IsActive, Created/Updated
		sql = new StringBuffer ("UPDATE I_Movement "
			  + "SET AD_Client_ID = COALESCE (AD_Client_ID,").append (p_AD_Client_ID).append ("),"
			  + " AD_Org_ID = COALESCE (AD_Org_ID,").append (p_AD_Org_ID).append ("),");
		if (p_MovementDate != null)
			sql.append(" MovementDate = COALESCE (MovementDate,").append (DB.TO_DATE(p_MovementDate)).append ("),");
		sql.append(" IsActive = COALESCE (IsActive, 'Y'),"
			  + " Created = COALESCE (Created, SysDate),"
			  + " CreatedBy = COALESCE (CreatedBy, 0),"
			  + " Updated = COALESCE (Updated, SysDate),"
			  + " UpdatedBy = COALESCE (UpdatedBy, 0),"
			  + " I_ErrorMsg = NULL,"
			  + " I_IsImported = 'N' "
			  + "WHERE I_IsImported<>'Y' OR I_IsImported IS NULL");
		no = DB.executeUpdate (sql.toString (), get_TrxName());
		log.info ("Reset=" + no);

		sql = new StringBuffer ("UPDATE I_Movement o "
			+ "SET I_IsImported='E', I_ErrorMsg=I_ErrorMsg||'ERR=Invalid Org, '"
			+ "WHERE (AD_Org_ID IS NULL OR AD_Org_ID=0"
			+ " OR EXISTS (SELECT * FROM AD_Org oo WHERE o.AD_Org_ID=oo.AD_Org_ID AND (oo.IsSummary='Y' OR oo.IsActive='N')))"
			+ " AND I_IsImported<>'Y'").append (clientCheck);
		no = DB.executeUpdate (sql.toString (), get_TrxName());
		if (no != 0)
			log.warning ("Invalid Org=" + no);


		//	Locator
		sql = new StringBuffer ("UPDATE I_Movement i "
			+ "SET M_Locator_ID=(SELECT M_Locator_ID FROM M_Locator l"
			+ " WHERE i.LocatorValue=l.Value AND i.AD_Client_ID=l.AD_Client_ID AND ROWNUM=1) "
			+ "WHERE M_Locator_ID IS NULL AND LocatorValue IS NOT NULL"
			+ " AND I_IsImported<>'Y'").append (clientCheck);
		no = DB.executeUpdate (sql.toString (), get_TrxName());
		log.fine("Set Locator from Value =" + no);

        // Locator is null and parameter Locator is null
		if ( p_M_Locator_ID==0)
		{	
		sql = new StringBuffer ("UPDATE I_Movement "
				+ "SET I_IsImported='E', I_ErrorMsg=I_ErrorMsg||'ERR=No Locator, ' "
				+ "WHERE M_Locator_ID IS NULL"
				+ " AND I_IsImported<>'Y'").append (clientCheck);
			no = DB.executeUpdate (sql.toString (), get_TrxName());
			if (no != 0)
				log.warning ("No Locator=" + no);
		 } 
		 else
		 { 
		 sql = new StringBuffer ("UPDATE I_Movement "
				+ " SET M_Locator_ID=" + p_M_Locator_ID
				+ " WHERE M_Locator_ID IS NULL"
				+ " AND I_IsImported<>'Y'").append (clientCheck);
			no = DB.executeUpdate (sql.toString (), get_TrxName());
			log.fine("Set Locator from Value =" + no);
		 }
		
		//	LocatorTo
		sql = new StringBuffer ("UPDATE I_Movement i "
			+ "SET M_LocatorTo_ID=(SELECT M_Locator_ID FROM M_Locator l"
			+ " WHERE i.LocatorToValue=l.Value AND i.AD_Client_ID=l.AD_Client_ID AND ROWNUM=1) "
			+ "WHERE M_LocatorTo_ID IS NULL AND LocatorToValue IS NOT NULL"
			+ " AND I_IsImported<>'Y'").append (clientCheck);
		no = DB.executeUpdate (sql.toString (), get_TrxName());
		log.fine("Set LocatorTo from Value =" + no);

        // LocatorTo is zero and parameter Locator is zero
		if ( p_M_LocatorTo_ID == 0)
		 {	
		sql = new StringBuffer ("UPDATE I_Movement "
			+ "SET I_IsImported='E', I_ErrorMsg=I_ErrorMsg||'ERR=No LocatorTo, ' "
			+ "WHERE M_LocatorTo_ID IS NULL"
			+ " AND I_IsImported<>'Y'").append (clientCheck);
			no = DB.executeUpdate (sql.toString (), get_TrxName());
			if (no != 0)
				log.warning ("No LocatorTo=" + no);
    	 }
		 else
		 { 
		 sql = new StringBuffer ("UPDATE I_Movement "
				+ " SET M_LocatorTo_ID=" + p_M_LocatorTo_ID
				+ " WHERE M_LocatorTo_ID IS NULL"
				+ " AND I_IsImported<>'Y'").append (clientCheck);
			no = DB.executeUpdate (sql.toString (), get_TrxName());
			log.fine("Set LocatorTo from Value =" + no);
		 }



		//	Product
		sql = new StringBuffer ("UPDATE I_Movement i "
			  + "SET M_Product_ID=(SELECT M_Product_ID FROM M_Product p"
			  + " WHERE i.Value=p.Value AND i.AD_Client_ID=p.AD_Client_ID AND ROWNUM=1) "
			  + "WHERE M_Product_ID IS NULL AND Value IS NOT NULL"
			  + " AND I_IsImported<>'Y'").append (clientCheck);
		no = DB.executeUpdate (sql.toString (), get_TrxName());
		log.fine("Set Product from Value=" + no);
		sql = new StringBuffer ("UPDATE I_Movement "
			+ "SET I_IsImported='E', I_ErrorMsg=I_ErrorMsg||'ERR=No Product, ' "
			+ "WHERE M_Product_ID IS NULL"
			+ " AND I_IsImported<>'Y'").append (clientCheck);
		no = DB.executeUpdate (sql.toString (), get_TrxName());
		if (no != 0)
			log.warning ("No Product=" + no);

		//	No MovementQty
		sql = new StringBuffer ("UPDATE I_Movement "
			+ "SET I_IsImported='E', I_ErrorMsg=I_ErrorMsg||'ERR=No MOVEMENTQTY, ' "
			+ "WHERE MOVEMENTQTY =0 "
			+ " AND I_IsImported<>'Y'").append (clientCheck);
		no = DB.executeUpdate (sql.toString (), get_TrxName());
		if (no != 0)
			log.warning ("No MOVEMENTQTY=" + no);

		commit();
		
		/*********************************************************************/

		MMovement movement = null;

		int noInsert = 0;
		int noInsertLine = 0;

		//	Go through Inventory Records
		sql = new StringBuffer ("SELECT * FROM I_Movement "
			+ "WHERE I_IsImported='N'").append (clientCheck)
			.append(" ORDER BY TRUNC(MovementDate), I_Movement_ID");
		try
		{
			PreparedStatement pstmt = DB.prepareStatement (sql.toString (), get_TrxName());
			ResultSet rs = pstmt.executeQuery ();
			//
			
			Timestamp x_MovementDate = null;
			int lineNo = 1;
			while (rs.next())
			{
				X_I_Movement imp = new X_I_Movement (getCtx (), rs, get_TrxName());
				Timestamp MovementDate = TimeUtil.getDay(imp.getMovementDate());

				if (movement == null
					|| !MovementDate.equals(x_MovementDate))
				{
					movement = new MMovement (getCtx(), 0, get_TrxName());
				//	movement.setClientOrg(imp.getAD_Client_ID(), imp.getAD_Org_ID());
					movement.setDescription("Import " + " " + MovementDate);
					movement.setMovementDate(MovementDate);
					//
					if (!movement.save())
					{
						log.log(Level.SEVERE, "Movement not saved");
						break;
					}
					x_MovementDate = MovementDate;
					noInsert++;
				}

				//	Line
				int M_AttributeSetInstance_ID = 0;
				int M_AttributeSetInstanceTo_ID = 0;
				MMovementLine line = new MMovementLine ( Env.getCtx(), 0, get_TrxName() );
//				line.setM_MovementLine_ID( 1 );
				line.setM_Movement_ID( movement.getM_Movement_ID() );
				line.setAD_Org_ID( imp.getAD_Org_ID() );
				line.setM_Locator_ID( imp.getM_Locator_ID() );
				line.setM_LocatorTo_ID( imp.getM_LocatorTo_ID() );
				line.setM_Product_ID( imp.getM_Product_ID() );
				line.setM_AttributeSetInstance_ID( M_AttributeSetInstance_ID );
				line.setM_AttributeSetInstanceTo_ID( M_AttributeSetInstanceTo_ID );
				line.setMovementQty( imp.getMovementQty() );
				line.setLine(lineNo);
				lineNo += 1;
				if (line.save())
				{
					imp.setI_IsImported(true);
					imp.setM_Movement_ID(line.getM_Movement_ID());
					imp.setM_MovementLine_ID(line.getM_MovementLine_ID());
					imp.setProcessed(true);
					if (imp.save())
						noInsertLine++;
				}
			}
			rs.close();
			pstmt.close();
		}
		catch (Exception e)
		{
			log.log(Level.SEVERE, sql.toString(), e);
		}

		//	Set Error to indicator to not imported
		sql = new StringBuffer ("UPDATE I_Movement "
			+ "SET I_IsImported='N', Updated=SysDate "
			+ "WHERE I_IsImported<>'Y'").append(clientCheck);
		no = DB.executeUpdate(sql.toString(), get_TrxName());
		addLog (0, null, new BigDecimal (no), "@Errors@");
		//
		addLog (0, null, new BigDecimal (noInsert), "@M_Movement_ID@: @Inserted@");
		addLog (0, null, new BigDecimal (noInsertLine), "@M_MovementLine_ID@: @Inserted@");
		return "";
	}	//	doIt

}	//	ImportMovement
