package org.compiere.apps;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.event.WindowEvent;
import java.util.Properties;
import java.util.logging.Level;

import org.compiere.apps.form.FormFrame;
import org.compiere.apps.form.FormPanel;
import org.compiere.db.CConnection;
import org.compiere.model.MQuery;
import org.compiere.model.MRole;
import org.compiere.model.MWorkbench;
import org.compiere.swing.CPanel;
import org.compiere.util.CLogger;
import org.compiere.util.Env;

/**
 * Curtain Order Form
 *
 * @author  Jirimuto
 * @version $Id: AFormWindow.java,v 1.1 2006/09/25 14:19:27 hans Exp $
 */
public class AFormWindow extends CPanel
	implements FormPanel 
{

	/**
	 *	Initialize Panel
	 *  @param WindowNo window
	 *  @param frame frame
	 */
	public void init (int WindowNo, FormFrame frame)
	{
		m_ctx = Env.getCtx();
		m_curWindowNo = WindowNo;
		m_frame = frame;
		Env.setContext(m_ctx, m_curWindowNo, "IsSOTrx", "Y");   //  defaults to no
		m_C_Currency_ID = Env.getContextAsInt(m_ctx, "$C_Currency_ID");   //  default

		//
		log.info("Currency=" + m_C_Currency_ID);
		try
		{
			m_frame.setIconImage(org.compiere.Compiere.getImage16());
			m_frame.getContentPane().add(m_APanel, BorderLayout.CENTER);
			m_frame.setGlassPane(m_glassPane);

		}
		catch(Exception e)
		{
			log.log(Level.SEVERE, "", e);
		}
		
		setPreferredSize( new Dimension(980, 600) );
		setSize( new Dimension(980, 600) );
		
	}	//	init

	private String trxName = null;

	private Properties      m_ctx;

	/** Current Window No                               */
	private int				m_curWindowNo;
	
	private int         m_C_Currency_ID = 0;

	/** Workbench Model                                 */
	private MWorkbench		m_mWorkbench;
	
	/** Dispose active                                  */
	private boolean         m_disposing = false;
	/** Process Info                                    */
	private boolean         m_isLocked = false;
	/** Show Personal Lock								*/
	private boolean 		m_isPersonalLock = MRole.getDefault().isPersonalLock();
	/**	Last Modifier of Action Event					*/
	private int 			m_lastModifiers;
	
	/**	FormFrame			*/
	private FormFrame 	m_frame;

	/** Application Window  	*/
	private ACPanel			m_APanel = new ACPanel();
	
	/**	Logger			*/
	private static CLogger log = CLogger.getCLogger(AFormWindow.class);

	private AGlassPane  m_glassPane = new AGlassPane();	
	
	/**
	 *	Dispose
	 */
	public void dispose()
	{
	//	log.config("");
		//  ignore changes
		m_disposing = true;
		//  close panels
		
		//  Get rid of remaining model
		if (m_mWorkbench != null)
			m_mWorkbench.dispose();
		m_mWorkbench = null;
		
		if (m_APanel != null)
			m_APanel.dispose();
		m_APanel = null;
		this.removeAll();
		m_frame.removeAll();
		m_frame.dispose();
		
	}	//	dispose

	/**
	 *	Dynamic Initialization Single Window
	 *  @param AD_Window_ID window
	 *  @param query query
	 *  @return true if loaded OK
	 */
	public boolean initWindow (int AD_Window_ID, MQuery query)
	{
		m_frame.setName("AWindow_" + AD_Window_ID);
		m_frame.setAD_Window_ID(AD_Window_ID);
		//
		boolean loadedOK = m_APanel.initPanel (0, AD_Window_ID, query);
		commonInit();
		return loadedOK;
	}	//	initWindow

	/**
	 *  Common Init.
	 * 	After APanel loaded
	 */
	private void commonInit()
	{
		m_frame.setJMenuBar(m_APanel.getMenuBar());
		m_frame.setTitle(m_APanel.getTitle());
		//
		Image image = m_APanel.getImage();
		if (image != null)
			m_frame.setIconImage(image);
	}   //  commonInit

	
	/*************************************************************************
	 *  Set Window Busy
	 *  @param busy busy
	 */
	public void setBusy (boolean busy)
	{
		if (busy == m_glassPane.isVisible()
			|| CConnection.get().isTerminalServer())
			return;
		log.config(getName() + " - " + busy);
		m_glassPane.setMessage(null);
		m_glassPane.setVisible(busy);
		if (busy)
			m_glassPane.requestFocus();
	}   //  setBusy

	/**
	 *  Set Busy Message
	 *  @param AD_Message message
	 */
	public void setBusyMessage (String AD_Message)
	{
		m_glassPane.setMessage(AD_Message);
	}   //  setBusyMessage

	/**
	 *  Set and start Busy Counter
	 *  @param time in seconds
	 */
	public void setBusyTimer (int time)
	{
		m_glassPane.setBusyTimer (time);
	}   //  setBusyTimer

	/**
	 *  Window Events
	 *  @param e event
	 */
	protected void processWindowEvent(WindowEvent e)
	{
		// m_frame.processWindowEvent(e);
//		System.out.println(">> Apps WE_" + e.getID()    // + " Frames=" + getFrames().length
//			+ " " + e);
	}   //  processWindowEvent

	/**
	 * 	Get Application Panel
	 *	@return application panel
	 */
	protected ACPanel getAPanel()
	{
		return m_APanel;
	}	//	getAPanel
	
	/**
	 * 	Get Window No of Panel
	 *	@return window no
	 */
	public int getWindowNo()
	{
		if (m_APanel != null)
			return m_APanel.getWindowNo();
		return 0;
	}	//	getWindowNo
	
	/**
	 *  String Representation
	 *  @return Name
	 */
	public  String toString()
	{
		return getName() + "_" + getWindowNo();
	}   //  toString

	
}   //  VFormWindow
