/******************************************************************************
 * The contents of this file are subject to the   Compiere License  Version 1.1
 * ("License"); You may not use this file except in compliance with the License
 * You may obtain a copy of the License at http://www.compiere.org/license.html
 * Software distributed under the License is distributed on an  "AS IS"  basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 * The Original Code is Compiere ERP & CRM Smart Business Solution. The Initial
 * Developer of the Original Code is Jorg Janke. Portions created by Jorg Janke
 * are Copyright (C) 1999-2005 Jorg Janke.
 * All parts are Copyright (C) 1999-2005 ComPiere, Inc.  All Rights Reserved.
 * Contributor(s): ______________________________________.
 *****************************************************************************/
package org.compiere.apps.form;

import java.awt.*;
import java.awt.event.*;
import java.math.*;
import java.sql.*;
import java.text.*;
import java.util.*;
import java.util.logging.*;
import javax.swing.*;
import javax.swing.event.*;
import org.compiere.*;
import org.compiere.apps.*;
import org.compiere.grid.ed.*;
import org.compiere.model.*;
import org.compiere.swing.*;
import org.compiere.util.*;


/**
 *	Product Attribute Table.
 *	Select one or two attributes for view/etc.
 *	
 *  @author Jorg Janke
 *  @version $Id: VAttributeGrid.java,v 1.1 2006/09/25 14:19:51 hans Exp $
 */
public class VAttributeGrid extends CPanel
	implements FormPanel, ChangeListener, ActionListener
{
	/**
	 * 	Init
	 *	@param WindowNo
	 *	@param frame
	 */
	public void init (int WindowNo, FormFrame frame)
	{
		m_WindowNo = WindowNo;
		m_frame = frame;
		//	Layout
		frame.getContentPane().add(this, BorderLayout.CENTER);
		this.setLayout(new BorderLayout());
		this.add(tabbedPane, BorderLayout.CENTER);
		this.add(confirmPanel, BorderLayout.SOUTH);
		tabbedPane.addChangeListener(this);
		confirmPanel.addActionListener(this);
		//
		tabbedPane.add(selectPanel, "Select");
		selectPanel.add(attributeLabel1, new ALayoutConstraint(0,0));
		m_attributes = MAttribute.getOfClient(Env.getCtx(), true, true);
		Vector<KeyNamePair> vector = new Vector<KeyNamePair>();
		vector.add(new KeyNamePair(0,""));
		for (int i = 0; i < m_attributes.length; i++)
			vector.add(m_attributes[i].getKeyNamePair());
		attributeCombo1 = new CComboBox(vector);
		selectPanel.add(attributeCombo1, null);
		selectPanel.add(attributeLabel2, new ALayoutConstraint(1,0));
		attributeCombo2 = new CComboBox(vector);
		selectPanel.add(attributeCombo2, null);
		//
		fillPicks();
		selectPanel.add(labelPriceList, new ALayoutConstraint(2,0));
		selectPanel.add(pickPriceList);
		selectPanel.add(labelWarehouse, new ALayoutConstraint(3,0));
		selectPanel.add(pickWarehouse);
		//
		//	Grid
		tabbedPane.add(gridPanel, "Grid");
		modePanel.add(modeLabel);
		modePanel.add(modeCombo);
	}	//	init

	/**	Window No			*/
	private int         	m_WindowNo = 0;
	/**	FormFrame			*/
	private FormFrame 		m_frame;
	/** Product Attributes	*/
	private MAttribute[]	m_attributes = null;
	/** Setting Grid		*/
	private boolean			m_setting = false;
	/**	Logger	*/
	private static CLogger log = CLogger.getCLogger (VAttributeGrid.class);
	
	private static String[]	MODES = new String[]{
		"View"
	};
	private int				m_M_PriceList_Version_ID = 0;
	private DecimalFormat	m_price = DisplayType.getNumberFormat(DisplayType.CostPrice);
	private int				m_M_Warehouse_ID = 0;
	private DecimalFormat	m_qty = DisplayType.getNumberFormat(DisplayType.Quantity);
	
	/** UI			**/
	private CTabbedPane tabbedPane = new CTabbedPane();
	private CPanel		selectPanel = new CPanel(new ALayout());
	private CLabel		attributeLabel1 = new CLabel(Msg.getElement(Env.getCtx(), "M_Attribute_ID") + " 1");
	private CComboBox	attributeCombo1 = null;
	private CLabel		attributeLabel2 = new CLabel(Msg.getElement(Env.getCtx(), "M_Attribute_ID") + " 2");
	private CComboBox	attributeCombo2 = null;
	private CLabel 		labelPriceList = new CLabel(Msg.getElement(Env.getCtx(), "M_PriceList_ID"));
	private VComboBox 	pickPriceList = new VComboBox();
	private CLabel 		labelWarehouse = new CLabel(Msg.getElement(Env.getCtx(), "M_Warehouse_ID"));
	private VComboBox 	pickWarehouse = new VComboBox();
	private ConfirmPanel confirmPanel = new ConfirmPanel(true);
	//
	private CPanel		gridPanel = new CPanel(new BorderLayout());
	private CPanel		modePanel = new CPanel();
	private CLabel		modeLabel = new CLabel("Mode");
	private CComboBox	modeCombo = new CComboBox(MODES);
	
	/**
	 * 	Dispose
	 *	@see org.compiere.apps.form.FormPanel#dispose()
	 */
	public void dispose ()
	{
		if (m_frame != null)
			m_frame.dispose();
		m_frame = null;
	}	//	dispose

	/**
	 *	Fill Picks with values
	 */
	private void fillPicks ()
	{
		//	Price List
		String sql = "SELECT M_PriceList_Version.M_PriceList_Version_ID,"
			+ " M_PriceList_Version.Name || ' (' || c.Iso_Code || ')' AS ValueName "
			+ "FROM M_PriceList_Version, M_PriceList pl, C_Currency c "
			+ "WHERE M_PriceList_Version.M_PriceList_ID=pl.M_PriceList_ID"
			+ " AND pl.C_Currency_ID=c.C_Currency_ID"
			+ " AND M_PriceList_Version.IsActive='Y' AND pl.IsActive='Y'";
		//	Add Access & Order
	//	sql = MRole.getDefault().addAccessSQL (sql, "M_PriceList_Version", true, false)	// fully qualidfied - RO 
	//		+ " ORDER BY M_PriceList_Version.Name";
		try
		{
			pickPriceList.addItem(new KeyNamePair (0, ""));
			PreparedStatement pstmt = DB.prepareStatement(sql, null);
			ResultSet rs = pstmt.executeQuery();
			while (rs.next())
			{
				KeyNamePair kn = new KeyNamePair (rs.getInt(1), rs.getString(2));
				pickPriceList.addItem(kn);
			}
			rs.close();
			pstmt.close();

			//	Warehouse
			sql = "SELECT M_Warehouse_ID, Value || ' - ' || Name AS ValueName "
				+ "FROM M_Warehouse "
				+ "WHERE IsActive='Y'";
		//	sql = MRole.getDefault().addAccessSQL (sql,
		//			"M_Warehouse", MRole.SQL_NOTQUALIFIED, MRole.SQL_RO)
		//		+ " ORDER BY Value";
			pickWarehouse.addItem(new KeyNamePair (0, ""));
			pstmt = DB.prepareStatement(sql, null);
			rs = pstmt.executeQuery();
			while (rs.next())
			{
				KeyNamePair kn = new KeyNamePair
					(rs.getInt("M_Warehouse_ID"), rs.getString("ValueName"));
				pickWarehouse.addItem(kn);
			}
			rs.close();
			pstmt.close();
		}
		catch (SQLException e)
		{
			log.log(Level.SEVERE, sql, e);
		}
	}	//	fillPicks

	
	/**
	 * 	Change Listener
	 *	@param e event
	 */
	public void stateChanged (ChangeEvent e)
	{
		if (e.getSource() != tabbedPane)
			return;
		if (tabbedPane.getSelectedIndex() == 1)
			createGrid();
	}	//	stateChanged

	/**
	 * 	Action Performed
	 *	@param e event
	 */
	public void actionPerformed (ActionEvent e)
	{
	//	log.fine(e.toString());
		if (e.getActionCommand().equals(ConfirmPanel.A_OK))
		{
			if (tabbedPane.getSelectedIndex() == 0)
				createGrid();
		}
		else if (e.getActionCommand().equals(ConfirmPanel.A_CANCEL))
			m_frame.dispose();
	}	//	actionPerformed

	/**
	 * 	Create Grid
	 */
	private void createGrid()
	{
		if (attributeCombo1 == null || m_setting)
			return;		//	init
		int indexAttr1 = attributeCombo1.getSelectedIndex();
		int indexAttr2 = attributeCombo2.getSelectedIndex();
		if (indexAttr1 == indexAttr2)
		{
			log.warning("Same Attribute Selected");
			tabbedPane.setSelectedIndex(0);
			return;
		}
		m_setting = true;
		m_M_PriceList_Version_ID = 0;
		KeyNamePair pl = (KeyNamePair)pickPriceList.getSelectedItem();
		if (pl != null)
			m_M_PriceList_Version_ID = pl.getKey();
		m_M_PriceList_Version_ID = 0;
		KeyNamePair wh = (KeyNamePair)pickWarehouse.getSelectedItem();
		if (wh != null)
			m_M_Warehouse_ID = pl.getKey();
		
		//	x dimension
		int cols = 2;
		MAttributeValue[] xValues = null;
		if (indexAttr1 > 0)
			xValues = m_attributes[indexAttr1-1].getMAttributeValues();
		if (xValues != null)
		{
			cols = xValues.length;
			log.info("X - " + m_attributes[indexAttr1-1].getName() + " #" + xValues.length);
		}
		
		//	y dimension
		int rows = 2;
		MAttributeValue[] yValues = null;
		if (indexAttr2 > 0)
			yValues = m_attributes[indexAttr2-1].getMAttributeValues();
		if (yValues != null)
		{
			rows = yValues.length;
			log.info("Y - " + m_attributes[indexAttr2-1].getName() + " #" + yValues.length);
		}
		
		//
		gridPanel.removeAll();
		CPanel grid = new CPanel(new GridLayout(rows, cols, 5,5));
		gridPanel.add(modePanel, BorderLayout.NORTH);
		gridPanel.add(new CScrollPane(grid), BorderLayout.CENTER);
		//
		log.info("Rows=" + rows + " - Cols=" + cols);
		for (int row = 0; row < rows; row++)
		{
			for (int col = 0; col < cols; col++)
			{
				MAttributeValue xValue = null;
				if (xValues != null)
					xValue = xValues[col];
				MAttributeValue yValue = null;
				if (yValues != null)
					yValue = yValues[row];
			//	log.fine("Row=" + row + " - Col=" + col);
				//
				if (row == 0 && col == 0)
				{
					CPanel descr = new CPanel(new GridLayout(2,1,0,0));
					if (xValues != null)
						descr.add(new JLabel(m_attributes[indexAttr1-1].getName(),JLabel.TRAILING));
					if (yValues != null)
						descr.add(new JLabel(m_attributes[indexAttr2-1].getName()));
					grid.add(descr);
				}
				else if (row == 0)	//	column labels
				{
					if (xValue != null)
					{
						grid.add(new JLabel(xValue.getName(), JLabel.TRAILING));
					}
					else
						grid.add(new JLabel());
				}
				else if (col == 0)	//	row labels
				{
					if (yValue != null)
						grid.add(new JLabel(yValue.getName()));
					else
						grid.add(new JLabel());
				}
				else
				{
					grid.add(getGridElement (xValue, yValue));
				}
			}
		}
		//
		tabbedPane.setSelectedIndex(1);
		m_setting = false;
		m_frame.pack();
	}	//	createGrid
	
	/**
	 * 	Get Grid Element
	 *	@param xValue X value
	 *	@param yValue Y value
	 *	@return Panel with Info
	 */
	private CPanel getGridElement (MAttributeValue xValue, MAttributeValue yValue)
	{
		CPanel element = new CPanel();
		element.setBorder(BorderFactory.createLineBorder(Color.BLACK, 2));
		element.setLayout(new BoxLayout(element, BoxLayout.Y_AXIS));
		
		String sql = "SELECT * FROM M_Product WHERE IsActive='Y'";
		//	Product Attributes
		if (xValue != null)
			sql += " AND M_AttributeSetInstance_ID IN "
				+ "(SELECT M_AttributeSetInstance_ID "
				+ "FROM M_AttributeInstance "
				+ "WHERE M_Attribute_ID=" + xValue.getM_Attribute_ID()
				+ " AND M_AttributeValue_ID=" + xValue.getM_AttributeValue_ID() + ")"; 
		if (yValue != null)
			sql += " AND M_AttributeSetInstance_ID IN "
				+ "(SELECT M_AttributeSetInstance_ID "
				+ "FROM M_AttributeInstance "
				+ "WHERE M_Attribute_ID=" + yValue.getM_Attribute_ID()
				+ " AND M_AttributeValue_ID=" + yValue.getM_AttributeValue_ID() + ")"; 
	//	sql = MRole.getDefault().addAccessSQL(sql, "M_Product", 
	//		MRole.SQL_NOTQUALIFIED, MRole.SQL_RO);
		PreparedStatement pstmt = null;
		try
		{
			pstmt = DB.prepareStatement (sql, null);
			ResultSet rs = pstmt.executeQuery ();
			while (rs.next ())
			{
				MProduct product = new MProduct(Env.getCtx(), rs, null);
				addProduct (element, product);
			}
			rs.close ();
			pstmt.close ();
			pstmt = null;
		}
		catch (Exception e)
		{
			log.log (Level.SEVERE, sql, e);
		}
		try
		{
			if (pstmt != null)
				pstmt.close ();
			pstmt = null;
		}
		catch (Exception e)
		{
			pstmt = null;
		}
		
		return element;
	}	//	getGridElement
	
	/**
	 * 	Add Product
	 *	@param element panel
	 *	@param product product
	 */
	private void addProduct(CPanel element, MProduct product)
	{
		int M_Product_ID = product.getM_Product_ID();
		CPanel pe = new CPanel();
		pe.setBorder(BorderFactory.createLineBorder(Color.BLUE, 1));
		pe.setLayout(new GridBagLayout());
		//	Product Value - Price
		pe.add(new JLabel(product.getValue()), new GridBagConstraints(0,0, 1,1, 0,0, 
				GridBagConstraints.WEST, GridBagConstraints.BOTH, new Insets(0,0,0,0), 0,0));
		if (m_M_PriceList_Version_ID != 0)
		{
			MProductPrice pp = MProductPrice.get(Env.getCtx(), m_M_PriceList_Version_ID, M_Product_ID, null);
			if (pp != null)
			{
				BigDecimal price = pp.getPriceStd();
				String form = m_price.format(price);
				pe.add(new JLabel(form), new GridBagConstraints(1,0, 1,1, 0,0, 
					GridBagConstraints.EAST, GridBagConstraints.BOTH, new Insets(0,0,0,0), 0,0));
			}
		}
		//	Product Name - Qty
		pe.add(new JLabel(product.getName()), new GridBagConstraints(0,1, 1,1, 0,0, 
			GridBagConstraints.WEST, GridBagConstraints.BOTH, new Insets(0,0,0,0), 0,0));
		if (m_M_Warehouse_ID != 0)
		{
			BigDecimal qty = MStorage.getQtyAvailable(m_M_Warehouse_ID, M_Product_ID, 0, null);
			if (qty != null)
			{
				String form = m_qty.format(qty);
				pe.add(new JLabel(form), new GridBagConstraints(1,1, 1,1, 0,0, 
					GridBagConstraints.EAST, GridBagConstraints.BOTH, new Insets(0,0,0,0), 0,0));
			}
		}
		//
		element.add(pe);
	}	//	addProduct
	
	
	/*************************************************************************
	 * 	Test 
	 *	@param args ignored
	 */
	public static void main (String[] args)
	{
		Compiere.startup(true);
		Env.setContext(Env.getCtx(), "#AD_Client_ID", 11);
		FormFrame ff = new FormFrame();
		ff.openForm(1111, "org.compiere.apps.form.VAttributeGrid", "Attribute Table");
		ff.pack();
		AEnv.showCenterScreen(ff);
	}	//	main

	
}	//	VAttributeTable
