package org.compiere.extend.pt;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.math.BigDecimal;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.logging.Level;

import org.compiere.impexp.BankStatementLoaderInterface;
import org.compiere.model.MBankStatementLoader;
import org.compiere.model.MPayment;
import org.compiere.model.X_I_BankStatement;
import org.compiere.util.CLogger;
import org.compiere.util.DB;
import org.compiere.util.Env;
import org.compiere.util.Msg;

public class DSKBankStatementLoader implements BankStatementLoaderInterface{

    private final String STATEMENT_HEADER_MARK = "1";
    private final String STATEMENT_DATA_MARK = "2";
    private final String STATEMENT_TRAIL_MARK = "8";
    private final String STATEMENT_END_MARK = "9";

	private final int STATEMENT_DATE_START = 1;
    private final int STATEMENT_DATE_END = 9;
	private final int STATEMENT_BANKACCOUNT_START = 9;
    private final int STATEMENT_BANKACCOUNT_END = 14;
	
	private final int STATEMENT_REFERENCE_START = 1;
    private final int STATEMENT_REFERENCE_END = 3;
	private final int STATEMENT_ACCOUNTDATE_START = 3;
    private final int STATEMENT_ACCOUNTDATE_END = 15;
	private final int STATEMENT_BPARTNER_START = 29;
    private final int STATEMENT_BPARTNER_END = 35;
	private final int STATEMENT_INVOICENO_START = 41;
    private final int STATEMENT_INVOICENO_END = 48;
	private final int STATEMENT_AMOUNT_START = 55;
    private final int STATEMENT_AMOUNT_END = 61;
	private final int STATEMENT_VALUTADATE_START = 73;
    private final int STATEMENT_VALUTADATE_END = 81;
	private final int STATEMENT_LINEDATE_START = 81;
    private final int STATEMENT_LINEDATE_END = 89;
    
	private final int STATEMENT_SOKUHOCOUNT_START = 1;
    private final int STATEMENT_SOKUHOCOUNT_END = 7;
	private final int STATEMENT_SOKUHOAMOUNT_START = 7;
    private final int STATEMENT_SOKUHOAMOUNT_END = 17;
	private final int STATEMENT_KAKUHOCOUNT_START = 17;
    private final int STATEMENT_KAKUHOCOUNT_END = 23;
	private final int STATEMENT_KAKUHOAMOUNT_START = 23;
    private final int STATEMENT_KAKUHOAMOUNT_END = 33;
	private final int STATEMENT_CANCELCOUNT_START = 33;
    private final int STATEMENT_CANCELCOUNT_END = 39;
	private final int STATEMENT_CANCELAMOUNT_START = 39;
    private final int STATEMENT_CANCELAMOUNT_END = 49;
	private final int STATEMENT_YUBINCOUNT_START = 49;
    private final int STATEMENT_YUBINCOUNT_END = 55;
	private final int STATEMENT_YUBINAMOUNT_START = 55;
    private final int STATEMENT_YUBINAMOUNT_END = 65;
    
    /**	Logger							*/
	protected CLogger			log = CLogger.getCLogger (getClass());
	
	private MBankStatementLoader m_controller = null;
	
	protected String m_errorMessage = "";
	protected String m_errorDescription = "";
	protected BufferedReader m_reader = null;
	private Timestamp dateLastRun;

	private String routingNo = null;
	private String bankAccountNo = null;
	private Timestamp statementDate = null;

	private String trxName = null;
	
	private String statementReference = null;
	private String trxID = null;
	private String reference = null;
	private String checkNo = null;
	private String payeeName = null;
	private String payeeAccountNo = null;
	private Timestamp statementLineDate = null;
	private Timestamp valutaDate = null;
	private Timestamp accountDate = null;
	private String trxType = null; 
	private boolean isReversal = false; 
	private String currency = null;
	private BigDecimal stmtAmt = null; 
	private BigDecimal trxAmt = null; 
	private BigDecimal interestAmt = null; 
	private String memo = null; 
	private String chargeName = null;
	private BigDecimal chargeAmt = null;	
	private String bPartnerValue = null; 
	private String invoiceDocumentNo = null; 
	private BigDecimal lineNo = null; 
	private boolean deleteBeforeImport = true;

	public boolean init(MBankStatementLoader controller) {
		boolean result = false;
		if (controller == null)
		{
			m_errorMessage = "ErrorInitializingParser";
			m_errorDescription = "ImportController is a null reference";
			return result;
		}
		this.m_controller = controller;
		try
		{
			File file = new File( this.m_controller.getLocalFileName());
			InputStream is = new FileInputStream( file ); 
			this.m_reader = new BufferedReader(new InputStreamReader(is));
			result = true;
		}
		catch(IOException e)
		{
			m_errorMessage = "ErrorInitializingParser";
			m_errorDescription = "Unable to configure SAX parser: " + e.getMessage();
		}
		return result;
	}

	/**	Verify the validity of the OFX data
	 *	@return true if input is valid OFX data
	 */
	public boolean isValid()
	{
		boolean result = true;
		return result;
	}	//isValid

	/**
	 * Read statementlines from InputStream.
	 * @return load success
	 * This method will be invoked from ImportController.
	 */
	public boolean loadLines()
	{
		int sokuhoTotalCount = 0;
		int sokuhoTotalAmount = 0;
		int kakuhoTotalCount = 0;
		int kakuhoTotalAmount = 0;
		int cancelTotalCount = 0;
		int cancelTotalAmount = 0;
		int yubinTotalCount = 0;
		int yubinTotalAmount = 0;
		
		
		boolean result = false;
		try
		{
			int lineNumber = 0;
			String line = this.m_reader.readLine();
			while ( line != null ) {
				
				this.statementReference = null;
				this.trxID = null;
				this.reference = null;
				this.checkNo = null;
				this.payeeName = null;
				this.payeeAccountNo = null;
				this.statementLineDate = null;
				this.accountDate = null;
				this.valutaDate=null;
				this.trxType = null; 
				this.isReversal = false; 
				this.currency = null;
				this.stmtAmt = null; 
				this.trxAmt = null; 
				this.interestAmt = null; 
				this.memo = null; 
				this.chargeName = null;
				this.chargeAmt = null;	
				this.bPartnerValue = null; 
				this.invoiceDocumentNo = null;
				
				if( line.startsWith( STATEMENT_HEADER_MARK )) {
					
					sokuhoTotalCount = 0;
					sokuhoTotalAmount = 0;
					kakuhoTotalCount = 0;
					kakuhoTotalAmount = 0;
					cancelTotalCount = 0;
					cancelTotalAmount = 0;
					yubinTotalCount = 0;
					yubinTotalAmount = 0;
					
					if( line.length() >= STATEMENT_DATE_END ){
						String tmpDate = line.substring( STATEMENT_DATE_START, STATEMENT_DATE_END );
						SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
						try {
							this.statementDate = new Timestamp( sdf.parse( tmpDate ).getTime());
						} catch( Exception e ) {
							m_errorMessage = "DSK Header Line Error";
							m_errorDescription = "Statement Date is incorrect";
							return result;
						}
					}
					if( line.length() >= STATEMENT_BANKACCOUNT_END ){
						String tmpNo = line.substring( STATEMENT_BANKACCOUNT_START, STATEMENT_BANKACCOUNT_END );
						this.bankAccountNo = tmpNo;
					}
					
					lineNumber = 0;
					
				} else if( line.startsWith( STATEMENT_DATA_MARK ) ) {
					
					lineNumber++;
					this.lineNo = new BigDecimal( lineNumber );
					
					if( line.length() < STATEMENT_LINEDATE_END ){
						m_errorMessage = "DSK Data Line Error";
						m_errorDescription = "Data Line length is not enough";
						return result;							
					}
					
					if( line.length() >= STATEMENT_REFERENCE_END ){
						String tmpStr = line.substring( STATEMENT_REFERENCE_START, STATEMENT_REFERENCE_END );
						if( "01".equals( tmpStr ) ){
							sokuhoTotalCount++;
						} else if( "02".equals( tmpStr ) ){
							kakuhoTotalCount++;
						} else if( "03".equals( tmpStr )) {
							cancelTotalCount++;
						} else if( "12".equals( tmpStr) || "22".equals( tmpStr) ){
							yubinTotalCount++;
						} else {
							m_errorMessage = "DSK Data Line Error";
							m_errorDescription = "Data type is incorrect";
							return result;							
						}
						this.reference = tmpStr;
					}
					
					if( line.length() >= STATEMENT_ACCOUNTDATE_END ){
						String tmpDate = line.substring( STATEMENT_ACCOUNTDATE_START, STATEMENT_ACCOUNTDATE_END );
						SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmm");
						try {
							this.accountDate = new Timestamp( sdf.parse( tmpDate ).getTime());
						} catch( Exception e ) {
							m_errorMessage = "DSK Data Line Error";
							m_errorDescription = "Payment Date is incorrect";
							return result;							
						}
					}
					if( line.length() >= STATEMENT_BPARTNER_END ){
						String tmpStr = line.substring( STATEMENT_BPARTNER_START, STATEMENT_BPARTNER_END );
						this.bPartnerValue = removeLeadingZero(tmpStr);
					}
					if( line.length() >= STATEMENT_INVOICENO_END ){
						String tmpStr = line.substring( STATEMENT_INVOICENO_START, STATEMENT_INVOICENO_END );
						this.memo = tmpStr;
						if( tmpStr.startsWith( "00") ){
							String invoiceNo = lookupInvoiceNoFromOrder( tmpStr );
							if( invoiceNo == null ){
								log.warning("Invoice DocumentNo Lookup Failed, OrderNo = " + tmpStr);
							}
							this.invoiceDocumentNo = invoiceNo;
						} else {
							String invoiceNo = lookupInvoiceNoFromShipment( tmpStr );
							if( invoiceNo == null ){
								log.warning("Invoice DocumentNo Lookup Failed, ShipmentNo = " + tmpStr);
							}
							this.invoiceDocumentNo = invoiceNo;
						}
					}
					if( line.length() >= STATEMENT_AMOUNT_END ){
						String tmpStr = line.substring( STATEMENT_AMOUNT_START, STATEMENT_AMOUNT_END );
						int iValue = 0; 
						try {
							iValue = Integer.parseInt( tmpStr );
						} catch ( Exception e ) {
							m_errorMessage = "DSK Data Line Error";
							m_errorDescription = "Payment amount is incorrect";
							return result;							
						}						
						this.stmtAmt = new BigDecimal(iValue);
						this.trxAmt = new BigDecimal(iValue);
						
						if( "01".equals( this.reference ) ){
							sokuhoTotalAmount += iValue;
						} else if( "02".equals( this.reference ) ){
							kakuhoTotalAmount += iValue;
						} else if( "03".equals( this.reference )) {
							cancelTotalAmount += iValue;
						} else if( "12".equals( this.reference) || "22".equals( this.reference) ){
							yubinTotalAmount += iValue;
						} else {
							m_errorMessage = "DSK Data Line Error";
							m_errorDescription = "Data type is incorrect";
							return result;							
						}
						
					}
					if( line.length() >= STATEMENT_VALUTADATE_END ){
						Calendar calendar = Calendar.getInstance();
						calendar.setTime( this.getStatementDate() );
						calendar.add(Calendar.MONTH, 1);
						String tmpDate = line.substring( STATEMENT_VALUTADATE_START, STATEMENT_VALUTADATE_END );
						if( !"00000000".equals( tmpDate ) ){
							SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
							try {
								this.valutaDate = new Timestamp( sdf.parse( tmpDate ).getTime());
							} catch( Exception e ) {
								m_errorMessage = "DSK Data Line Error";
								m_errorDescription = "Transfer Scheduled Date is incorrect";
								return result;							
							}
						} else {
							this.valutaDate = new Timestamp( calendar.getTimeInMillis() );
						}
					}
					if( line.length() >= STATEMENT_LINEDATE_END ){
						Calendar calendar = Calendar.getInstance();
						String tmpDate = line.substring( STATEMENT_LINEDATE_START, STATEMENT_LINEDATE_END );
						SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
						try {
							this.statementLineDate = new Timestamp( sdf.parse( tmpDate ).getTime());
						} catch( Exception e ) {
							m_errorMessage = "DSK Data Line Error";
							m_errorDescription = "Statement Line Date is incorrect";
							return result;							
						}
						
					}

					this.m_controller.saveLine();
					
				} else if( line.startsWith( STATEMENT_TRAIL_MARK ) ) {
					
					if( line.length() >= STATEMENT_SOKUHOCOUNT_END ){
						String tmpStr = line.substring( STATEMENT_SOKUHOCOUNT_START, STATEMENT_SOKUHOCOUNT_END );
						int iValue = 0; 
						try {
							iValue = Integer.parseInt( tmpStr );
						} catch ( Exception e ) {
							iValue = 0;
						}
						if( sokuhoTotalCount != iValue ){
							m_errorMessage = "DSK Trail Data Error";
							m_errorDescription = "Sokuho Line Total Count is incorrect";
							return result;
						}
					}
					if( line.length() >= STATEMENT_SOKUHOAMOUNT_END ){
						String tmpStr = line.substring( STATEMENT_SOKUHOAMOUNT_START, STATEMENT_SOKUHOAMOUNT_END );
						int iValue = 0; 
						try {
							iValue = Integer.parseInt( tmpStr );
						} catch ( Exception e ) {
							iValue = 0;
						}
						if( sokuhoTotalAmount != iValue ){
							m_errorMessage = "DSK Trail Data Error";
							m_errorDescription = "Sokuho Line Total Amount is incorrect";
							return result;
						}
					}
					
					if( line.length() >= STATEMENT_KAKUHOCOUNT_END ){
						String tmpStr = line.substring( STATEMENT_KAKUHOCOUNT_START, STATEMENT_KAKUHOCOUNT_END );
						int iValue = 0; 
						try {
							iValue = Integer.parseInt( tmpStr );
						} catch ( Exception e ) {
							iValue = 0;
						}
						if( kakuhoTotalCount != iValue ){
							m_errorMessage = "DSK Trail Data Error";
							m_errorDescription = "Kakuho Line Total Count is incorrect";
							return result;
						}
					}
					if( line.length() >= STATEMENT_KAKUHOAMOUNT_END ){
						String tmpStr = line.substring( STATEMENT_KAKUHOAMOUNT_START, STATEMENT_KAKUHOAMOUNT_END );
						int iValue = 0; 
						try {
							iValue = Integer.parseInt( tmpStr );
						} catch ( Exception e ) {
							iValue = 0;
						}
						if( kakuhoTotalAmount != iValue ){
							m_errorMessage = "DSK Trail Data Error";
							m_errorDescription = "Kakuho Line Total Amount is incorrect";
							return result;
						}
					}
					
					if( line.length() >= STATEMENT_CANCELCOUNT_END ){
						String tmpStr = line.substring( STATEMENT_CANCELCOUNT_START, STATEMENT_CANCELCOUNT_END );
						int iValue = 0; 
						try {
							iValue = Integer.parseInt( tmpStr );
						} catch ( Exception e ) {
							iValue = 0;
						}
						if( cancelTotalCount != iValue ){
							m_errorMessage = "DSK Trail Data Error";
							m_errorDescription = "Cancel Line Total Count is incorrect";
							return result;
						}
					}
					if( line.length() >= STATEMENT_CANCELAMOUNT_END ){
						String tmpStr = line.substring( STATEMENT_CANCELAMOUNT_START, STATEMENT_CANCELAMOUNT_END );
						int iValue = 0;
						try {
							iValue = Integer.parseInt( tmpStr );
						} catch ( Exception e ) {
							iValue = 0;
						}
						if( cancelTotalAmount != iValue ){
							m_errorMessage = "DSK Trail Data Error";
							m_errorDescription = "Cancel Line Total Amount is incorrect";
							return result;
						}
					}
					
					if( line.length() >= STATEMENT_YUBINCOUNT_END ){
						String tmpStr = line.substring( STATEMENT_YUBINCOUNT_START, STATEMENT_YUBINCOUNT_END );
						int iValue = 0; 
						try {
							iValue = Integer.parseInt( tmpStr );
						} catch ( Exception e ) {
							iValue = 0;
						}
						if( yubinTotalCount != iValue ){
							m_errorMessage = "DSK Trail Data Error";
							m_errorDescription = "Yubin Line Total Count is incorrect";
							return result;
						}
					}
					if( line.length() >= STATEMENT_YUBINAMOUNT_END ){
						String tmpStr = line.substring( STATEMENT_YUBINAMOUNT_START, STATEMENT_YUBINAMOUNT_END );
						int iValue = 0;
						try {
							iValue = Integer.parseInt( tmpStr );
						} catch ( Exception e ) {
							iValue = 0;
						}
						if( yubinTotalAmount != iValue ){
							m_errorMessage = "DSK Trail Data Error";
							m_errorDescription = "Yubin Line Total Amount is incorrect";
							return result;
						}
					}

					this.routingNo = null;
					this.bankAccountNo = null;
					this.statementDate = null;
					
				} else if( line.startsWith( STATEMENT_END_MARK ) ) {
					
					lineNumber++;
					this.routingNo = null;
					this.bankAccountNo = null;
					this.statementDate = null;
					
				} else {
					m_errorMessage = "DSK Line Mark Error";
					m_errorDescription = "Maybe the file format incorrect";
					return result;
				}
				line = this.m_reader.readLine();
				
			};
			
			result = true;
		}
		catch(IOException e)
		{
			m_errorMessage = "ErrorReadingData";
			m_errorDescription = e.getMessage();
		}
		return result;
		
	}	//	loadLines

	public String getLastErrorMessage(){
		return this.m_errorMessage;
	}

	public String getLastErrorDescription(){
		return this.m_errorDescription;
	}
	
	public Timestamp getDateLastRun(){
		return this.dateLastRun;
	}
	
	
	
	public String getBankAccountNo() {
		return bankAccountNo;
	}

	public void setBankAccountNo(String bankAccountNo) {
		this.bankAccountNo = bankAccountNo;
	}

	public BigDecimal getChargeAmt() {
		return chargeAmt;
	}

	public void setChargeAmt(BigDecimal chargeAmt) {
		this.chargeAmt = chargeAmt;
	}

	public String getChargeName() {
		return chargeName;
	}

	public void setChargeName(String chargeName) {
		this.chargeName = chargeName;
	}

	public String getCheckNo() {
		return checkNo;
	}

	public void setCheckNo(String checkNo) {
		this.checkNo = checkNo;
	}

	public String getCurrency() {
		return currency;
	}

	public void setCurrency(String currency) {
		this.currency = currency;
	}

	public BigDecimal getInterestAmt() {
		return interestAmt;
	}

	public void setInterestAmt(BigDecimal interestAmt) {
		this.interestAmt = interestAmt;
	}

	public String getMemo() {
		return memo;
	}

	public void setMemo(String memo) {
		this.memo = memo;
	}

	public String getPayeeAccountNo() {
		return payeeAccountNo;
	}

	public void setPayeeAccountNo(String payeeAccountNo) {
		this.payeeAccountNo = payeeAccountNo;
	}

	public String getPayeeName() {
		return payeeName;
	}

	public void setPayeeName(String payeeName) {
		this.payeeName = payeeName;
	}

	public String getReference() {
		return reference;
	}

	public void setReference(String reference) {
		this.reference = reference;
	}

	public boolean getIsReversal() {
		return isReversal;
	}

	public void setIsReversal(boolean reversal) {
		this.isReversal = reversal;
	}

	public String getRoutingNo() {
		return routingNo;
	}

	public void setRoutingNo(String routingNo) {
		this.routingNo = routingNo;
	}

	public Timestamp getStatementDate() {
		return statementDate;
	}

	public void setStatementDate(Timestamp statementDate) {
		this.statementDate = statementDate;
	}

	public Timestamp getStatementLineDate() {
		return statementLineDate;
	}

	public void setStatementLineDate(Timestamp statementLineDate) {
		this.statementLineDate = statementLineDate;
	}

	public String getStatementReference() {
		return statementReference;
	}

	public void setStatementReference(String statementReference) {
		this.statementReference = statementReference;
	}

	public BigDecimal getStmtAmt() {
		return stmtAmt;
	}

	public void setStmtAmt(BigDecimal stmtAmt) {
		this.stmtAmt = stmtAmt;
	}

	public BigDecimal getTrxAmt() {
		return trxAmt;
	}

	public void setTrxAmt(BigDecimal trxAmt) {
		this.trxAmt = trxAmt;
	}

	public String getTrxID() {
		return trxID;
	}

	public void setTrxID(String trxID) {
		this.trxID = trxID;
	}

	public String getTrxType() {
		return trxType;
	}

	public void setTrxType(String trxType) {
		this.trxType = trxType;
	}

	public Timestamp getValutaDate() {
		return valutaDate;
	}

	public void setValutaDate(Timestamp valutaDate) {
		this.valutaDate = valutaDate;
	}

	public Timestamp getAccountDate() {
		return accountDate;
	}

	public void setAccountDate(Timestamp accountDate) {
		this.accountDate = accountDate;
	}

	public String getBPartnerValue() {
		return bPartnerValue;
	}

	public void setBPartnerValue(String partnerValue) {
		bPartnerValue = partnerValue;
	}

	public String getInvoiceDocumentNo() {
		return invoiceDocumentNo;
	}

	public void setInvoiceDocumentNo(String invoiceDocumentNo) {
		this.invoiceDocumentNo = invoiceDocumentNo;
	}

	public BigDecimal getLineNo() {
		return lineNo;
	}

	public void setLineNo(BigDecimal lineNo) {
		this.lineNo = lineNo;
	}

	public boolean isDeleteBeforeImport() {
		return deleteBeforeImport;
	}

	public void setDeleteBeforeImport(boolean deleteBeforeImport) {
		this.deleteBeforeImport = deleteBeforeImport;
	}
	
	private String lookupInvoiceNoFromShipment( String shipmentNo ) {
		
		String shortShipmentNo = removeLeadingZero( shipmentNo ); 
		String invoiceNo = null;
		
		int C_Invoice_ID = 0;
		boolean changed = false;
		
		StringBuffer sql = new StringBuffer("SELECT C_INVOICE_ID FROM X_SBS_SHIPMENT"
			+ " WHERE AD_CLIENT_ID = " + m_controller.getAD_Client_ID()  
			+ " AND AD_ORG_ID = " + m_controller.getAD_Org_ID()
			+ " AND ISACTIVE = 'Y'"
			+ " AND PROCESSED = 'Y'"
			+ " AND A70_D0030 = '" + shortShipmentNo + "'");
			
		PreparedStatement pstmt = null;
		try
		{
			pstmt = DB.prepareStatement(sql.toString(), m_controller.get_TrxName());
			ResultSet rs = pstmt.executeQuery();
			
			if (rs.next())
			{ 
				C_Invoice_ID = rs.getInt( 1 );
			}
			
			//	Close database connection
			rs.close();
			pstmt.close();
			rs = null;
			pstmt = null;

		}
		catch(Exception e)
		{
			log.log(Level.SEVERE, sql.toString(), e);
		}
		
		if( C_Invoice_ID == 0 ){

			sql = new StringBuffer("SELECT INV.C_INVOICE_ID FROM C_INVOICE INV, M_INOUT INOUT"
				+ " WHERE INV.AD_CLIENT_ID = INOUT.AD_CLIENT_ID " 
				+ " AND INV.AD_ORG_ID = INOUT.AD_ORG_ID "
				+ " AND INV.C_ORDER_ID = INOUT.C_ORDER_ID "
				+ " AND INV.AD_CLIENT_ID = " + m_controller.getAD_Client_ID()  
				+ " AND INV.AD_ORG_ID = " + m_controller.getAD_Org_ID()
				+ " AND INV.ISACTIVE = 'Y'"
				+ " AND INOUT.ISACTIVE = 'Y'"
				+ " AND INOUT.MOVEMENTTYPE = 'C-'"
				+ " AND INOUT.DOCUMENTNO = '" + shortShipmentNo + "'");
				
			pstmt = null;
			try
			{
				pstmt = DB.prepareStatement(sql.toString(), m_controller.get_TrxName());
				ResultSet rs = pstmt.executeQuery();
				
				if (rs.next())
				{ 
					C_Invoice_ID = rs.getInt( 1 );
					changed = true;
				}
				
				//	Close database connection
				rs.close();
				pstmt.close();
				rs = null;
				pstmt = null;

			}
			catch(Exception e)
			{
				log.log(Level.SEVERE, sql.toString(), e);
			}
		
		}
		
		if( C_Invoice_ID != 0 ){
			
			sql = new StringBuffer("SELECT DOCUMENTNO FROM C_INVOICE "
				+ " WHERE AD_CLIENT_ID = " + m_controller.getAD_Client_ID()  
				+ " AND AD_ORG_ID = " + m_controller.getAD_Org_ID()
				+ " AND ISACTIVE = 'Y'"
				+ " AND C_INVOICE_ID = " + C_Invoice_ID );
				
			pstmt = null;
			try
			{
				pstmt = DB.prepareStatement(sql.toString(), m_controller.get_TrxName());
				ResultSet rs = pstmt.executeQuery();
				
				if (rs.next())
				{ 
					invoiceNo = rs.getString( 1 );
				}
				
				//	Close database connection
				rs.close();
				pstmt.close();
				rs = null;
				pstmt = null;

				if( changed ){
					sql = new StringBuffer("UPDATE X_SBS_SHIPMENT" 
							+ " SET C_INVOICE_ID = " + C_Invoice_ID
							+ " WHERE AD_CLIENT_ID = " + m_controller.getAD_Client_ID()  
							+ " AND AD_ORG_ID = " + m_controller.getAD_Org_ID()
							+ " AND ISACTIVE = 'Y'"
							+ " AND PROCESSED = 'Y'"
							+ " AND A70_D0030 = '" + shortShipmentNo + "'");
					int no = DB.executeUpdate(sql.toString(), m_controller.get_TrxName());
				}
				
			}
			catch(Exception e)
			{
				log.log(Level.SEVERE, sql.toString(), e);
			}
			
		}
		
		return invoiceNo;
	}

	private String lookupInvoiceNoFromOrder( String orderNo ) {
		
		String shortOrderNo = removeLeadingZero( orderNo ); 
		String invoiceNo = null;
		
		int C_Invoice_ID = 0;
		int C_Order_ID = 0;
		boolean changed = false;
		
		PreparedStatement pstmt = null;
		
		StringBuffer sql = new StringBuffer("SELECT I.C_INVOICE_ID, I.C_ORDER_ID FROM C_INVOICE I, C_ORDER O"
				+ " WHERE I.AD_CLIENT_ID = O.AD_CLIENT_ID " 
				+ " AND I.AD_ORG_ID = O.AD_ORG_ID "
				+ " AND I.C_ORDER_ID = O.C_ORDER_ID "
				+ " AND I.AD_CLIENT_ID = " + m_controller.getAD_Client_ID()  
				+ " AND I.AD_ORG_ID = " + m_controller.getAD_Org_ID()
				+ " AND I.ISACTIVE = 'Y'"
				+ " AND O.ISACTIVE = 'Y'"
				+ " AND O.ISSOTRX = 'Y'"
				+ " AND O.DOCUMENTNO = '" + shortOrderNo + "'");
				
		pstmt = null;
		try
		{
			pstmt = DB.prepareStatement(sql.toString(), m_controller.get_TrxName());
			ResultSet rs = pstmt.executeQuery();
			
			if (rs.next())
			{ 
				C_Invoice_ID = rs.getInt( 1 );
				C_Order_ID = rs.getInt( 2 );
				changed = true;
			}
			
			//	Close database connection
			rs.close();
			pstmt.close();
			rs = null;
			pstmt = null;

		}
		catch(Exception e)
		{
			log.log(Level.SEVERE, sql.toString(), e);
		}
	
		
		if( C_Invoice_ID != 0 && C_Order_ID != 0 ){
			
			sql = new StringBuffer("SELECT DOCUMENTNO FROM C_INVOICE "
				+ " WHERE AD_CLIENT_ID = " + m_controller.getAD_Client_ID()  
				+ " AND AD_ORG_ID = " + m_controller.getAD_Org_ID()
				+ " AND ISACTIVE = 'Y'"
				+ " AND C_INVOICE_ID = " + C_Invoice_ID );
				
			pstmt = null;
			try
			{
				pstmt = DB.prepareStatement(sql.toString(), m_controller.get_TrxName());
				ResultSet rs = pstmt.executeQuery();
				
				if (rs.next())
				{ 
					invoiceNo = rs.getString( 1 );
				}
				
				//	Close database connection
				rs.close();
				pstmt.close();
				rs = null;
				pstmt = null;

				if( changed ){
					sql = new StringBuffer("UPDATE X_SBS_SHIPMENT" 
							+ " SET C_INVOICE_ID = " + C_Invoice_ID
							+ " WHERE AD_CLIENT_ID = " + m_controller.getAD_Client_ID()  
							+ " AND AD_ORG_ID = " + m_controller.getAD_Org_ID()
							+ " AND ISACTIVE = 'Y'"
							+ " AND PROCESSED = 'Y'"
							+ " AND C_ORDER_ID = " + C_Order_ID + " ");
					int no = DB.executeUpdate(sql.toString(), m_controller.get_TrxName());
				}
				
			}
			catch(Exception e)
			{
				log.log(Level.SEVERE, sql.toString(), e);
			}
			
		}
		
		return invoiceNo;
	}

	private String removeLeadingZero( String documentNo ){
		if( documentNo == null ){
			return documentNo;
		}
		StringBuffer documentNoBuffer = new StringBuffer( documentNo ); 
		int i = 0;
		for(i = 0; i<documentNoBuffer.length(); i++){
			char tmp = documentNoBuffer.charAt(i);
			if( tmp != '0' ){
				break;
			}
		}
		documentNoBuffer.replace(0, i, "");
		
		return documentNoBuffer.toString();
	}
}
