/******************************************************************************
 * The contents of this file are subject to the   Compiere License  Version 1.1
 * ("License"); You may not use this file except in compliance with the License
 * You may obtain a copy of the License at http://www.compiere.org/license.html
 * Software distributed under the License is distributed on an  "AS IS"  basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 * The Initial Developer is ActFact BV.
 * Copyright (C) 2003-2004 ActFact BV and Compiere Inc.;   All Rights Reserved.
 * Contributor(s): ______________________________________.
 *****************************************************************************/

package org.compiere.extend.pt;

import java.math.BigDecimal;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.Properties;
import java.util.logging.Level;

import org.compiere.model.MInvoice;
import org.compiere.model.MPayment;
import org.compiere.model.X_I_BankStatement;
import org.compiere.process.ProcessInfoParameter;
import org.compiere.process.SvrProcess;
import org.compiere.util.DB;
import org.compiere.util.Env;
import org.compiere.util.Msg;

/**
 *	Create payment from I_BankStatement
 *
 *	author Jirimuto
 *	@version $Id: DSKPaymentBatch.java,v 1.1 2006/09/25 14:20:44 hans Exp $
 */
public class DSKPaymentBatch extends SvrProcess
{
	/**	Client to be imported to		*/
	private int				m_AD_Client_ID = 0;
	/**	Organization to be imported to	*/
	private int				m_AD_Org_ID = 0;
	/** Default Bank Account			*/
	private int				m_C_BankAccount_ID = 0;
	
	private Properties m_ctx;

	/**
	 *  Prepare - e.g., get Parameters.
	 */
	protected void prepare()
	{
		ProcessInfoParameter[] para = getParameter();
		for (int i = 0; i < para.length; i++)
		{
			String name = para[i].getParameterName();
			if (para[i].getParameter() == null)
				;
			else if (name.equals("AD_Client_ID"))
				m_AD_Client_ID = ((BigDecimal)para[i].getParameter()).intValue();
			else if (name.equals("AD_Org_ID"))
				m_AD_Org_ID = ((BigDecimal)para[i].getParameter()).intValue();
			else if (name.equals("C_BankAccount_ID"))
				m_C_BankAccount_ID = ((BigDecimal)para[i].getParameter()).intValue();
			else
				log.log(Level.SEVERE, "Unknown Parameter: " + name);
		}
		m_ctx = Env.getCtx();
		
	}	//	prepare


	/**
	 *  Perform process.
	 *  @return Message
	 *  @throws Exception
	 */
	protected String doIt() throws java.lang.Exception
	{
		log.info("");
		StringBuffer sql = null;
		int errNo = 0;	
		int no = 0;	
		String clientCheck = " AND AD_Client_ID=" + m_AD_Client_ID;

		//	****	Prepare	****

		//	Set Client, Org, IsActive, Created/Updated
		sql = new StringBuffer ("UPDATE I_BankStatement "
			  + "SET AD_Client_ID = COALESCE (AD_Client_ID,").append (m_AD_Client_ID).append ("),"
			  + " AD_Org_ID = COALESCE (AD_Org_ID,").append (m_AD_Org_ID).append ("),");
		sql.append(" IsActive = COALESCE (IsActive, 'Y'),"
			  + " Created = COALESCE (Created, SysDate),"
			  + " CreatedBy = COALESCE (CreatedBy, 0),"
			  + " Updated = COALESCE (Updated, SysDate),"
			  + " UpdatedBy = COALESCE (UpdatedBy, 0),"
			  + " I_ErrorMsg = NULL,"
			  + " I_IsImported = 'N' "
			  + "WHERE I_IsImported<>'Y' OR I_IsImported IS NULL");
		no = DB.executeUpdate(sql.toString(), get_TrxName());
		log.info ("Reset=" + no);

		sql = new StringBuffer ("UPDATE I_BankStatement o "
			+ "SET I_IsImported='E', I_ErrorMsg=I_ErrorMsg||'ERR=Invalid Org, '"
			+ "WHERE (AD_Org_ID IS NULL OR AD_Org_ID=0"
			+ " OR EXISTS (SELECT * FROM AD_Org oo WHERE o.AD_Org_ID=oo.AD_Org_ID AND (oo.IsSummary='Y' OR oo.IsActive='N')))"
			+ " AND I_IsImported<>'Y'").append (clientCheck);
		no = DB.executeUpdate(sql.toString(), get_TrxName());
		errNo += no;
		if (no != 0)
			log.warning ("Invalid Org=" + no);
			
		//	Set Bank Account
		sql = new StringBuffer("UPDATE I_BankStatement i "
			+ "SET C_BankAccount_ID="
			+ "( "
			+ " SELECT C_BankAccount_ID "
			+ " FROM C_BankAccount a, C_Bank b "
			+ " WHERE b.IsOwnBank='Y' "
			+ " AND a.AD_Client_ID=i.AD_Client_ID "
			+ " AND a.C_Bank_ID=b.C_Bank_ID "
			+ " AND a.AccountNo=i.BankAccountNo "
			+ " AND b.RoutingNo=i.RoutingNo "
			+ " OR b.SwiftCode=i.RoutingNo "
			+ ") "
			+ "WHERE i.C_BankAccount_ID IS NULL "
			+ "AND i.I_IsImported<>'Y' "
			+ "OR i.I_IsImported IS NULL").append(clientCheck);
		no = DB.executeUpdate(sql.toString(), get_TrxName());
		if (no != 0)
			log.info("Bank Account (With Routing No)=" + no);
		//
		sql = new StringBuffer("UPDATE I_BankStatement i " 
		 	+ "SET C_BankAccount_ID="
			+ "( "
			+ " SELECT C_BankAccount_ID "
			+ " FROM C_BankAccount a, C_Bank b "
			+ " WHERE b.IsOwnBank='Y' "
			+ " AND a.C_Bank_ID=b.C_Bank_ID " 
			+ " AND a.AccountNo=i.BankAccountNo "
			+ " AND a.AD_Client_ID=i.AD_Client_ID "
			+ ") "
			+ "WHERE i.C_BankAccount_ID IS NULL "
			+ "AND i.I_isImported<>'Y' "
			+ "OR i.I_isImported IS NULL").append(clientCheck);
		no = DB.executeUpdate(sql.toString(), get_TrxName());
		if (errNo != 0)
			log.info("Bank Account (Without Routing No)=" + no);
		//
		sql = new StringBuffer("UPDATE I_BankStatement i "
			+ "SET C_BankAccount_ID=(SELECT C_BankAccount_ID FROM C_BankAccount a WHERE a.C_BankAccount_ID=").append(m_C_BankAccount_ID);
		sql.append(" and a.AD_Client_ID=i.AD_Client_ID) "
			+ "WHERE i.C_BankAccount_ID IS NULL "
			+ "AND i.BankAccountNo IS NULL "
			+ "AND i.I_isImported<>'Y' "
			+ "OR i.I_isImported IS NULL").append(clientCheck);
		no = DB.executeUpdate(sql.toString(), get_TrxName());
		if (no != 0)
			log.info("Bank Account=" + no);
		//	
		sql = new StringBuffer("UPDATE I_BankStatement "
			+ "SET I_isImported='E', I_ErrorMsg=I_ErrorMsg||'ERR=Invalid Bank Account, ' "
			+ "WHERE C_BankAccount_ID IS NULL "
			+ "AND I_isImported<>'Y' "
			+ "OR I_isImported IS NULL").append(clientCheck);
		no = DB.executeUpdate(sql.toString(), get_TrxName());
		if (no != 0)
			log.warning("Invalid Bank Account=" + no);
		 
		//	Set Currency
		sql = new StringBuffer ("UPDATE I_BankStatement i "
			+ "SET C_Currency_ID=(SELECT C_Currency_ID FROM C_Currency c"
			+ " WHERE i.ISO_Code=c.ISO_Code AND c.AD_Client_ID IN (0,i.AD_Client_ID)) "
			+ "WHERE C_Currency_ID IS NULL"
			+ " AND I_IsImported<>'Y'").append(clientCheck);
		no = DB.executeUpdate(sql.toString(), get_TrxName());
		if (no != 0)
			log.info("Set Currency=" + no);
		//
		sql = new StringBuffer("UPDATE I_BankStatement i "
			+ "SET i.C_Currency_ID=(SELECT C_Currency_ID FROM C_BankAccount WHERE C_BankAccount_ID=i.C_BankAccount_ID) "
			+ "WHERE i.C_Currency_ID IS NULL "
			+ "AND i.ISO_Code IS NULL").append(clientCheck);
		no = DB.executeUpdate(sql.toString(), get_TrxName());
		if (no != 0)
			log.info("Set Currency=" + no);
		//
		sql = new StringBuffer ("UPDATE I_BankStatement "
			+ "SET I_IsImported='E', I_ErrorMsg=I_ErrorMsg||'ERR=Invalid Currency,' "
			+ "WHERE C_Currency_ID IS NULL "
			+ "AND I_IsImported<>'E' "
			+ " AND I_IsImported<>'Y'").append(clientCheck);
		no = DB.executeUpdate(sql.toString(), get_TrxName());
		errNo += no;
		if (no != 0)
			log.warning("Invalid Currency=" + no);
		
		/**
		 *  Jirimuto Modified 2005/11/29
		 */
		// set C_Invoice_ID 
		sql = new StringBuffer("UPDATE I_BankStatement i " 
		 	+ "SET C_Invoice_ID="
			+ "( "
			+ " SELECT C_Invoice_ID "
			+ " FROM C_Invoice a "
			+ " WHERE a.IsActive='Y' "
			+ " AND a.AD_Client_ID=i.AD_Client_ID "
			+ " AND a.DocumentNo=i.InvoiceDocumentNo "
			+ ") "
			+ "WHERE i.C_Invoice_ID IS NULL "
			+ "AND i.I_isImported<>'Y' "
			+ "OR i.I_isImported IS NULL").append(clientCheck);
		no = DB.executeUpdate(sql.toString(), get_TrxName());
		if (no != 0)
			log.info("Invoice ID adjusted =" + no);

		/**
		 *  Jirimuto Modified 2005/11/29
		 */
		// set C_Bpartner_ID 
		sql = new StringBuffer("UPDATE I_BankStatement i " 
		 	+ "SET C_Bpartner_ID="
			+ "( "
			+ " SELECT C_Bpartner_ID "
			+ " FROM C_Bpartner a "
			+ " WHERE a.IsActive='Y' "
			+ " AND a.AD_Client_ID=i.AD_Client_ID "
			+ " AND a.Value=i.BpartnerValue "
			+ ") "
			+ " "
			+ " WHERE i.I_isImported<>'Y' "
			+ "OR i.I_isImported IS NULL").append(clientCheck);
		no = DB.executeUpdate(sql.toString(), get_TrxName());
		if (no != 0)
			log.info("Bpartner ID adjusted =" + no);
		
		//	Set Amount
		 sql = new StringBuffer("UPDATE I_BankStatement "
		 	+ "SET ChargeAmt=0 "
			+ "WHERE ChargeAmt IS NULL "
			+ "AND I_IsImported<>'Y'").append(clientCheck);
		no = DB.executeUpdate(sql.toString(), get_TrxName());
		if (no != 0)
			log.info("Charge Amount=" + no);
		//
		 sql = new StringBuffer("UPDATE I_BankStatement "
		 	+ "SET InterestAmt=0 "
			+ "WHERE InterestAmt IS NULL "
			+ "AND I_IsImported<>'Y'").append(clientCheck);
		no = DB.executeUpdate(sql.toString(), get_TrxName());
		if (no != 0)
			log.info("Interest Amount=" + no);
		//
		 sql = new StringBuffer("UPDATE I_BankStatement "
		 	+ "SET TrxAmt=StmtAmt - InterestAmt - ChargeAmt "
			+ "WHERE TrxAmt IS NULL "
			+ "AND I_IsImported<>'Y'").append(clientCheck);
		no = DB.executeUpdate(sql.toString(), get_TrxName());
		if (no != 0)
			log.info("Transaction Amount=" + no);
		//
		sql = new StringBuffer("UPDATE I_BankStatement "
			+ "SET I_isImported='E', I_ErrorMsg=I_ErrorMsg||'Err=Invalid Amount, ' "
			+ "WHERE TrxAmt + ChargeAmt + InterestAmt <> StmtAmt "
			+ "AND I_isImported<>'Y'").append(clientCheck);
		no = DB.executeUpdate(sql.toString(), get_TrxName());
		errNo += no;
		if (no != 0)
			log.info("Invaid Amount=" + no);
		 
		 //	Set Valuta Date
		sql = new StringBuffer("UPDATE I_BankStatement "
		 	+ "SET ValutaDate=StatementLineDate "
			+ "WHERE ValutaDate IS NULL "
			+ "AND I_isImported<>'Y'").append(clientCheck);
		no = DB.executeUpdate(sql.toString(), get_TrxName());
		if (no != 0)
			log.info("Valuta Date=" + no);
			
		//	Check Payment<->Invoice combination
		sql = new StringBuffer("UPDATE I_BankStatement "
			+ "SET I_IsImported='E', I_ErrorMsg=I_ErrorMsg||'Err=Invalid Payment<->Invoive, ' "
			+ "WHERE I_BankStatement_ID IN "
			+ "( "
			+ " SELECT I_BankStatement_ID "
			+ " FROM I_BankStatement i, C_Payment p "
			+ " WHERE i.C_Invoice_ID IS NOT NULL "
			+ " AND i.C_Payment_ID IS NOT NULL "
			+ " AND p.C_Invoice_ID IS NOT NULL "
			+ " AND p.C_Invoice_ID<>i.C_Invoice_ID "
			+ " GROUP BY I_BankStatement_ID "
			+ ")").append(clientCheck);
		no = DB.executeUpdate(sql.toString(), get_TrxName());
		errNo += no;
		if (no != 0)
			log.info("Payment<->Invoice Mismatch=" + no);
			
		//	Check Payment<->BPartner combination
		sql = new StringBuffer("UPDATE I_BankStatement "
			+ "SET I_IsImported='E', I_ErrorMsg=I_ErrorMsg||'Err=Invalid Payment<->BPartner, ' "
			+ "WHERE I_BankStatement_ID IN "
			+ "( "
			+ " SELECT I_BankStatement_ID "
			+ " FROM I_BankStatement i, C_Payment p "
			+ " WHERE i.C_Payment_ID IS NOT NULL "
			+ " AND i.C_BPartner_ID IS NOT NULL "
			+ " AND p.C_BPartner_ID IS NOT NULL "
			+ " AND p.C_BPartner_ID<>i.C_BPartner_ID "
			+ " GROUP BY I_BankStatement_ID "
			+ ")").append(clientCheck);
		no = DB.executeUpdate(sql.toString(), get_TrxName());
		errNo += no;
		if (no != 0)
			log.info("Payment<->BPartner Mismatch=" + no);
			
		//	Check Invoice<->BPartner combination
		sql = new StringBuffer("UPDATE I_BankStatement "
			+ "SET I_IsImported='I', I_ErrorMsg=I_ErrorMsg||'Err=Invalid Invoice<->BPartner, ' "
			+ "WHERE I_BankStatement_ID IN "
			+ "( "
			+ " SELECT I_BankStatement_ID "
			+ " FROM I_BankStatement i, C_Invoice v "
			+ " WHERE i.C_BPartner_ID IS NOT NULL "
			+ " AND i.C_Invoice_ID IS NOT NULL "
			+ " AND v.C_BPartner_ID IS NOT NULL "
			+ " AND v.C_Invoice_ID=i.C_Invoice_ID "
			+ " AND v.C_BPartner_ID<>i.C_BPartner_ID "
			+ " GROUP BY I_BankStatement_ID "
			+ ")").append(clientCheck);
		no = DB.executeUpdate(sql.toString(), get_TrxName());
		if (no != 0)
			log.info("Invoice<->BPartner Mismatch=" + no);
			
		//	Check Invoice<->BPartner combination
		sql = new StringBuffer("UPDATE I_BankStatement "
			+ "SET I_IsImported='I', I_ErrorMsg=I_ErrorMsg||'Err=Invalid Invoice<->BPartner, ' "
			+ "WHERE I_BankStatement_ID IN "
			+ "( "
			+ " SELECT I_BankStatement_ID "
			+ " FROM I_BankStatement i "
			+ " WHERE ( i.C_BPartner_ID IS NULL "
			+ " OR i.C_Invoice_ID IS NULL) "
			+ " AND ( i.REFERENCENO='01' "
			+ " OR i.REFERENCENO='01' ) "
			+ " GROUP BY I_BankStatement_ID "
			+ ")").append(clientCheck);
		no = DB.executeUpdate(sql.toString(), get_TrxName());
		if (no != 0)
			log.info("Invoice<->BPartner Mismatch=" + no);
		
		//	Check Invoice.BPartner<->Payment.BPartner combination
		sql = new StringBuffer("UPDATE I_BankStatement "
			+ "SET I_IsImported='E', I_ErrorMsg=I_ErrorMsg||'Err=Invalid Invoice.BPartner<->Payment.BPartner, ' "
			+ "WHERE I_BankStatement_ID IN "
			+ "( "
			+ " SELECT I_BankStatement_ID "
			+ " FROM I_BankStatement i, C_Invoice v, C_Payment p "
			+ " WHERE p.C_Invoice_ID<>v.C_Invoice_ID"
			+ " AND i.C_Invoice_ID IS NOT NULL "
			+ " AND i.C_Payment_ID IS NOT NULL "
			+ " AND v.C_BPartner_ID IS NOT NULL "
			+ " AND p.C_BPartner_ID IS NOT NULL "
			+ " AND v.C_BPartner_ID<>p.C_BPartner_ID "
			+ " GROUP BY I_BankStatement_ID "
			+ ")").append(clientCheck);
		no = DB.executeUpdate(sql.toString(), get_TrxName());
		errNo += no;
		if (no != 0)
			log.info("Invoice.BPartner<->Payment.BPartner Mismatch=" + no);
			
		//	Check Bank statement already processed 
		sql = new StringBuffer("UPDATE I_BankStatement "
			+ "SET I_IsImported='E', I_ErrorMsg=I_ErrorMsg||'Err=Bank Statement already processed. ' "
			+ "WHERE I_BankStatement_ID IN "
			+ "( "
			+ " SELECT I_BankStatement_ID "
			+ " FROM I_BankStatement i, X_DSK_BankStatement x "
			+ " WHERE i.StatementDate = x.StatementDate"
			+ " AND i.Line = x.Line "
			+ " AND i.ReferenceNo = x.ReferenceNo "
			+ " AND i.BPartnerValue = x.BPartnerValue "
			+ " AND i.InvoiceDocumentNo = x.InvoiceDocumentNo "
			+ " GROUP BY I_BankStatement_ID "
			+ ")").append(clientCheck);
		no = DB.executeUpdate(sql.toString(), get_TrxName());
		errNo += no;
		if (no != 0)
			log.info("Bank Statement already created payment = " + no);
			
		//	Detect Duplicates
		 sql = new StringBuffer("SELECT i.I_BankStatement_ID, l.C_BankStatementLine_ID, i.EftTrxID "
			+ "FROM I_BankStatement i, C_BankStatement s, C_BankStatementLine l "
			+ "WHERE i.I_isImported='N' "
			+ "AND s.C_BankStatement_ID=l.C_BankStatement_ID "
			+ "AND i.EftTrxID IS NOT NULL AND "
			//	Concatinate EFT Info
			+ "(l.EftTrxID||l.EftAmt||l.EftStatementLineDate||l.EftValutaDate||l.EftTrxType||l.EftCurrency||l.EftReference||s.EftStatementReference "
			+ "||l.EftCheckNo||l.EftMemo||l.EftPayee||l.EftPayeeAccount) "
			+ "= "
			+ "(i.EftTrxID||i.EftAmt||i.EftStatementLineDate||i.EftValutaDate||i.EftTrxType||i.EftCurrency||i.EftReference||i.EftStatementReference "
			+ "||i.EftCheckNo||i.EftMemo||i.EftPayee||i.EftPayeeAccount) ");
		
		StringBuffer updateSql = new StringBuffer("UPDATE I_Bankstatement "
				+ "SET I_IsImported='E', I_ErrorMsg=I_ErrorMsg||'Err=Duplicate['||?||']' "
				+ "WHERE I_BankStatement_ID=?").append(clientCheck);
		PreparedStatement pupdt = DB.prepareStatement(updateSql.toString(), get_TrxName());
		
		PreparedStatement pstmtDuplicates = null;
		no = 0;
		try
		{
			pstmtDuplicates = DB.prepareStatement(sql.toString(), get_TrxName());
			ResultSet rs = pstmtDuplicates.executeQuery();
			while (rs.next())
			{
				String info = "Line_ID=" + rs.getInt(2)		//	l.C_BankStatementLine_ID
				 + ",EDTTrxID=" + rs.getString(3);			//	i.EftTrxID
				pupdt.setString(1, info);	
				pupdt.setInt(2, rs.getInt(1));	//	i.I_BankStatement_ID
				pupdt.executeUpdate();
				no++;
			}
			rs.close();
			pstmtDuplicates.close();
			pupdt.close();
			
			rs = null;
			pstmtDuplicates = null;
			pupdt = null;
		}
		catch(Exception e)
		{
			log.log(Level.SEVERE, "DetectDuplicates " + e.getMessage());
		}
		errNo += no;
		if (no != 0)
			log.info("Duplicates=" + no);
		
		commit();
		
		PreparedStatement pstmt = null;
		
		// Checking the data before import
		sql = new StringBuffer("SELECT * FROM I_BankStatement"
			+ " WHERE C_Payment_ID IS NULL AND I_IsImported='I'"
			+ " ORDER BY C_BankAccount_ID, Name, StatementDate, Line, StatementLineDate");
			
		try
		{
			pstmt = DB.prepareStatement(sql.toString(), get_TrxName());
			ResultSet rs = pstmt.executeQuery();
			
			while (rs.next())
			{ 
				X_I_BankStatement imp = new X_I_BankStatement(m_ctx, rs, get_TrxName());

				if (imp != null || imp.getC_Payment_ID() == 0 ){

					if( imp.getI_ErrorMsg() == null ){
						imp.setI_ErrorMsg( "");
					}
					log.fine(imp.toString());
	
					
					if( "01".equals( imp.getReferenceNo() )){
						// Sokuho
						continue;
					} else if( "03".equals( imp.getReferenceNo() )) {
						// Cancel
						if( sokuhoExist( imp ) ){
							continue;
						}
					}
					imp.setI_ErrorMsg( "ERROR : Invoice No or BPartner Value Error." );
					errNo++;
					imp.save();
				}
			}
			
			//	Close database connection
			rs.close();
			pstmt.close();
			rs = null;
			pstmt = null;

		}
		catch(Exception e)
		{
			log.log(Level.SEVERE, sql.toString(), e);
		}
		commit();


		MPayment payment = null;
		
		// Checking the data before import
		sql = new StringBuffer("SELECT * FROM I_BankStatement"
			+ " WHERE C_Payment_ID IS NULL AND I_IsImported<>'Y' AND I_IsImported<>'E' AND I_IsImported<>'I'"
			+ " ORDER BY C_BankAccount_ID, Name, StatementDate, Line, StatementLineDate");
			
		try
		{
			pstmt = DB.prepareStatement(sql.toString(), get_TrxName());
			ResultSet rs = pstmt.executeQuery();
			
			while (rs.next())
			{ 
				X_I_BankStatement imp = new X_I_BankStatement(m_ctx, rs, get_TrxName());

				if (imp != null || imp.getC_Payment_ID() == 0 ){

					if( imp.getI_ErrorMsg() == null ){
						imp.setI_ErrorMsg( "");
					}
					log.fine(imp.toString());
					if (imp.getC_Invoice_ID() == 0 ){
						String msg = Msg.translate(getCtx(), " @NotFound@ @C_Invoice_ID@  ");
						imp.setI_ErrorMsg( imp.getI_ErrorMsg()+ msg);
						imp.save();
						errNo++;
						continue;
					}
					if ( imp.getC_BPartner_ID() == 0 ){
						String msg = Msg.translate(getCtx(), " @NotFound@ @C_BPartner_ID@ ");
						imp.setI_ErrorMsg( imp.getI_ErrorMsg()+ msg);
						imp.save();
						errNo++;
						continue;
					}
					if (imp.getC_BankAccount_ID() == 0){
						String msg = Msg.translate(getCtx(), "@NotFound@ @C_BankAccount_ID@ ");
						imp.setI_ErrorMsg( imp.getI_ErrorMsg()+ msg);
						imp.save();
						errNo++;
						continue;
					}
	
					int C_Invoice_ID = imp.getC_Invoice_ID();
					int C_BPartner_ID = imp.getC_BPartner_ID();
					int C_BankAccount_ID = imp.getC_BankAccount_ID();
					Timestamp statementLineDate = imp.getStatementLineDate();
					Timestamp dateAcct = imp.getDateAcct();
					
					if( "01".equals( imp.getReferenceNo() )){
						// Sokuho
						String paymentStatus = checkPaymentStatus(C_Invoice_ID, C_BPartner_ID, C_BankAccount_ID, statementLineDate, dateAcct, null );
						
						if( MPayment.DOCSTATUS_Completed.equals( paymentStatus ) || MPayment.DOCSTATUS_Closed.equals( paymentStatus ) ){
							imp.setI_ErrorMsg( "ERROR : Payment Already Completed or Closed" );
							errNo++;
						}
						
					} else if( "02".equals( imp.getReferenceNo() )) {
						// Kakuho
						String paymentStatus = checkPaymentStatus(C_Invoice_ID, C_BPartner_ID, C_BankAccount_ID, statementLineDate, dateAcct, null );
						
						if( MPayment.DOCSTATUS_Completed.equals( paymentStatus ) || MPayment.DOCSTATUS_Closed.equals( paymentStatus ) ){
							imp.setI_ErrorMsg( "ERROR : Payment Already Completed or Closed" );
							errNo++;
						} else if( !MPayment.DOCSTATUS_Drafted.equals( paymentStatus ) && !MPayment.DOCSTATUS_InProgress.equals( paymentStatus ) ){
							imp.setI_ErrorMsg( "ERROR : In progress Payment not exist " );
							errNo++;
						}
						
					} else if( "03".equals( imp.getReferenceNo() )) {
						// Cancel
						String paymentStatus = checkPaymentStatus(C_Invoice_ID, C_BPartner_ID, C_BankAccount_ID, statementLineDate, dateAcct, null );
						
						if( MPayment.DOCSTATUS_Completed.equals( paymentStatus ) || MPayment.DOCSTATUS_Closed.equals( paymentStatus ) ){
							imp.setI_ErrorMsg( "ERROR : Payment Already Completed or Closed" );
							errNo++;
						} else if( !MPayment.DOCSTATUS_Drafted.equals( paymentStatus ) && !MPayment.DOCSTATUS_InProgress.equals( paymentStatus ) ){
							imp.setI_ErrorMsg( "ERROR : In progress Payment not exist " );
							errNo++;
						}
						
					} else if( "12".equals( imp.getReferenceNo() ) || "22".equals( imp.getReferenceNo()) ) {
						
						String paymentStatus = checkPaymentStatus(C_Invoice_ID, C_BPartner_ID, C_BankAccount_ID, statementLineDate, dateAcct, null );
						
						if( MPayment.DOCSTATUS_Completed.equals( paymentStatus ) || MPayment.DOCSTATUS_Closed.equals( paymentStatus ) ){
							imp.setI_ErrorMsg( "ERROR : Payment Already Completed or Closed" );
							errNo++;
						}
						
					} else {
							imp.setI_ErrorMsg( "ERROR : Action mark not exist" );
							errNo++;
					}
					imp.save();
				}
			}
			
			//	Close database connection
			rs.close();
			pstmt.close();
			rs = null;
			pstmt = null;

		}
		catch(Exception e)
		{
			log.log(Level.SEVERE, sql.toString(), e);
		}
		commit();

		int noPaymentCreated = 0;
		int noPaymentPrepared = 0;
		int noPaymentCompleted = 0;
		int noPaymentVoided = 0;

		if( errNo == 0 ){
			
			pstmt = null;
			//Create Payment Begin 
			sql = new StringBuffer("INSERT INTO X_DSK_BANKSTATEMENT SELECT * FROM I_BankStatement"
					+ " WHERE C_Payment_ID IS NULL AND I_IsImported<>'Y' AND I_IsImported<>'E'"
					+ " ORDER BY C_BankAccount_ID, Name, StatementDate, Line, StatementLineDate");
			no = DB.executeUpdate(sql.toString(), get_TrxName());
			commit();
			
			sql = new StringBuffer("SELECT * FROM I_BankStatement"
					+ " WHERE C_Payment_ID IS NULL AND I_IsImported<>'Y' AND I_IsImported<>'E' AND I_IsImported<>'I'"
					+ " ORDER BY C_BankAccount_ID, Name, StatementDate, Line, StatementLineDate");
				
			try
			{
				pstmt = DB.prepareStatement(sql.toString(), get_TrxName());
				ResultSet rs = pstmt.executeQuery();
				
				while (rs.next())
				{ 
					X_I_BankStatement imp = new X_I_BankStatement(m_ctx, rs, get_TrxName());
	
					if (imp != null || imp.getC_Payment_ID() != 0 ){
						
						if( imp.getI_ErrorMsg() == null ){
							imp.setI_ErrorMsg( "");
						}
						log.fine(imp.toString());
						if (imp.getC_Invoice_ID() == 0 ){
							String msg = Msg.translate(getCtx(), " @NotFound@ @C_Invoice_ID@ ");
							imp.setI_ErrorMsg( imp.getI_ErrorMsg()+ msg);
							errNo++;
							continue;
						}
						if (imp.getC_BPartner_ID() == 0){
							String msg = Msg.translate(getCtx(), " @NotFound@ @C_BPartner_ID@ ");
							imp.setI_ErrorMsg( imp.getI_ErrorMsg()+ msg);
							errNo++;
							continue;
						}
						if (imp.getC_BankAccount_ID() == 0){
							String msg = Msg.translate(getCtx(), "@NotFound@ @C_BankAccount_ID@ ");
							imp.setI_ErrorMsg( imp.getI_ErrorMsg()+ msg);
							errNo++;
							continue;
						}
		
						int C_Invoice_ID = imp.getC_Invoice_ID();
						int C_BPartner_ID = imp.getC_BPartner_ID();
						int C_BankAccount_ID = imp.getC_BankAccount_ID();
						int C_Currency_ID = imp.getC_Currency_ID();
						Timestamp statementLineDate = imp.getStatementLineDate();
						Timestamp dateAcct = imp.getDateAcct();
						Timestamp statementDate = imp.getStatementDate();
						BigDecimal stmtAmt = imp.getStmtAmt();
						BigDecimal trxAmt = imp.getTrxAmt();
						String description = (imp.getDescription()==null)?"":imp.getDescription();

						String stamp = (imp.getEftReference()==null)?"0":imp.getEftReference();
						Timestamp valutaDate = imp.getValutaDate();
						SimpleDateFormat sdf = new SimpleDateFormat( "yyyyMMdd");
						int lineNo = imp.getLine();
						String referenceNo = imp.getReferenceNo();
							
						if( "01".equals( referenceNo )){
							
							// Sokuho
							description = "DSK"+sdf.format(statementDate)+","+lineNo+",R01,00000000,"+stamp+","+description;

							String paymentStatus = checkPaymentStatus(C_Invoice_ID, C_BPartner_ID, C_BankAccount_ID, statementLineDate, dateAcct, null );
							
							if( MPayment.DOCSTATUS_Completed.equals( paymentStatus ) || MPayment.DOCSTATUS_Completed.equals( paymentStatus ) ){
								imp.setI_ErrorMsg( "ERROR : Payment Already Completed or Closed" );
							} else if( MPayment.DOCSTATUS_Drafted.equals( paymentStatus ) || MPayment.DOCSTATUS_InProgress.equals( paymentStatus ) ){
								payment = updatePayment (C_Invoice_ID, C_BPartner_ID,
										C_Currency_ID, stmtAmt, trxAmt, 
										C_BankAccount_ID, statementLineDate == null ? statementDate : statementLineDate, 
												dateAcct, description, paymentStatus, MPayment.DOCACTION_Prepare );
							} else {
								payment = createPayment (C_Invoice_ID, C_BPartner_ID,
										C_Currency_ID, stmtAmt, trxAmt, 
										C_BankAccount_ID, statementLineDate == null ? statementDate : statementLineDate, 
												dateAcct, description, MPayment.DOCACTION_Prepare );
							}
							
						} else if( "02".equals( referenceNo )) {

							// Kakuho
							description = "DSK"+sdf.format(statementDate)+","+lineNo+",R02,"+sdf.format( valutaDate )+","+stamp+","+description;

							String paymentStatus = checkPaymentStatus(C_Invoice_ID, C_BPartner_ID, C_BankAccount_ID, statementLineDate, dateAcct, null );
							
							if( MPayment.DOCSTATUS_Completed.equals( paymentStatus ) || MPayment.DOCSTATUS_Completed.equals( paymentStatus ) ){
								imp.setI_ErrorMsg( "ERROR : Payment Already Completed or Closed" );
							} else if( MPayment.DOCSTATUS_Drafted.equals( paymentStatus ) || MPayment.DOCSTATUS_InProgress.equals( paymentStatus ) ){
								
								payment = updatePayment (C_Invoice_ID, C_BPartner_ID,
										C_Currency_ID, stmtAmt, trxAmt, 
										C_BankAccount_ID, statementLineDate == null ? statementDate : statementLineDate, 
												dateAcct, description, paymentStatus, MPayment.DOCACTION_Complete );
							} else {
								
								imp.setI_ErrorMsg( "ERROR : In progress Payment not exist " );
							}
							
						} else if( "03".equals( referenceNo )) {

							// Cancel
							description = "DSK"+sdf.format(statementDate)+","+lineNo+",R03,"+sdf.format(statementDate)+","+stamp+",　INVOICE="+imp.getInvoiceDocumentNo()+","+description;

							String paymentStatus = checkPaymentStatus(C_Invoice_ID, C_BPartner_ID, C_BankAccount_ID, statementLineDate, dateAcct, null );
							
							if( MPayment.DOCSTATUS_Completed.equals( paymentStatus ) || MPayment.DOCSTATUS_Completed.equals( paymentStatus ) ){
								
								imp.setI_ErrorMsg( "ERROR : Payment Already Completed or Closed" );
								
							} else if( MPayment.DOCSTATUS_Drafted.equals( paymentStatus ) || MPayment.DOCSTATUS_InProgress.equals( paymentStatus ) ){
								
								payment = voidPayment (C_Invoice_ID, C_BPartner_ID,
										C_Currency_ID, stmtAmt, trxAmt, 
										C_BankAccount_ID, statementLineDate == null ? statementDate : statementLineDate, 
												dateAcct, description, paymentStatus );
							} else {
								
								imp.setI_ErrorMsg( "ERROR : In progress Payment not exist " );
							}
							
						} else if( "12".equals( referenceNo ) || "22".equals( referenceNo ) ) {
							
							// Yubin
							description = "DSK"+sdf.format(statementDate)+","+lineNo+",R"+referenceNo+","+sdf.format( valutaDate )+","+stamp+","+description;

							String paymentStatus = checkPaymentStatus(C_Invoice_ID, C_BPartner_ID, C_BankAccount_ID, statementLineDate, dateAcct, null );
							
							if( MPayment.DOCSTATUS_Completed.equals( paymentStatus ) || MPayment.DOCSTATUS_Completed.equals( paymentStatus ) ){
								
								imp.setI_ErrorMsg( "ERROR : Payment Already Completed or Closed" );
								
							} else {
								payment = createPayment (C_Invoice_ID, C_BPartner_ID,
										C_Currency_ID, stmtAmt, trxAmt, 
										C_BankAccount_ID, statementLineDate == null ? statementDate : statementLineDate, 
												dateAcct, description, MPayment.DOCACTION_Complete );
							}
							
						} else {
							
							imp.setI_ErrorMsg( "ERROR : Action mark not exist" );
							
						}
						
						if (payment == null){
							imp.setI_ErrorMsg( imp.getI_ErrorMsg()+ (" Could not create Payment ") );
							imp.save();
							continue;
						}
						
						//	Save payment line
						if ( payment.save() )
						{
							imp.setC_Payment_ID(payment.getC_Payment_ID());
							imp.setC_Currency_ID (payment.getC_Currency_ID());
							
							//
							String retString = "@C_Payment_ID@ = " + payment.getDocumentNo();
							if (payment.getOverUnderAmt().signum() != 0) {
								retString += " - @OverUnderAmt@=" + payment.getOverUnderAmt();
								String msg = Msg.translate(getCtx(), retString);
								imp.setI_ErrorMsg ( imp.getI_ErrorMsg() + " Payment Batch Error " + msg );
							}
							
							imp.save();
							noPaymentCreated++;
							if( MPayment.DOCSTATUS_InProgress.equals( payment.getDocStatus()) ){
								noPaymentPrepared++;
							} else if ( MPayment.DOCSTATUS_Completed.equals( payment.getDocStatus()) ) {
								noPaymentCompleted++;
							} else if ( MPayment.DOCSTATUS_Voided.equals( payment.getDocStatus()) ) {
								noPaymentVoided++;
							}
						} else {
							imp.save();
						}
					}
					payment = null;
					
				}
				
				//	Close database connection
				rs.close();
				pstmt.close();
				rs = null;
				pstmt = null;
	
			}
			catch(Exception e)
			{
				log.log(Level.SEVERE, sql.toString(), e);
				
			}
			
		}
		addLog (0, null, new BigDecimal (errNo), "@Errors@");
		
		addLog (0, null, new BigDecimal (noPaymentCreated), "@C_Payment_ID@: @Created@");
		addLog (0, null, new BigDecimal (noPaymentPrepared), "@C_Payment_ID@: InProgress");
		addLog (0, null, new BigDecimal (noPaymentCompleted), "@C_Payment_ID@: Completed");
		addLog (0, null, new BigDecimal (noPaymentVoided), "@C_Payment_ID@: Voided");
		
		return "";

	}	//	doIt
	
	/**
	 * 	check if Payment exist
	 *	@param C_Invoice_ID invoice
	 *	@param C_BPartner_ID partner
	 *	@param C_BankAccount_ID Bank Account
	 *	@param DateTrx trx date
	 *	@param DateAcct account date
	 *	@return DocStatus
	 */
	private String checkPaymentStatus (int C_Invoice_ID, int C_BPartner_ID, 
		int C_BankAccount_ID, Timestamp DateTrx, Timestamp DateAcct, 
		String docStatus )
	{

		String clientCheck = " AND AD_Client_ID=" + m_AD_Client_ID;
		
		String result = null;
		
		//Import Bank Statement
		StringBuffer sql = new StringBuffer("SELECT DocStatus FROM C_Payment" );
		sql.append( " WHERE C_Invoice_ID=" + C_Invoice_ID ); 
		if( C_BPartner_ID > 0 ){
			sql.append( " AND C_BPartner_ID=" + C_BPartner_ID); 
		}
		sql.append( " AND C_BankAccount_ID=" + C_BankAccount_ID); 
		if( docStatus != null ){
			sql.append( " AND docStatus='" + docStatus + "'");
		}
		if( DateTrx != null ){
			SimpleDateFormat sdf = new SimpleDateFormat( "yyyyMMdd");
			sql.append( " AND to_char(DateTrx,'YYYYMMDD')='" + sdf.format(DateTrx) + "'");
		}
		if( DateAcct != null ){
			SimpleDateFormat sdf = new SimpleDateFormat( "yyyyMMdd");
			sql.append( " AND to_char(DateAcct,'YYYYMMDD')='" + sdf.format(DateAcct) + "'");
		}
		sql.append( clientCheck );
		sql.append( " ORDER BY DateTrx DESC");
			
		PreparedStatement pstmt = null;
		try
		{
			pstmt = DB.prepareStatement(sql.toString(), get_TrxName());
			ResultSet rs = pstmt.executeQuery();
				
			while (rs.next())
			{ 
				result = rs.getString( 1 );
				break;
			}
			//	Close database connection
			rs.close();
			pstmt.close();
			rs = null;
			pstmt = null;

		}
		catch(Exception e)
		{
			log.log(Level.SEVERE, sql.toString(), e);
		}
	
		return result;		
	}	//	createPayment

	
	/**
	 * 	Create actual Payment
	 *	@param C_Invoice_ID invoice
	 *	@param C_BPartner_ID partner ignored when invoice exists
	 *	@param C_Currency_ID currency
	 *	@param StmtAmt statement amount
	 *	@param TrxAmt transaction amt
	 *	@return payment
	 */
	private MPayment createPayment (int C_Invoice_ID, int C_BPartner_ID, 
		int C_Currency_ID, BigDecimal StmtAmt, BigDecimal TrxAmt,
		int C_BankAccount_ID, Timestamp DateTrx, Timestamp DateAcct, String Description, 
		String docAction )
	{

		int C_Payment_ID = 0;
		//	Trx Amount = Payment overwrites Statement Amount if defined
		BigDecimal PayAmt = TrxAmt;
		if (PayAmt == null || Env.ZERO.compareTo(PayAmt) == 0)
			PayAmt = StmtAmt;
		if (C_Invoice_ID == 0
			&& (PayAmt == null || Env.ZERO.compareTo(PayAmt) == 0))
			throw new IllegalStateException ("@PayAmt@ = 0");
		if (PayAmt == null)
			PayAmt = Env.ZERO;

		//
		MPayment payment = new MPayment (getCtx(), C_Payment_ID, get_TrxName());
		payment.setC_BankAccount_ID(C_BankAccount_ID);
		payment.setTenderType(MPayment.TENDERTYPE_Check);
		if (DateTrx != null)
			payment.setDateTrx(DateTrx);
		else if (DateAcct != null)
			payment.setDateTrx(DateAcct);
		if (DateAcct != null)
			payment.setDateAcct(DateAcct);
		else
			payment.setDateAcct(payment.getDateTrx());
		payment.setDescription(Description);
		//
		if (C_Invoice_ID != 0)
		{
			MInvoice invoice = new MInvoice (getCtx(), C_Invoice_ID, null);
			payment.setC_DocType_ID(invoice.isSOTrx());		//	Receipt
			payment.setC_Invoice_ID(invoice.getC_Invoice_ID());
			payment.setC_BPartner_ID (invoice.getC_BPartner_ID());
			if (PayAmt.signum() != 0)	//	explicit Amount
			{
				payment.setC_Currency_ID(C_Currency_ID);
				if (invoice.isSOTrx())
					payment.setPayAmt(PayAmt);
				else	//	payment is likely to be negative
					payment.setPayAmt(PayAmt.negate());
				payment.setOverUnderAmt(invoice.getGrandTotal(true).subtract(payment.getPayAmt()));
			}
			else	// set Pay Amout from Invoice
			{
				payment.setC_Currency_ID(invoice.getC_Currency_ID());
				payment.setPayAmt(invoice.getGrandTotal(true));
			}
		}
		else if (C_BPartner_ID != 0)
		{
			payment.setC_BPartner_ID(C_BPartner_ID);
			payment.setC_Currency_ID(C_Currency_ID);
			if (PayAmt.signum() < 0)	//	Payment
			{
				payment.setPayAmt(PayAmt.abs());
				payment.setC_DocType_ID(false);
			}
			else	//	Receipt
			{
				payment.setPayAmt(PayAmt);
				payment.setC_DocType_ID(true);
			}
		}
		else
			return null;
		payment.save();
		//
		if( payment.getOverUnderAmt().intValue() != 0 ){
			payment.processIt(MPayment.DOCACTION_Prepare);
		} else {
			payment.processIt(docAction);
		}
		payment.save();
		return payment;		
	}	//	createPayment

	/**
	 * 	update Payment
	 *	@param C_Invoice_ID invoice
	 *	@param C_BPartner_ID partner ignored when invoice exists
	 *	@param C_Currency_ID currency
	 *	@param StmtAmt statement amount
	 *	@param TrxAmt transaction amt
	 *	@return payment
	 */
	private MPayment updatePayment (int C_Invoice_ID, int C_BPartner_ID, 
		int C_Currency_ID, BigDecimal StmtAmt, BigDecimal TrxAmt,
		int C_BankAccount_ID, Timestamp DateTrx, Timestamp DateAcct, String Description, 
		String docStatus, String docAction )
	{

		String clientCheck = " AND AD_Client_ID=" + m_AD_Client_ID;
		
		int C_Payment_ID = 0;
		//	Trx Amount = Payment overwrites Statement Amount if defined
		BigDecimal PayAmt = TrxAmt;
		if (PayAmt == null || Env.ZERO.compareTo(PayAmt) == 0)
			PayAmt = StmtAmt;
		if (C_Invoice_ID == 0
			&& (PayAmt == null || Env.ZERO.compareTo(PayAmt) == 0))
			throw new IllegalStateException ("@PayAmt@ = 0");
		if (PayAmt == null)
			PayAmt = Env.ZERO;

		if( C_Invoice_ID != 0 ){
		
			//Import Bank Statement
			StringBuffer sql = new StringBuffer("SELECT C_Payment_ID FROM C_Payment" );
			sql.append( " WHERE C_Invoice_ID=" + C_Invoice_ID ); 
			if( C_BPartner_ID > 0 ){
				sql.append( " AND C_BPartner_ID=" + C_BPartner_ID); 
			}
			sql.append( " AND C_BankAccount_ID=" + C_BankAccount_ID); 
			if( docStatus != null ){
				sql.append( " AND docStatus='" + docStatus + "'");
			}
			if( DateTrx != null ){
				SimpleDateFormat sdf = new SimpleDateFormat( "yyyyMMdd");
				sql.append( " AND to_char(DateTrx,'YYYYMMDD')='" + sdf.format(DateTrx) + "'");
			}
			if( DateAcct != null ){
				SimpleDateFormat sdf = new SimpleDateFormat( "yyyyMMdd");
				sql.append( " AND to_char(DateAcct,'YYYYMMDD')='" + sdf.format(DateAcct) + "'");
			}
			sql.append( clientCheck );
			sql.append( " ORDER BY DateTrx DESC");
				
			PreparedStatement pstmt = null;
			try
			{
				pstmt = DB.prepareStatement(sql.toString(), get_TrxName());
				ResultSet rs = pstmt.executeQuery();
					
				while (rs.next())
				{ 
					C_Payment_ID = rs.getInt( 1 );
					break;
				}
				//	Close database connection
				rs.close();
				pstmt.close();
				rs = null;
				pstmt = null;

			}
			catch(Exception e)
			{
				log.log(Level.SEVERE, sql.toString(), e);
			}
		}
		
		if( C_Payment_ID == 0 )
			return null;
		
		//
		MPayment payment = new MPayment (getCtx(), C_Payment_ID, get_TrxName());
		payment.setC_BankAccount_ID(C_BankAccount_ID);
		payment.setTenderType(MPayment.TENDERTYPE_Check);
		if (DateTrx != null)
			payment.setDateTrx(DateTrx);
		else if (DateAcct != null)
			payment.setDateTrx(DateAcct);
		if (DateAcct != null)
			payment.setDateAcct(DateAcct);
		else
			payment.setDateAcct(payment.getDateTrx());
		payment.setDescription(Description);
		//
		if (C_Invoice_ID != 0)
		{
			MInvoice invoice = new MInvoice (getCtx(), C_Invoice_ID, null);
			payment.setC_DocType_ID(invoice.isSOTrx());		//	Receipt
			payment.setC_Invoice_ID(invoice.getC_Invoice_ID());
			payment.setC_BPartner_ID (invoice.getC_BPartner_ID());
			if (PayAmt.signum() != 0)	//	explicit Amount
			{
				payment.setC_Currency_ID(C_Currency_ID);
				if (invoice.isSOTrx())
					payment.setPayAmt(PayAmt);
				else	//	payment is likely to be negative
					payment.setPayAmt(PayAmt.negate());
				payment.setOverUnderAmt(invoice.getGrandTotal(true).subtract(payment.getPayAmt()));
			}
			else	// set Pay Amout from Invoice
			{
				payment.setC_Currency_ID(invoice.getC_Currency_ID());
				payment.setPayAmt(invoice.getGrandTotal(true));
			}
		}
		else if (C_BPartner_ID != 0)
		{
			payment.setC_BPartner_ID(C_BPartner_ID);
			payment.setC_Currency_ID(C_Currency_ID);
			if (PayAmt.signum() < 0)	//	Payment
			{
				payment.setPayAmt(PayAmt.abs());
				payment.setC_DocType_ID(false);
			}
			else	//	Receipt
			{
				payment.setPayAmt(PayAmt);
				payment.setC_DocType_ID(true);
			}
		}
		else
			return null;
		
		payment.save();

		//
		if( payment.getOverUnderAmt().intValue() != 0 ){
			payment.processIt(MPayment.DOCACTION_Prepare);
		} else {
			payment.processIt(docAction);
		}
		payment.save();
		return payment;
		
	}	//	updatePayment

	
	/**
	 * 	void Payment
	 *	@param C_Invoice_ID invoice
	 *	@param C_BPartner_ID partner ignored when invoice exists
	 *	@param C_Currency_ID currency
	 *	@param StmtAmt statement amount
	 *	@param TrxAmt transaction amt
	 *	@return payment
	 */
	private MPayment voidPayment (int C_Invoice_ID, int C_BPartner_ID, 
		int C_Currency_ID, BigDecimal StmtAmt, BigDecimal TrxAmt,
		int C_BankAccount_ID, Timestamp DateTrx, Timestamp DateAcct, String Description, 
		String docStatus )
	{

		String clientCheck = " AND AD_Client_ID=" + m_AD_Client_ID;
		
		int C_Payment_ID = 0;
		//	Trx Amount = Payment overwrites Statement Amount if defined
		BigDecimal PayAmt = TrxAmt;
		if (PayAmt == null || Env.ZERO.compareTo(PayAmt) == 0)
			PayAmt = StmtAmt;
		if (C_Invoice_ID == 0
			&& (PayAmt == null || Env.ZERO.compareTo(PayAmt) == 0))
			throw new IllegalStateException ("@PayAmt@ = 0");
		if (PayAmt == null)
			PayAmt = Env.ZERO;

		if( C_Invoice_ID != 0 ){
		
			//Import Bank Statement
			StringBuffer sql = new StringBuffer("SELECT C_Payment_ID FROM C_Payment" );
			sql.append( " WHERE C_Invoice_ID=" + C_Invoice_ID ); 
			if( C_BPartner_ID > 0 ){
				sql.append( " AND C_BPartner_ID=" + C_BPartner_ID); 
			}
			sql.append( " AND C_BankAccount_ID=" + C_BankAccount_ID); 
			if( docStatus != null ){
				sql.append( " AND docStatus='" + docStatus + "'");
			}
			if( DateTrx != null ){
				SimpleDateFormat sdf = new SimpleDateFormat( "yyyyMMdd");
				sql.append( " AND to_char(DateTrx,'YYYYMMDD')='" + sdf.format(DateTrx) + "'");
			}
			if( DateAcct != null ){
				SimpleDateFormat sdf = new SimpleDateFormat( "yyyyMMdd");
				sql.append( " AND to_char(DateAcct,'YYYYMMDD')='" + sdf.format(DateAcct) + "'");
			}
			sql.append( clientCheck );
			sql.append( " ORDER BY DateTrx DESC");
				
			PreparedStatement pstmt = null;
			try
			{
				pstmt = DB.prepareStatement(sql.toString(), get_TrxName());
				ResultSet rs = pstmt.executeQuery();
					
				while (rs.next())
				{ 
					C_Payment_ID = rs.getInt( 1 );
					break;
				}
				//	Close database connection
				rs.close();
				pstmt.close();
				rs = null;
				pstmt = null;

			}
			catch(Exception e)
			{
				log.log(Level.SEVERE, sql.toString(), e);
			}
		}
		
		if( C_Payment_ID == 0 )
			return null;
		
		//
		MPayment payment = new MPayment (getCtx(), C_Payment_ID, get_TrxName());
		payment.setC_BankAccount_ID(C_BankAccount_ID);
		payment.setTenderType(MPayment.TENDERTYPE_Check);
		if (DateTrx != null)
			payment.setDateTrx(DateTrx);
		else if (DateAcct != null)
			payment.setDateTrx(DateAcct);
		if (DateAcct != null)
			payment.setDateAcct(DateAcct);
		else
			payment.setDateAcct(payment.getDateTrx());

		payment.setDescription(Description);
		//
		if (C_Invoice_ID != 0)
		{
			MInvoice invoice = new MInvoice (getCtx(), C_Invoice_ID, null);
			payment.setC_DocType_ID(invoice.isSOTrx());		//	Receipt
			payment.setC_Invoice_ID(invoice.getC_Invoice_ID());
			payment.setC_BPartner_ID (invoice.getC_BPartner_ID());
			if (PayAmt.signum() != 0)	//	explicit Amount
			{
				payment.setC_Currency_ID(C_Currency_ID);
				if (invoice.isSOTrx())
					payment.setPayAmt(PayAmt);
				else	//	payment is likely to be negative
					payment.setPayAmt(PayAmt.negate());
				payment.setOverUnderAmt(invoice.getGrandTotal(true).subtract(payment.getPayAmt()));
			}
			else	// set Pay Amout from Invoice
			{
				payment.setC_Currency_ID(invoice.getC_Currency_ID());
				payment.setPayAmt(invoice.getGrandTotal(true));
			}
		}
		else if (C_BPartner_ID != 0)
		{
			payment.setC_BPartner_ID(C_BPartner_ID);
			payment.setC_Currency_ID(C_Currency_ID);
			if (PayAmt.signum() < 0)	//	Payment
			{
				payment.setPayAmt(PayAmt.abs());
				payment.setC_DocType_ID(false);
			}
			else	//	Receipt
			{
				payment.setPayAmt(PayAmt);
				payment.setC_DocType_ID(true);
			}
		}
		else
			return null;
		
		payment.save();

		payment.processIt( MPayment.DOCACTION_Void );
		payment.save();
		return payment;
		
	}	//	voidPayment

	private boolean sokuhoExist( X_I_BankStatement imp ){
		boolean result = false;
		
		//Import Bank Statement
		StringBuffer sql = new StringBuffer("SELECT * FROM X_DSK_BANKSTATEMENT x, I_BANKSTATEMENT i " );
		sql.append( " WHERE x.STATEMENTDATE = i.STATEMENTDATE "
				+ " AND x.BPARTNERVALUE = i.BPARTNERVALUE "
				+ " AND x.INVOICEDOCUMENTNO = i.INVOICEDOCUMENTNO "
				+ " AND (x.INVOICEDOCUMENTNO = i.INVOICEDOCUMENTNO "
				+ " OR x.MEMO = i.MEMO) "
				+ " AND x.AD_CLIENT_ID = i.AD_CLIENT_ID "
				+ " AND x.AD_ORG_ID = i.AD_ORG_ID "
				+ " AND i.BANKSTATEMENT_ID = " + imp.getI_BankStatement_ID()
				+ " AND x.REFERENCENO='01'");
			
		PreparedStatement pstmt = null;
		try
		{
			pstmt = DB.prepareStatement(sql.toString(), get_TrxName());
			ResultSet rs = pstmt.executeQuery();
				
			if (rs.next())
			{
				result = true;
			}
			//	Close database connection
			rs.close();
			pstmt.close();
			rs = null;
			pstmt = null;

		}
		catch(Exception e)
		{
			log.log(Level.SEVERE, sql.toString(), e);
		}
		
		return result;
	}
	
	
}	//	ImportBankStatement
