/******************************************************************************
 * The contents of this file are subject to the   Compiere License  Version 1.1
 * ("License"); You may not use this file except in compliance with the License
 * You may obtain a copy of the License at http://www.compiere.org/license.html
 * Software distributed under the License is distributed on an  "AS IS"  basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 * The Original Code is Compiere ERP & CRM Smart Business Solution. The Initial
 * Developer of the Original Code is Jorg Janke. Portions created by Jorg Janke
 * are Copyright (C) 1999-2005 Jorg Janke.
 * All parts are Copyright (C) 1999-2005 ComPiere, Inc.  All Rights Reserved.
 * Contributor(s): ______________________________________.
 *****************************************************************************/
package org.compiere.grid;

import java.awt.*;
import java.awt.event.*;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyVetoException;

import javax.swing.*;

import java.math.BigDecimal;
import java.util.*;
import java.util.logging.Level;
//
import org.compiere.apps.AGlassPane;
import org.compiere.grid.ed.*;
import org.compiere.model.*;
import org.compiere.swing.*;
import org.compiere.util.*;

/**
 *  Single Row Panel.
 *  Called from GridController
 *  <pre>
 *	Structure
 *		this (CPanel - Group Bag Layout)
 *			group
 *			label - field
 *
 *  Spacing:
 *  -------------------
 *  Total Top = 10+2
 *  Total Right = 0+12
 *  Total Left = 0+12
 *  Total Bottom = 3+9
 *  -------------------
 *       2
 *  12 Label 0+5 Field 0
 *       3+2=5
 *  12 Label 0+5 Field 0
 *       3
 *
 *  </pre>
 *  @author 	Jirimuto
 *  @version 	$Id: VACPanel.java,v 1.1 2006/09/25 14:19:51 hans Exp $
 */
public final class VACPanel extends CPanel 
	implements VEditor, ActionListener, KeyListener, FocusListener
{
	/**
	 *	Constructor
	 */
	public VACPanel()
	{
		super(new BorderLayout());
		setName("VACPanel");
		m_baseStackPanel.setHorizon( false );
//		m_baseStackPanel.setHeightFlexible( true );
//		m_vStackPanel.setWidthFlexible( true );
		basecomponentList.add( m_vStackPanel );
		m_baseStackPanel.setComponentList( basecomponentList );
		this.add( m_baseStackPanel );
		
	}	//	VPanel
	
	/** Orientation             */
	private final boolean       m_leftToRight = Language.getLoginLanguage().isLeftToRight();
	
	private boolean 			m_hGapAdded = false;					//	only once
	/** Previous Field Group Header     */
	private String              m_oldFieldGroup = null;
	
	private Integer             m_mValue = null;
	private Integer             m_mOldValue = null;
	private Integer             m_mInitValue = null;
	private boolean				m_setting;
	private boolean				m_modified = false;
	private boolean				m_justFocusIn = false;
	private VEditor 			m_firstFocusable = null;
	
	private VStackPanel m_baseStackPanel = new VStackPanel();
	private VStackPanel m_vStackPanel = new VStackPanel();

	ArrayList<Component> basecomponentList = new ArrayList<Component>();
	ArrayList<Component> componentList = new ArrayList<Component>();
	
	/**	Logger	*/
	private static CLogger log = CLogger.getCLogger (VACPanel.class);
	
	public void setRowAndCol( int row, int col ) {
		
		m_vStackPanel.setComponentList( componentList );
		
		for( int i=1; i<=row; i++ ){
			
			VStackPanel tmp_vStackPanel = new VStackPanel();
			ArrayList<Component> tmpcomponentList = new ArrayList<Component>();
			for( int j=1; j<=col; j++) {
				tmpcomponentList.add((Component)new JLabel(""));
			}
			tmp_vStackPanel.setComponentList( tmpcomponentList );
			tmp_vStackPanel.setStacked( false );
			
			basecomponentList.add( tmp_vStackPanel );
		}
	}
	
	public void prepareComplete( int colcount ) {
		
		if( colcount > 0 ){
			double[] colsize_max = new double[colcount];
			for( int i=1; i<basecomponentList.size(); i++ ){
				VStackPanel tmp_vStackPanel = (VStackPanel)basecomponentList.get(i);
				ArrayList<Component> tmpcomponentList = tmp_vStackPanel.getComponentList();
				for(int j=0; j<tmpcomponentList.size(); j++ ){
					Component component = (Component)tmpcomponentList.get(j);
					if( component != null ) {
						if( (j+1)<tmpcomponentList.size() ){
							Component next_component = (Component)tmpcomponentList.get(j+1);
							if( next_component != null ){
								Dimension tmpSize = component.getSize();
								if( colsize_max[j] < tmpSize.getWidth() ) {
									colsize_max[j] = tmpSize.getWidth();
								}
							}
						} else {
							Dimension tmpSize = component.getSize();
							if( colsize_max[j] < tmpSize.getWidth() ) {
								colsize_max[j] = tmpSize.getWidth();
							}
						}
					}
				}
			}

			for( int i=1; i<basecomponentList.size(); i++ ){
				VStackPanel tmp_vStackPanel = (VStackPanel)basecomponentList.get(i);
				ArrayList<Component> tmpcomponentList = tmp_vStackPanel.getComponentList();
				for(int j=0; j<tmpcomponentList.size(); j++ ){
					Component component = (Component)tmpcomponentList.get(j);
					if( component != null ){
						if( (j+1)<tmpcomponentList.size() ){
							Component next_component = (Component)tmpcomponentList.get(j+1);
							if( next_component != null ){
								Dimension tmpSize = component.getSize();
								if( colsize_max[j] > tmpSize.getWidth() ) {
									tmpSize.setSize(colsize_max[j], tmpSize.getHeight());
									component.setSize( tmpSize );
									component.setPreferredSize( tmpSize );
								}
							}
						} else {
							Dimension tmpSize = component.getSize();
							if( colsize_max[j] > tmpSize.getWidth() ) {
								tmpSize.setSize(colsize_max[j], tmpSize.getHeight());
								component.setSize( tmpSize );
								component.setPreferredSize( tmpSize );
							}
						}
					}
				}
			}
		}
		
		for( int i=0; i<basecomponentList.size(); i++ ){
			VStackPanel tmp_vStackPanel = (VStackPanel)basecomponentList.get(i);
			ArrayList<Component> tmpcomponentList = tmp_vStackPanel.getComponentList();
			tmp_vStackPanel.setComponentList( tmpcomponentList );
		}
		m_baseStackPanel.setComponentList( basecomponentList );
		
		this.setSize( m_baseStackPanel.getSize( ) );
		this.setPreferredSize( m_baseStackPanel.getPreferredSize( ) );
		
	}

	/**
	 *	Add Component vertically to Panel
	 *  @param mField field model
	 */
	public void addComponentVert ( Component component )
	{
		basecomponentList.add( component );
		m_baseStackPanel.setComponentList( basecomponentList );
	}	
	/**
	 *	Add Component vertically to Panel
	 *  @param mField field model
	 */
	public void addComponentHorz ( Component component )
	{
		componentList.add( component );
		m_vStackPanel.setComponentList( componentList );
	}	
	/**
	 *	Add Field and Label to Panel
	 *  @param editor editor
	 *  @param mField field model
	 */
	public void addField (VEditor editor, GridField mField)
	{
		CLabel label = VEditorFactory.getLabel(mField);
		if (label == null && editor == null)
			return;
		
		boolean sameLine = mField.isSameLine();
		if (addGroup(mField.getFieldGroup()))               		//	sets top
			sameLine = false;

		if ( !sameLine  ) {    							//	Set line #
						
			m_vStackPanel = new VStackPanel();
//			m_vStackPanel.setWidthFlexible( true );
			componentList = new ArrayList<Component>();
			
			basecomponentList.add( m_vStackPanel );
			
		}

		//	*** The Label ***
		if (label != null)
		{
			Dimension labelDimension = new Dimension( 100, 16 );
			label.setPreferredSize( labelDimension );
			label.setSize( labelDimension);
			//
			if (mField.isCreateMnemonic())
				setMnemonic(label, mField.getMnemonic());
			//  Add Label
			componentList.add(label);
		} else {
			JLabel tmpLabel = new JLabel("");
			Dimension labelDimension = new Dimension( 100, 16 );
			tmpLabel.setPreferredSize( labelDimension );
			tmpLabel.setSize( labelDimension);
			//  Add Label
			componentList.add(tmpLabel);
		}

		//	*** The Field ***
		if (editor != null)
		{
			Component field = (Component)editor;
			Dimension fieldDimension = new Dimension( 300, 20 );
			Dimension preferedDimension = new Dimension( 200, 20 );
			
			if( DisplayType.Button == mField.getDisplayType() ) {
				if( mField.isLongField() ){
					fieldDimension.setSize( 704, 32);
					preferedDimension.setSize( 504, 32 );
				} else {
					fieldDimension.setSize( 300, 32);
					preferedDimension.setSize( 200, 32 );
				}
			} else if ( DisplayType.Text == mField.getDisplayType() 
					|| DisplayType.TextLong == mField.getDisplayType() ){
				if( mField.isLongField() ) {
					fieldDimension.setSize( 704, 60);
					preferedDimension.setSize( 504, 60 );
				} else {
					fieldDimension.setSize( 300, 60);
					preferedDimension.setSize( 200, 60 );
				}
			}
			
			field.setPreferredSize(fieldDimension);
			field.setSize(fieldDimension);
			
			//	Link Label to Field
			if (label != null)
				label.setLabelFor(field);
			else if (mField.isCreateMnemonic())
				setMnemonic(editor, mField.getMnemonic());
			//	Add Field
			componentList.add(field);
			// editor.addActionListener( this );			
			if( m_firstFocusable == null && editor.isReadWrite() ){
				m_firstFocusable = editor;
			}
			
		}
		
		m_vStackPanel.setComponentList( componentList );
		
		m_baseStackPanel.setComponentList( basecomponentList );
		
	}	//	addField

	/**
	 *	Add Field and Label to Panel with Format
	 *  @param editor editor
	 *  @param mField field model
	 */
	public void addFieldWithFormat (VEditor editor, GridField mField, M_FieldFormat mFieldFormat )
	{
		Dimension zerosize = new Dimension( 0, 0 );
		CLabel label = VEditorFactory.getLabel(mField);
		if (label == null && editor == null)
			return;
		
		//	*** The Label ***
		if (label != null)
		{
			if( mFieldFormat != null ){
				
				int colwidth = mFieldFormat.getLabel_ColWidth();
				Dimension labelDimension = new Dimension( colwidth, 12 );
				
				if( DisplayType.Button == mField.getDisplayType() ) {
					labelDimension.setSize( colwidth, 24);
				} else if ( DisplayType.Text == mField.getDisplayType() 
						|| DisplayType.TextLong == mField.getDisplayType() ){
					labelDimension.setSize( colwidth, 12);
				}
				
				if ( mFieldFormat.getLabel_RowNo()==mFieldFormat.getField_RowNo() 
						&& mFieldFormat.getLabel_ColNo()==mFieldFormat.getField_ColNo()-1){
					label.setPreferredSize( labelDimension );
					label.setSize( labelDimension);
					label.setHorizontalAlignment( SwingConstants.RIGHT );
				}else {
					label.setPreferredSize( labelDimension );
					label.setSize( labelDimension);
					label.setHorizontalAlignment( SwingConstants.LEFT );
				}
				//
				if (mField.isCreateMnemonic())
					setMnemonic(label, mField.getMnemonic());
				
				VStackPanel tmpStackPanel = null;
				if( mFieldFormat.getLabel_RowNo()>0 
						&& mFieldFormat.getLabel_RowNo() <= basecomponentList.size() ){
					tmpStackPanel = (VStackPanel)basecomponentList.get( mFieldFormat.getLabel_RowNo() );
				}
				
				if( tmpStackPanel != null ){
					//  Add Label
					ArrayList<Component> tmpComponentList =  tmpStackPanel.getComponentList();
					if( mFieldFormat.getLabel_ColNo()>0 
							&& mFieldFormat.getLabel_ColNo() <= tmpComponentList.size() ){
						tmpComponentList.remove( mFieldFormat.getLabel_ColNo()-1 );
						tmpComponentList.add(mFieldFormat.getLabel_ColNo()-1, label);
						int colcount = mFieldFormat.getLabel_ColCount();
						int next_col = mFieldFormat.getLabel_ColNo();
						while( colcount > 1 && next_col < tmpComponentList.size() ) {
							tmpComponentList.remove( next_col );
							tmpComponentList.add( next_col, (Component)null );
							next_col++;
							colcount--;
						}
					}
				}
			}
		}

		//	*** The Field ***
		if (editor != null)
		{
			Component field = (Component)editor;
			if( mFieldFormat != null ) {
				int colwidth = mFieldFormat.getField_ColWidth();
				Dimension fieldDimension = new Dimension( colwidth, 18 );
				
				if( DisplayType.Button == mField.getDisplayType() ) {
						fieldDimension.setSize( colwidth, 24);
				} else if ( DisplayType.Text == mField.getDisplayType() 
						|| DisplayType.TextLong == mField.getDisplayType() ){
					fieldDimension.setSize( colwidth, 36);
				}
				
				field.setPreferredSize(fieldDimension);
				field.setSize(fieldDimension);
	
				VStackPanel tmpStackPanel = null;
				if( mFieldFormat.getField_RowNo()>0 
						&& mFieldFormat.getField_RowNo() <= basecomponentList.size() ){
					tmpStackPanel = (VStackPanel)basecomponentList.get(mFieldFormat.getField_RowNo());
				}
				
				if( tmpStackPanel != null ){
					//  Add Field
					ArrayList<Component> tmpComponentList =  tmpStackPanel.getComponentList();
					if( mFieldFormat.getField_ColNo()>0 
							&& mFieldFormat.getField_ColNo() <= tmpComponentList.size() ){
						tmpComponentList.remove( mFieldFormat.getField_ColNo()-1 );
						tmpComponentList.add(mFieldFormat.getField_ColNo()-1, field);
						int colcount = mFieldFormat.getField_ColCount();
						int next_col = mFieldFormat.getField_ColNo();
						while( colcount > 1 && next_col < tmpComponentList.size() ) {
							tmpComponentList.remove( next_col );
							tmpComponentList.add( next_col, (Component)null );
							next_col++;
							colcount--;
						}
						// editor.addActionListener( this );			
						if( m_firstFocusable == null && editor.isReadWrite() ){
							m_firstFocusable = editor;
						}
					}
				}
				//	Link Label to Field
				if (label != null){
					label.setLabelFor(field);
					if( field instanceof VCellRendererEditor ){
						((VCellRendererEditor)field).setM_Label( label );
					}
				}
				else if (mField.isCreateMnemonic())
					setMnemonic(editor, mField.getMnemonic());
			}
		}
		
		m_vStackPanel.setComponentList( componentList );
		
		m_baseStackPanel.setComponentList( basecomponentList );
		
	}	//	addField

	/**
	 *	Add Field and Label to Panel with Format
	 *  @param editor editor
	 *  @param mField field model
	 */
	public void addACGridController (ACGridController multiline_gc, M_TabFormat m_TabFormat )
	{
		Dimension zerosize = new Dimension( 0, 0 );
		
		if( multiline_gc != null ) {
			VStackPanel tmpStackPanel = null;
			if( m_TabFormat.getInParent_RowNo()>0 
					&& m_TabFormat.getInParent_RowNo() <= basecomponentList.size() ){
				tmpStackPanel = (VStackPanel)basecomponentList.get( m_TabFormat.getInParent_RowNo() );
			}
			
			if( tmpStackPanel != null ){
				//  Add Label
				ArrayList<Component> tmpComponentList =  tmpStackPanel.getComponentList();
				if( m_TabFormat.getInParent_ColNo()>0 
						&& m_TabFormat.getInParent_ColNo() <= tmpComponentList.size() ){
					tmpComponentList.remove( m_TabFormat.getInParent_ColNo()-1 );
					tmpComponentList.add(m_TabFormat.getInParent_ColNo()-1, multiline_gc);
					int colcount = m_TabFormat.getInParent_ColCount();
					int next_col = m_TabFormat.getInParent_ColNo();
					while( colcount > 1 && next_col < tmpComponentList.size() ) {
						tmpComponentList.remove( next_col );
						tmpComponentList.add( next_col, (Component)null );
						next_col++;
						colcount--;
					}
					Dimension gridsize = new Dimension( m_TabFormat.getTab_Width(), m_TabFormat.getTab_Height() );
					multiline_gc.setSize( gridsize );
					multiline_gc.setPreferredSize( gridsize );
					
				}
				
			}
			
		}
		
		m_vStackPanel.setComponentList( componentList );
		
		m_baseStackPanel.setComponentList( basecomponentList );
		
	}	//	addField

	/**
	 * 	Set Field Mnemonic
	 *	@param mField field
	 */
	public void setMnemonic (GridField mField)
	{
		if (mField.isCreateMnemonic())
			return;
		String text = mField.getHeader();
		int pos = text.indexOf("&");
		if (pos != -1 && text.length() > pos)	//	We have a nemonic - creates Ctrl_Shift_
		{
			char mnemonic = text.toUpperCase().charAt(pos+1);
			if (mnemonic != ' ')
			{
				if (!m_mnemonics.contains(mnemonic))
				{
					mField.setMnemonic(mnemonic);
					m_mnemonics.add(mnemonic);
				}
				else
					log.warning(mField.getColumnName() 
						+ " - Conflict - Already exists: " + mnemonic + " (" + text + ")");
			}
		}
	}	//	setMnemonic
	
	/**
	 *	Add Group
	 *  @param fieldGroup field group
	 *  @return true if group added
	 */
	private boolean addGroup(String fieldGroup)
	{
		//	First time - add top
		if (m_oldFieldGroup == null)
		{
			addTop();
			m_oldFieldGroup = "";
		}

		if (fieldGroup == null || fieldGroup.length() == 0 || fieldGroup.equals(m_oldFieldGroup))
			return false;
		m_oldFieldGroup = fieldGroup;

		return true;
		
	}	//	addGroup

	/**
	 *	Add Top (10) and right (12) gap
	 */
	private void addTop()
	{
	}	//	addTop

	/**
	 *	Add End (9) of Form
	 */
	public void addEnd()
	{
	}	//	addEnd

	/**
	 * 	Set Mnemonic for Label CTRL_SHIFT_x
	 *	@param label label
	 *	@param predefinedMnemonic predefined Mnemonic
	 */
	private void setMnemonic (CLabel label, char predefinedMnemonic)
	{
		String text = label.getText();
		int pos = text.indexOf("&");
		if (pos != -1 && predefinedMnemonic != 0)
		{
			text = text.substring(0, pos) + text.substring(pos+1);
			label.setText(text);
			label.setSavedMnemonic(predefinedMnemonic);
			m_fields.add(label);
			log.finest(predefinedMnemonic + " - " + label.getName());
		}
		else
		{
			char mnemonic = getMnemonic(text, label);
			if (mnemonic != 0)
				label.setSavedMnemonic(mnemonic);
		//	label.setDisplayedMnemonic(mnemonic);
		}
	}	//	setMnemonic
	
	/**
	 * 	Set Mnemonic for Check Box or Button
	 *	@param editor check box or button - other ignored
	 *	@param predefinedMnemonic predefined Mnemonic
	 */
	private void setMnemonic (VEditor editor, char predefinedMnemonic)
	{
		if (editor instanceof VCheckBox)
		{
			VCheckBox cb = (VCheckBox)editor;
			String text = cb.getText();
			int pos = text.indexOf("&");
			if (pos != -1 && predefinedMnemonic != 0)
			{
				text = text.substring(0, pos) + text.substring(pos+1);
				cb.setText(text);
				cb.setSavedMnemonic(predefinedMnemonic);
				m_fields.add(cb);
				log.finest(predefinedMnemonic + " - " + cb.getName());
			}
			else
			{
				char mnemonic = getMnemonic(text, cb);
				if (mnemonic != 0)
					cb.setSavedMnemonic(mnemonic);
			//	cb.setMnemonic(mnemonic);
			}
		}
		//	Button
		else if (editor instanceof VButton)
		{
			VButton b = (VButton)editor;
			String text = b.getText();
			int pos = text.indexOf("&");
			if (pos != -1 && predefinedMnemonic != 0)
			{
				text = text.substring(0, pos) + text.substring(pos+1);
				b.setText(text);
				b.setSavedMnemonic(predefinedMnemonic);
				m_fields.add(b);
				log.finest(predefinedMnemonic + " - " + b.getName());
			}
			else if (b.getColumnName().equals("DocAction"))
			{
				b.getInputMap(WHEN_IN_FOCUSED_WINDOW)
					.put(KeyStroke.getKeyStroke(KeyEvent.VK_F4, Event.SHIFT_MASK, false), "pressed");
				b.getInputMap(WHEN_IN_FOCUSED_WINDOW)
					.put(KeyStroke.getKeyStroke(KeyEvent.VK_F4, Event.SHIFT_MASK, true), "released");
			//	Util.printActionInputMap(b);
			}
			else if (b.getColumnName().equals("Posted"))
			{
				b.getInputMap(WHEN_IN_FOCUSED_WINDOW)
					.put(KeyStroke.getKeyStroke(KeyEvent.VK_F5, Event.SHIFT_MASK, false), "pressed");
				b.getInputMap(WHEN_IN_FOCUSED_WINDOW)
					.put(KeyStroke.getKeyStroke(KeyEvent.VK_F5, Event.SHIFT_MASK, true), "released");
			//	Util.printActionInputMap(b);
			}
			else
			{
				char mnemonic = getMnemonic(text, b);
				if (mnemonic != 0)
					b.setSavedMnemonic(mnemonic);
			}
		}
	}	//	setMnemonic

	/**
	 * 	Get Mnemonic from text
	 *	@param text text
	 *	@return Mnemonic or 0 if not unique
	 */
	private char getMnemonic (String text, Component source)
	{
		if (text == null || text.length() == 0)
			return 0;
		String oText = text;
		text = text.trim().toUpperCase();
		char mnemonic = text.charAt(0);
		if (m_mnemonics.contains(mnemonic))
		{
			mnemonic = 0;
			//	Beginning new word
			int index = text.indexOf(' ');
			while (index != -1 && text.length() > index)
			{
				char c = text.charAt(index+1);
				if (Character.isLetterOrDigit(c) && !m_mnemonics.contains(c))
				{
					mnemonic = c;
					break;
				}
				index = text.indexOf(' ', index+1);
			}
			//	Any character
			if (mnemonic == 0)
			{
				for (int i = 1; i < text.length(); i++)
				{
					char c = text.charAt(i);
					if (Character.isLetterOrDigit(c) && !m_mnemonics.contains(c))
					{
						mnemonic = c;
						break;
					}
				}
			}
			//	Nothing found
			if (mnemonic == 0)
			{
				log.finest("None for: " + oText);
				return 0;	//	 if first char would be returned, the first occurance is invalid.
			}
		}
		m_mnemonics.add(mnemonic);
		m_fields.add(source);
		log.finest(mnemonic + " - " + source.getName());
		return mnemonic;
	}	//	getMnemonic
	
	/** Used Mnemonics		*/
	private ArrayList<Character> m_mnemonics = new ArrayList<Character>(30);
	/** Mnemonic Fields		*/
	private ArrayList<Component> m_fields = new ArrayList<Component>(30);
	
	/**
	 * 	Set Window level Mnemonics
	 *	@param set true if set otherwise unregiser
	 */
	public void setMnemonics (boolean set)
	{
		int size = m_fields.size();
		for (int i = 0; i < size; i++)
		{
			Component c = m_fields.get(i);
			if (c instanceof CLabel)
			{
				CLabel l = (CLabel)c;
				if (set)
					l.setDisplayedMnemonic(l.getSavedMnemonic());
				else
					l.setDisplayedMnemonic(0);
			}
			else if (c instanceof VCheckBox)
			{
				VCheckBox cb = (VCheckBox)c;
				if (set)
					cb.setMnemonic(cb.getSavedMnemonic());
				else
					cb.setMnemonic(0);
			}
			else if (c instanceof VButton)
			{
				VButton b = (VButton)c;
				if (set)
					b.setMnemonic(b.getSavedMnemonic());
				else
					b.setMnemonic(0);
			}
		}
	}	//	setMnemonics
	
	/**************************************************************************
	 *  Set Background to AD_Color_ID (nop)
	 *  @param AD_Color_ID Color
	 */
	public void setBackground (int AD_Color_ID)
	{
	}   //  setBackground
	
	/**
	 *	Set Editor to value
	 *  @param value value
	 */
	public void setValue(Object value)
	{
		log.finest("Value=" + value);
		if( value instanceof Integer ){
			m_mOldValue = (Integer)value;
			//	only set when not updated here
			if (m_setting)
				return;
			m_mValue = m_mOldValue;
			m_mInitValue = m_mOldValue;
			m_modified = false;
		}
	}	//	setValue

	/**
	 *	Return Editor value
	 *  @return value value (big decimal or integer)
	 */
	public Object getValue()
	{
		return m_mValue;
	}	//	getValue

	public void actionPerformed(ActionEvent e) {
		log.config(e.getActionCommand());
		
		System.out.println("VACPanel : Action="+e.getActionCommand());
	}

	public void addActionListener(ActionListener listener) {
		// TODO Auto-generated method stub
		
	}

	public void dispose() {
		m_mValue = null;
		m_baseStackPanel.removeAll();
		m_vStackPanel.removeAll();
	}

	public void setField(GridField mField) {
	}

	public String getDisplay() {
		return null;
	}

	public boolean isMandatory() {
		return false;
	}

	public boolean isReadWrite() {
		return false;
	}

	public void setBackground(boolean error) {
	}

	public void setMandatory(boolean mandatory) {
	}

	public void setReadWrite(boolean rw) {
	}

	public void focusGained(FocusEvent e) {
		m_justFocusIn = true;
		
	}

	public void focusLost(FocusEvent e) {
		m_justFocusIn = false;
		if (e.getOppositeComponent() instanceof AGlassPane)
		{
			m_mValue = m_mInitValue;
			return;
		}
	}

	public void keyPressed(KeyEvent e) {}
	public void keyTyped(KeyEvent e) {}

	public void keyReleased(KeyEvent e) {
		log.finest("Key=" + e.getKeyCode() + " - " + e.getKeyChar());

		//  ESC
		if (e.getKeyCode() == KeyEvent.VK_ESCAPE)
			m_mValue = m_mInitValue;
		m_modified = true;
		m_setting = true;
		if (e.getKeyCode() == KeyEvent.VK_ENTER)	//	10
		{
			/**
			 * Jirimuto Add For Focus Forward 
			 */
			if( !m_justFocusIn ){
				((Component)e.getSource()).transferFocus();
			}
		}
		m_setting = false;
		m_justFocusIn = false;
	}

	public void propertyChange(PropertyChangeEvent evt) {
		
	}

	public VEditor getFirstFocusable() {
		return m_firstFocusable;
	}

	public void setFirstFocusable(VEditor focusable) {
		m_firstFocusable = focusable;
	}
	
	/**
	 * 	Request Focus
	 */
	public void requestFocus ()
	{
		if( m_firstFocusable != null ){
			m_firstFocusable.requestFocus ();
		}
	}	//	requestFocus
	

}	//	VACPanel
