package org.compiere.grid;

import java.awt.Component;
import java.awt.Dimension;
import java.util.ArrayList;

import javax.swing.JPanel;

import org.compiere.swing.CPanel;

/**
 * │┌┐┘└├┬┤┴┼─
 * 
 * the layout is below
 * ┌─────────────┬────────────┬─────────────┐
 * │┌───────────┐│┌──────────┐│┌───────────┐│
 * ││JComponent │││JComponent│││JComponent ││
 * │└───────────┘│└──────────┘│└───────────┘│
 * └─────────────┴────────────┴─────────────┘
 * or
 * ┌─────────────┐
 * │┌───────────┐│
 * ││JComponent ││
 * │└───────────┘│
 * ├─────────────┤
 * │┌───────────┐│
 * ││JComponent ││
 * │└───────────┘│
 * ├─────────────┤
 * │┌───────────┐│
 * ││JComponent ││
 * │└───────────┘│
 * └─────────────┘
 * 
 * @author bato
 * @version $Id: VStackPanel.java,v 1.1 2006/09/25 14:19:51 hans Exp $
 */

public class VStackPanel extends CPanel {
	
	private static final long serialVersionUID = 1L;
	private boolean isStacked = true; // by default is stacked 
	private int marginLeft = 2; // margin left
	private int marginRight = 2; // margin right
	private int marginTop = 2; // margin top
	private int marginBottom = 2; // margin bottom
	private int padding = 2; // padding between component
	private ArrayList<Component> componentList = new ArrayList<Component>();
	private boolean horizon = true; // by default layout is horizontal
	private int panalOrgWidth = 0; // Original width;
	private int panalOrgHeight = 0; // Original height;
	private boolean widthFlexible = false; // by default width is fixed
	private boolean heightFlexible = false; // by default heihgt is fixed 
	private ArrayList<Integer> widthOrgList = new ArrayList<Integer>(); // Original width list
	private ArrayList<Integer> heightOrgList = new ArrayList<Integer>(); // Original height list
	private boolean initOk = false;
	private String fixedWidthComponents = null;
	private ArrayList<Integer> fixedComps = new ArrayList<Integer>();
	
	@Override
	public void setBounds(int x, int y, int width, int height) {
	
		if (widthFlexible && initOk){
			int elementNo = 0;
			int margin = marginLeft + marginRight;
			int vldWidth = panalOrgWidth - margin;
			int xPosition = marginLeft , yPosition = marginTop;
			int fixedW = 0;
			int pad = (padding*(componentList.size()-1));
			for (Integer item : fixedComps) {
				fixedW += widthOrgList.get(item);
			}
			for (Component component : componentList) {
				if( component != null ){
					int cmpWidth = widthOrgList.get(elementNo);
					if (horizon){
						int cmpNewWidth = (int)((width-margin-pad-fixedW)*((double)cmpWidth/(vldWidth-pad-fixedW)));
						if(componentList.size()==(elementNo+1)){//if the last component
							cmpNewWidth = width - xPosition - marginRight;
						}
						if (fixedComps.contains(elementNo)){
							cmpNewWidth = cmpWidth;
						}
						component.setBounds(
								xPosition, yPosition, 
								cmpNewWidth, component.getPreferredSize().height);
						xPosition += cmpNewWidth + padding;
					}else{
						int cmpNewWidth = (int)((width-margin)*((double)cmpWidth/(double)vldWidth));
						if (fixedComps.contains(elementNo)){
							cmpNewWidth = cmpWidth;
						}
						component.setSize(
								cmpNewWidth, component.getPreferredSize().height);
					}
					elementNo++;
				}
			}
		}
		
		if (heightFlexible && initOk){
			int elementNo = 0;
			int margin = marginTop + marginBottom;
			int vldHeight = panalOrgHeight - margin;
			int xPosition = marginLeft , yPosition = marginTop;
			int fixedH = 0;
			int pad = (padding*(componentList.size()-1));
			for (Integer item : fixedComps) {
				fixedH += heightOrgList.get(item);
			}
			for (Component component : componentList) {
				if( component != null ){
					int cmpHeight = heightOrgList.get(elementNo);
					if (!horizon){
						int cmpNewHeight = (int)((height-margin-pad-fixedH)*((double)cmpHeight/(vldHeight-pad-fixedH)));
						if(componentList.size()==(elementNo+1)){//if the last component
							cmpNewHeight = height - yPosition - marginRight;
						}
						if (fixedComps.contains(elementNo)){
							cmpNewHeight = cmpHeight;
						}
						component.setBounds(
								xPosition, yPosition, 
								component.getWidth(), cmpNewHeight);
						yPosition += cmpNewHeight + padding;
					}else{
						int cmpNewHeight = (int)((height-margin)*((double)cmpHeight/(double)vldHeight));
						if (fixedComps.contains(elementNo)){
							cmpNewHeight = cmpHeight;
						}
						component.setSize(
								component.getWidth(),cmpNewHeight);
					}
					elementNo++;
				}
			}
		}
		
		super.setBounds(x, y, width, height);
		
	}

	/**
	 * @return Returns the components.
	 */
	public ArrayList<Component> getComponentList() {
		return componentList;
	}

	/**
	 * @param components The components to set.
	 */
	public void setComponentList(ArrayList<Component> components) {
		this.componentList = components;
		this.setLayout(null);
		int xPosition = marginLeft , yPosition = marginTop;
		widthOrgList.clear();
		heightOrgList.clear();
		panalOrgHeight = 0;
		panalOrgWidth = 0;
		for (Component component : componentList) {
			if( component != null ){
				component.setBounds(xPosition, yPosition, 
						component.getWidth(), component.getHeight());
				component.setPreferredSize(
						new Dimension(component.getWidth(), component.getHeight()));
				widthOrgList.add(component.getWidth());
				heightOrgList.add(component.getHeight());
				this.add(component);
				if (horizon){
					xPosition += component.getWidth()  + padding;
					if (panalOrgHeight < component.getHeight()){
						panalOrgHeight = component.getHeight()+(marginTop + marginBottom);
					}
				}else{
					yPosition += component.getHeight() + padding;
					if (panalOrgWidth < component.getWidth()){
						panalOrgWidth = component.getWidth()+(marginLeft + marginRight);
					}
				}
			}
		}
		if (horizon){
			panalOrgWidth = xPosition - padding + marginRight;
		}else{
			panalOrgHeight = yPosition - padding + marginBottom;
		}
		this.setSize(panalOrgWidth, panalOrgHeight);
		this.setPreferredSize(new Dimension(panalOrgWidth, panalOrgHeight));
//		this.setBorder(javax.swing.BorderFactory.createLineBorder(java.awt.Color.black,1));
		initOk = true;
	}

	/**
	 * for spring framework
	 * @param d
	 */
	public void setSize2(Dimension d) {
		super.setSize(d);
	}
	
	/**
	 * for spring framework
	 * @param d
	 */
	public Dimension getSize2() {
		return super.getSize();
	}

	/**
	 * @return Returns the horizon.
	 */
	public boolean isHorizon() {
		return horizon;
	}

	/**
	 * @param horizon The horizon to set.
	 */
	public void setHorizon(boolean horizon) {
		this.horizon = horizon;
	}

	/**
	 * @return Returns the marginBottom.
	 */
	public int getMarginBottom() {
		return marginBottom;
	}

	/**
	 * @param marginBottom The marginBottom to set.
	 */
	public void setMarginBottom(int marginBottom) {
		this.marginBottom = marginBottom;
	}

	/**
	 * @return Returns the marginLeft.
	 */
	public int getMarginLeft() {
		return marginLeft;
	}

	/**
	 * @param marginLeft The marginLeft to set.
	 */
	public void setMarginLeft(int marginLeft) {
		this.marginLeft = marginLeft;
	}

	/**
	 * @return Returns the marginRight.
	 */
	public int getMarginRight() {
		return marginRight;
	}

	/**
	 * @param marginRight The marginRight to set.
	 */
	public void setMarginRight(int marginRight) {
		this.marginRight = marginRight;
	}

	/**
	 * @return Returns the marginTop.
	 */
	public int getMarginTop() {
		return marginTop;
	}

	/**
	 * @param marginTop The marginTop to set.
	 */
	public void setMarginTop(int marginTop) {
		this.marginTop = marginTop;
	}

	/**
	 * @return Returns the padding.
	 */
	public int getPadding() {
		return padding;
	}

	/**
	 * @param padding The padding to set.
	 */
	public void setPadding(int padding) {
		this.padding = padding;
	}

	/**
	 * @return Returns the fixedWidthComponents.
	 */
	public String getFixedWidthComponents() {
		return fixedWidthComponents;
	}

	/**
	 * it like something  1,4,5
	 * @param fixedWidthComponents The fixedWidthComponents to set.
	 */
	public void setFixedWidthComponents(String fWComponents) {

		String[] fixedWItems = fWComponents.split(",");
		for (int i = 0; i < fixedWItems.length; i++) {
			Integer item = Integer.valueOf(fixedWItems[i]);
			fixedComps.add(item);
		}

		this.fixedWidthComponents = fWComponents;
	}

	/**
	 * @return Returns the heightFlexible.
	 */
	public boolean isHeightFlexible() {
		return heightFlexible;
	}

	/**
	 * @param heightFlexible The heightFlexible to set.
	 */
	public void setHeightFlexible(boolean heightFlexible) {
		this.heightFlexible = heightFlexible;
	}

	/**
	 * @return Returns the widthFlexible.
	 */
	public boolean isWidthFlexible() {
		return widthFlexible;
	}

	/**
	 * @param widthFlexible The widthFlexible to set.
	 */
	public void setWidthFlexible(boolean widthFlexible) {
		this.widthFlexible = widthFlexible;
	}

	public boolean isStacked() {
		return isStacked;
	}

	public void setStacked(boolean isStacked) {
		this.isStacked = isStacked;
	}

}
