/******************************************************************************
 * The contents of this file are subject to the   Compiere License  Version 1.1
 * ("License"); You may not use this file except in compliance with the License
 * You may obtain a copy of the License at http://www.compiere.org/license.html
 * Software distributed under the License is distributed on an  "AS IS"  basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 * The Original Code is Compiere ERP & CRM Smart Business Solution. The Initial
 * Developer of the Original Code is Jorg Janke. Portions created by Jorg Janke
 * are Copyright (C) 1999-2005 Jorg Janke.
 * All parts are Copyright (C) 1999-2005 ComPiere, Inc.  All Rights Reserved.
 * Contributor(s): ______________________________________.
 *****************************************************************************/
package org.compiere.grid.ed;

import java.awt.*;
import java.awt.event.*;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.Properties;
import java.util.logging.*;

import javax.swing.*;
import org.compiere.apps.*;
import org.compiere.model.*;
import org.compiere.plaf.*;
import org.compiere.swing.*;
import org.compiere.util.*;

/**
 *	Business Partner Editor.
 *	Set BPartner: loadBPartner
 *	Get result: getC_BPartner_ID
 *
 *  @author 	Jorg Janke
 *  @version 	$Id: VBPartner.java,v 1.1 2006/09/25 14:19:56 hans Exp $
 */
public final class VBPartner extends CDialog implements ActionListener, FocusListener
{
	/**
	 *	Constructor.
	 *	Requires call loadBPartner
	 * 	@param frame	parent
	 * 	@param WindowNo	Window No
	 */
	public VBPartner(Frame frame, int WindowNo)
	{
		super(frame, Msg.translate(Env.getCtx(), "C_BPartner_ID"), true);
		m_WindowNo = WindowNo;
		m_readOnly = !MRole.getDefault().canUpdate(
			Env.getAD_Client_ID(Env.getCtx()), Env.getAD_Org_ID(Env.getCtx()), 
			MBPartner.Table_ID, 0, false);
		log.info("R/O=" + m_readOnly);
		try
		{
			jbInit();
		}
		catch(Exception ex)
		{
			log.log(Level.SEVERE, ex.getMessage());
		}
		initBPartner();
		//
		AEnv.positionCenterWindow(frame, this);
	}	//	VBPartner

	private int 			m_WindowNo;
	/** The Partner				*/
	private MBPartner		m_partner = null;
	/** The Location			*/
	private MBPartnerLocation	m_pLocation = null;
	/** The User				*/
	private MUser			m_user = null;
	/** Read Only				*/
	private boolean			m_readOnly = false;

	
	private Insets			m_labelInsets = new Insets(2,15,2,0);		// 	top,left,bottom,right
	private Insets			m_fieldInsets = new Insets(2,5,2,10);		// 	top,left,bottom,right
	private GridBagConstraints m_gbc = new GridBagConstraints();
	private int				m_line;
	private Object[]		m_greeting;
	private Object[]		m_sex;
	private Object[]		m_locationType;
	
	/**	Logger			*/
	private static CLogger log = CLogger.getCLogger(VBPartner.class);
	//
	private VString	fValue, fName, fName2, fContact, fTitle, fPhone, fFax, fPhone2, fEMail;
	private VLocation 		fAddress;
	private JComboBox 		fGreetingBP, fGreetingC;

	private VString			fSector, fDescription, fFirstName, fLastNameKana, fFirstNameKana, fAddressKana, fPhone3;
	private CComboBox 		fContactSex, fLocationType;
	private VDate	 		fBirthDate;
	private VCheckBox		fIsBPCompany, fIsUnsubscribed;
	//
	private CPanel mainPanel = new CPanel();
	private BorderLayout mainLayout = new BorderLayout();
	private CPanel centerPanel = new CPanel();
	private CPanel southPanel = new CPanel();
	private GridBagLayout centerLayout = new GridBagLayout();
	private ConfirmPanel confirmPanel = new ConfirmPanel(true);
	private BorderLayout southLayout = new BorderLayout();

	
	/**
	 *	Static Init
	 * 	@throws Exception
	 */
	void jbInit() throws Exception
	{
		mainPanel.setLayout(mainLayout);
		southPanel.setLayout(southLayout);
		centerPanel.setLayout(centerLayout);
		mainLayout.setVgap(5);
		getContentPane().add(mainPanel);
		mainPanel.add(centerPanel, BorderLayout.CENTER);
		mainPanel.add(southPanel, BorderLayout.SOUTH);
		southPanel.add(confirmPanel, BorderLayout.CENTER);
		//
		confirmPanel.addActionListener(this);
	}	//	jbInit

	/**
	 *	Dynamic Init
	 */
	private void initBPartner()
	{
		//	Get Data
		m_greeting = fillGreeting();
		m_sex = fillSex();
		m_locationType = fillLocationType();

		//	Display
		m_gbc.anchor = GridBagConstraints.NORTHWEST;
		m_gbc.gridx = 0;
		m_gbc.gridy = 0;
		m_gbc.gridwidth = 1;
		m_gbc.weightx = 0;
		m_gbc.weighty = 0;
		m_gbc.fill = GridBagConstraints.HORIZONTAL;
		m_gbc.ipadx = 0;
		m_gbc.ipady = 0;
		m_line = 0;

		//	Contact
		fContact = new VString("Contact", false, false, true, 20, 60, "", null);
		fContact.addActionListener(this);
		fContact.addFocusListener(this);		
		createLeft (fContact, "Contact", true).setFontBold(true);
		//	FirstName
		fFirstName = new VString("FirstName", false, false, true, 20, 60, "", null);
		fFirstName.addActionListener(this);
		fFirstName.addFocusListener(this);		
		createRight (fFirstName, "FirstName").setFontBold(true);
		//	LastNameKana
		fLastNameKana = new VString("LastNameKana", false, false, true, 20, 60, "", null);
		createLeft (fLastNameKana, "LastNameKana", false);
		//	FirstNameKana
		fFirstNameKana = new VString("FirstNameKana", false, false, true, 20, 60, "", null);
		createRight (fFirstNameKana, "FirstNameKana");
		//	Greeting Contact
		fGreetingC = new JComboBox (m_greeting);
		createLeft (fGreetingC, "Greeting", false);
		//	Title
		fTitle = new VString("Title", false, false, true, 20, 60, "", null);
		createRight(fTitle, "Title");
		
		//	Contact Sex
		fContactSex = new CComboBox (m_sex);
		createLeft (fContactSex, "Sex", false);
		fContactSex.setSelectedItem( getSex("0"));
		
		//	Contact Birthday
		fBirthDate = new VDate("Birthday", false, false, true, DisplayType.Date, "Birthday");
		createRight (fBirthDate, "Birthday");
		//	Email
		fEMail = new VString("EMail", false, false, true, 20, 40, "", null);
		createLine (fEMail, "EMail", false);
		
		String value = "<"+getSequenceDocumentNo("C_BPartner")+">";
		//	Value
		fValue = new VString("Value", true, false, true, 20, 60, "", null);
		fValue.setText( value );
		fValue.addActionListener(this);
		createLeft (fValue, "Value", true).setFontBold(true);
		
		//	Greeting Business Partner
		fGreetingBP = new JComboBox (m_greeting);
		//createRight (fGreetingBP, "Greeting");
		//	Sector
		fSector = new VString("Sector", false, false, true, 20, 20, "", null);
		createRight (fSector, "Sector");
		
		//	Name
		fName = new VString("Name", true, false, true, 20, 60, "", null);
		fName.addActionListener(this);
		createLine (fName, "Name", false);
		//	Name2
		fName2 = new VString("Name2", false, false, true, 20, 60, "", null);
		createLine (fName2, "Name2", false);
		//	Description
		fDescription = new VString("Description", false, false, true, 20, 60, "", null);
		createLine (fDescription, "Description", false);
		//	IsBPCompany
		fIsBPCompany = new VCheckBox();
		fIsBPCompany.addActionListener(this);
		createLeft (fIsBPCompany, "IsBPCompany", false);
		//	IsUnsubscribed
		fIsUnsubscribed = new VCheckBox();
		createRight (fIsUnsubscribed, "IsUnsubscribed");
		
		//	Location
		boolean ro = m_readOnly;
		if (!ro)
			ro = !MRole.getDefault().canUpdate(
				Env.getAD_Client_ID(Env.getCtx()), Env.getAD_Org_ID(Env.getCtx()), 
				MBPartnerLocation.Table_ID, 0, false);
		if (!ro)
			ro = !MRole.getDefault().canUpdate(
				Env.getAD_Client_ID(Env.getCtx()), Env.getAD_Org_ID(Env.getCtx()), 
				MLocation.Table_ID, 0, false);
		fAddress = new VLocation ("C_Location_ID", false, ro, true, new MLocationLookup (Env.getCtx(), m_WindowNo));
		fAddress.setValue (null);
		createLine (fAddress, "C_Location_ID", true).setFontBold(true);

		//	AddressKana
		fAddressKana = new VString("AddressKana", false, false, true, 20, 60, "", null);
		fAddressKana.setValue (null);
		fAddressKana.addFocusListener(this);		
		createLine (fAddressKana, "AddressKana", false);
		
		//	Contact LocationType
		fLocationType = new CComboBox (m_locationType);
		createLine (fLocationType, "C_LocationType", false);
		fLocationType.setSelectedItem( getLocationType("Home"));
		
		//	Phone
		fPhone = new VString("Phone", false, false, true, 20, 40, "", null);
		createLeft (fPhone, "Phone", true);
		//	Phone2
		fPhone2 = new VString("Phone2", false, false, true, 20, 40, "", null);
		createRight (fPhone2, "Phone2");

		//	Phone3
		fPhone3 = new VString("Phone3", false, false, true, 20, 40, "", null);
		createLeft (fPhone3, "Phone3", false);
		//	Fax
		fFax = new VString("Fax", false, false, true, 20, 40, "", null);
		createRight (fFax, "Fax");
		//
		fName.setBackground(CompierePLAF.getFieldBackground_Mandatory());
		fAddress.setBackground(CompierePLAF.getFieldBackground_Mandatory());
	}	//	initBPartner

	/**
	 * 	Create Line
	 * 	@param field 	field
	 * 	@param title	label value
	 * 	@param addSpace	add more space
	 * 	@return label
	 */
	private CLabel createLine (JComponent field, String title, boolean addSpace)
	{
		if (addSpace)
		{
			m_gbc.gridy = m_line++;
			m_gbc.gridx = 1;
			m_gbc.insets = m_fieldInsets;
			centerPanel.add (Box.createHorizontalStrut(6), m_gbc);
		}

		m_gbc.gridwidth = 1;
		//	Line
		m_gbc.gridy = m_line++;

		//	Label
		m_gbc.gridx = 0;
		m_gbc.insets = m_labelInsets;
		m_gbc.fill = GridBagConstraints.HORIZONTAL;
		CLabel label = new CLabel(Msg.translate(Env.getCtx(), title));
		centerPanel.add(label, m_gbc);

		m_gbc.gridwidth = 3;
		//	Field
		m_gbc.gridx = 1;
		m_gbc.insets = m_fieldInsets;
		m_gbc.fill = GridBagConstraints.HORIZONTAL;
		centerPanel.add(field, m_gbc);
		if (m_readOnly)
			field.setEnabled(false);
		return label;
	}	//	createLine

	/**
	 * 	Create Line
	 * 	@param field 	field
	 * 	@param title	label value
	 * 	@param addSpace	add more space
	 * 	@return label
	 */
	private CLabel createLeft (JComponent field, String title, boolean addSpace)
	{
		if (addSpace)
		{
			m_gbc.gridy = m_line++;
			m_gbc.gridx = 1;
			m_gbc.insets = m_fieldInsets;
			centerPanel.add (Box.createHorizontalStrut(6), m_gbc);
		}

		m_gbc.gridwidth = 1;
		//	Line
		m_gbc.gridy = m_line;

		//	Label
		m_gbc.gridx = 0;
		m_gbc.insets = m_labelInsets;
		m_gbc.fill = GridBagConstraints.HORIZONTAL;
		CLabel label = new CLabel(Msg.translate(Env.getCtx(), title));
		centerPanel.add(label, m_gbc);

		m_gbc.gridwidth = 1;
		//	Field
		m_gbc.gridx = 1;
		m_gbc.insets = m_fieldInsets;
		m_gbc.fill = GridBagConstraints.HORIZONTAL;
		centerPanel.add(field, m_gbc);
		if (m_readOnly)
			field.setEnabled(false);
		return label;
	}	//	createLine

	/**
	 * 	Create Line
	 * 	@param field 	field
	 * 	@param title	label value
	 * 	@param addSpace	add more space
	 * 	@return label
	 */
	private CLabel createRight (JComponent field, String title)
	{
		//	Line
		m_gbc.gridy = m_line++;

		m_gbc.gridwidth = 1;
		//	Label
		m_gbc.gridx = 2;
		m_gbc.insets = m_labelInsets;
		m_gbc.fill = GridBagConstraints.HORIZONTAL;
		CLabel label = new CLabel(Msg.translate(Env.getCtx(), title));
		centerPanel.add(label, m_gbc);

		m_gbc.gridwidth = 1;
		//	Field
		m_gbc.gridx = 3;
		m_gbc.insets = m_fieldInsets;
		m_gbc.fill = GridBagConstraints.HORIZONTAL;
		centerPanel.add(field, m_gbc);
		if (m_readOnly)
			field.setEnabled(false);
		return label;
	}	//	createLine

	/**
	 *	Fill Greeting
	 * 	@return KeyNamePair Array of Greetings
	 */
	private Object[] fillGreeting()
	{
		String sql = "SELECT C_Greeting_ID, Name FROM C_Greeting WHERE IsActive='Y' ORDER BY 2";
		sql = MRole.getDefault().addAccessSQL(sql, "C_Greeting", MRole.SQL_NOTQUALIFIED, MRole.SQL_RO);
		return DB.getKeyNamePairs(sql, true);
	}	//	fillGreeting

	/**
	 *	Fill Sex
	 * 	@return ValueNamePair Array of Sex
	 */
	private Object[] fillSex()
	{
		ArrayList<ValueNamePair> list = new ArrayList<ValueNamePair>();
		
		String sql = "SELECT ARL.VALUE, NVL((SELECT NAME FROM AD_REF_LIST_TRL ARLT WHERE ARLT.AD_REF_LIST_ID = ARL.AD_REF_LIST_ID "
			+ "AND ARLT.AD_LANGUAGE = '" + Env.getAD_Language( Env.getCtx() )+ "' ), ARL.NAME) AS NAME " 
			+ "FROM  AD_REFERENCE AR, AD_REF_LIST ARL WHERE ARL.AD_REFERENCE_ID = AR.AD_REFERENCE_ID AND AR.NAME = 'AD_UserSex' ";
		
		list.add(new ValueNamePair("", ""));
		PreparedStatement pstmt = null;
		try
		{
			pstmt = DB.prepareStatement(sql, null);
			ResultSet rs = pstmt.executeQuery();
			while (rs.next())
				list.add(new ValueNamePair(rs.getString(1), rs.getString(2)));
			rs.close();
			pstmt.close();
			pstmt = null;
		}
		catch (Exception e)
		{
			log.log(Level.SEVERE, sql, e);
		}
		try
		{
			if (pstmt != null)
				pstmt.close();
			pstmt = null;
		}
		catch (Exception e)
		{
			pstmt = null;
		}
		
		ValueNamePair[] retValue = new ValueNamePair[list.size()];
		list.toArray(retValue);
		return retValue;
		
	}	//	fillGreeting

	/**
	 *	Fill LocationType
	 * 	@return ValueNamePair Array of LocationType
	 */
	private Object[] fillLocationType()
	{
		ArrayList<ValueNamePair> list = new ArrayList<ValueNamePair>();
		
		String sql = "SELECT ARL.VALUE, NVL((SELECT NAME FROM AD_REF_LIST_TRL ARLT WHERE ARLT.AD_REF_LIST_ID = ARL.AD_REF_LIST_ID "
			+ "AND ARLT.AD_LANGUAGE = '" + Env.getAD_Language( Env.getCtx() )+ "' ), ARL.NAME) AS NAME " 
			+ "FROM  AD_REFERENCE AR, AD_REF_LIST ARL WHERE ARL.AD_REFERENCE_ID = AR.AD_REFERENCE_ID AND AR.NAME = 'C_LocationType' ";
		
		list.add(new ValueNamePair("", ""));
		PreparedStatement pstmt = null;
		try
		{
			pstmt = DB.prepareStatement(sql, null);
			ResultSet rs = pstmt.executeQuery();
			while (rs.next())
				list.add(new ValueNamePair(rs.getString(1), rs.getString(2)));
			rs.close();
			pstmt.close();
			pstmt = null;
		}
		catch (Exception e)
		{
			log.log(Level.SEVERE, sql, e);
		}
		try
		{
			if (pstmt != null)
				pstmt.close();
			pstmt = null;
		}
		catch (Exception e)
		{
			pstmt = null;
		}
		
		ValueNamePair[] retValue = new ValueNamePair[list.size()];
		list.toArray(retValue);
		return retValue;		
	}	//	fillGreeting

	/**
	 *	Search m_greeting for key
	 * 	@param key	C_Greeting_ID
	 * 	@return	Greeting
	 */
	private KeyNamePair getGreeting (int key)
	{
		for (int i = 0; i < m_greeting.length; i++)
		{
			KeyNamePair p = (KeyNamePair)m_greeting[i];
			if (p.getKey() == key)
				return p;
		}
		return new KeyNamePair(-1, " ");
	}	//	getGreeting

	/**
	 *	Search m_greeting for key
	 * 	@param key	value
	 * 	@return	Name
	 */
	private ValueNamePair getSex (String value )
	{
		for (int i = 0; i < m_sex.length; i++)
		{
			ValueNamePair p = (ValueNamePair)m_sex[i];
			if( value != null ){
				if (value.equals( p.getValue() ))
					return p;
			}
		}
		return new ValueNamePair("", "");
	}	//	getSex

	/**
	 *	Search m_locationType for key
	 * 	@param key	value
	 * 	@return	Name
	 */
	private ValueNamePair getLocationType (String value )
	{
		for (int i = 0; i < m_locationType.length; i++)
		{
			ValueNamePair p = (ValueNamePair)m_locationType[i];
			if( value != null ){
				if (value.equals( p.getValue() ))
					return p;
			}
		}
		return new ValueNamePair("", "");
	}	//	getLocationType

	/**
	 *	Load BPartner
	 *  @param C_BPartner_ID - existing BPartner or 0 for new
	 * 	@return true if loaded
	 */
	public boolean loadBPartner (int C_BPartner_ID)
	{
		log.config("C_BPartner_ID=" + C_BPartner_ID);
		//  New bpartner
		if (C_BPartner_ID == 0)
		{
			m_partner = null;
			m_pLocation = null;
			m_user = null;
			return true;
		}

		m_partner = new MBPartner (Env.getCtx(), C_BPartner_ID, null);
		if (m_partner.get_ID() == 0)
		{
			ADialog.error(m_WindowNo, this, "BPartnerNotFound");
			return false;
		}

		//	BPartner - Load values
		fValue.setText(m_partner.getValue());
		fGreetingBP.setSelectedItem(getGreeting(m_partner.getC_Greeting_ID()));
		fName.setText(m_partner.getName());
		fName2.setText(m_partner.getName2());
		fDescription.setText(m_partner.getDescription());
		fIsBPCompany.setSelected( m_partner.isBPCompany() );
		fIsUnsubscribed.setSelected( m_partner.isUnsubscribed() );
		fSector.setText(m_partner.getSector());

		//	Contact - Load values
		m_pLocation = m_partner.getLocation(
			Env.getContextAsInt(Env.getCtx(), m_WindowNo, "C_BPartner_Location_ID"));
		if (m_pLocation != null)
		{
			int location = m_pLocation.getC_Location_ID();
			fAddress.setValue (new Integer(location));
			fLocationType.setSelectedItem( getLocationType(m_pLocation.getC_LocationType()) );
			//
			fPhone.setText(m_pLocation.getPhone());
			fPhone2.setText(m_pLocation.getPhone2());
			fFax.setText(m_pLocation.getFax());
			fAddressKana.setText(m_pLocation.getAddressKana());
		}
		//	User - Load values
		m_user = m_partner.getContact(
			Env.getContextAsInt(Env.getCtx(), m_WindowNo, "AD_User_ID"));
		if (m_user != null)
		{
			fGreetingC.setSelectedItem(getGreeting(m_user.getC_Greeting_ID()));
			fContact.setText(m_user.getName());
			fFirstName.setText(m_user.getFirstName());
			fLastNameKana.setText(m_user.getLastNameKana());
			fFirstNameKana.setText(m_user.getFirstNameKana());
			fTitle.setText(m_user.getTitle());
			fContactSex.setSelectedItem(getSex(m_user.getSex()));
			fEMail.setText(m_user.getEMail());
			//
			fPhone.setText(m_user.getPhone());
			fPhone2.setText(m_user.getPhone2());
			fPhone3.setText(m_user.getPhone3());
			fFax.setText(m_user.getFax());
		}
		return true;
	}	//	loadBPartner


	/**
	 *	Action Listener
	 * 	@param e event
	 */
	public void actionPerformed(ActionEvent e)
	{
		if (m_readOnly)
			dispose();
		else if (e.getSource() == fContact)
		{
			if ((fName.getText() == null || fName.getText().length() == 0 
					|| fName.getText().equals( fFirstName.getText())) && !fIsBPCompany.isSelected() ){
				if( fFirstName.getText()==null || fFirstName.getText().length()==0){
					fName.setText( fContact.getText() );
				} else {
					fName.setText( fContact.getText()+ " "+ fFirstName.getText() );
				}
			}
		}
		else if (e.getSource() == fFirstName)
		{
			if ((fName.getText() == null || fName.getText().length() == 0 
					|| fName.getText().equals( fContact.getText())) && !fIsBPCompany.isSelected() ){
				if( fContact.getText()==null || fContact.getText().length()==0 ){
					fName.setText( fFirstName.getText() );
				} else {
					fName.setText( fContact.getText()+ " "+ fFirstName.getText() );
				}
			}
		}
		else if (e.getSource() == fIsBPCompany)
		{
			if ((fName.getText() == null || fName.getText().length() == 0 
					|| fName.getText().equals( fContact.getText())
					|| fName.getText().equals( fFirstName.getText())) && !fIsBPCompany.isSelected() ){
				
				if( fContact.getText()!=null && fContact.getText().length()!=0 
						&& fFirstName.getText()!=null && fFirstName.getText().length()!=0 ){
					fName.setText( fContact.getText()+ " "+ fFirstName.getText() );
				} else if( fContact.getText()!=null && fContact.getText().length()!=0 ){
					fName.setText( fContact.getText() );
				} else {
					fName.setText( fFirstName.getText() );
				}
			}
		}
		//	copy value
		else if (e.getSource() == fValue)
		{
			if (fName.getText() == null || fName.getText().length() == 0)
				fName.setText(fValue.getText());
		}
		//	OK pressed
		else if (e.getActionCommand().equals(ConfirmPanel.A_OK) 
			&& actionSave())
			dispose();
		//	Cancel pressed
		else if (e.getActionCommand().equals(ConfirmPanel.A_CANCEL))
			dispose();
	}	//	actionPerformed

	/**
	 *	Save.
	 *	Checks mandatory fields and saves Partner, Contact and Location
	 * 	@return true if saved
	 */
	private boolean actionSave()
	{
		log.config("");
		String sameAddressBPartner = getSameAddressBPartner(fAddress.getC_Location_ID());
		if( sameAddressBPartner != null ){
			ADialog.warn( m_WindowNo, this, "sameAddressBPartnerExist", Msg.translate(Env.getCtx(), "sameAddressBPartnerExist") +" : " + sameAddressBPartner);
		}
		
		//	Check Mandatory fields
		if (fName.getText().equals(""))
		{
			fName.setBackground(CompierePLAF.getFieldBackground_Error());
			return false;
		}
		else
			fName.setBackground(CompierePLAF.getFieldBackground_Mandatory());
		if (fAddress.getC_Location_ID() == 0)
		{
			fAddress.setBackground(CompierePLAF.getFieldBackground_Error());
			return false;
		}
		else
			fAddress.setBackground(CompierePLAF.getFieldBackground_Mandatory());

		//	***** Business Partner *****
		if (m_partner == null)
		{
			int AD_Client_ID = Env.getAD_Client_ID(Env.getCtx());
			m_partner = MBPartner.getTemplate(Env.getCtx(), AD_Client_ID);
			boolean isSOTrx = !"N".equals(Env.getContext(Env.getCtx(), m_WindowNo, "IsSOTrx"));
			m_partner.setIsCustomer (isSOTrx);
			m_partner.setIsVendor (!isSOTrx);
		}
		//	Check Value
		String value = fValue.getText();
		if (value == null || value.length() == 0 || (value.startsWith( "<") && value.endsWith( ">")))
		{
			//	get Table Documet No
			value = DB.getDocumentNo (Env.getAD_Client_ID(Env.getCtx()), "C_BPartner", null);
			fValue.setText(value);
		}
		m_partner.setValue(fValue.getText());
		//
		m_partner.setName(fName.getText());
		m_partner.setName2(fName2.getText());
		m_partner.setDescription(fDescription.getText());
		m_partner.setIsBPCompany(fIsBPCompany.isSelected());
		m_partner.setIsUnsubscribed(fIsUnsubscribed.isSelected());
		m_partner.setSector(fSector.getText());		
		KeyNamePair p = (KeyNamePair)fGreetingBP.getSelectedItem();
		if (p != null && p.getKey() > 0)
			m_partner.setC_Greeting_ID(p.getKey());
		else
			m_partner.setC_Greeting_ID(0);
		if (m_partner.save())
			log.fine("C_BPartner_ID=" + m_partner.getC_BPartner_ID());
		else
			ADialog.error(m_WindowNo, this, "BPartnerNotSaved");
		
		//	***** Business Partner - Location *****
		if (m_pLocation == null)
			m_pLocation = new MBPartnerLocation(m_partner);
		m_pLocation.setC_Location_ID(fAddress.getC_Location_ID());
		ValueNamePair locationType = (ValueNamePair)fLocationType.getSelectedItem();
		m_pLocation.setC_LocationType( locationType.getValue() );

		//
		m_pLocation.setPhone(fPhone.getText());
		m_pLocation.setPhone2(fPhone2.getText());
		m_pLocation.setFax(fFax.getText());
		m_pLocation.setAddressKana(fAddressKana.getText());
		if (m_pLocation.save())
			log.fine("C_BPartner_Location_ID=" + m_pLocation.getC_BPartner_Location_ID());
		else
			ADialog.error(m_WindowNo, this, "BPartnerNotSaved", Msg.translate(Env.getCtx(), "C_BPartner_Location_ID"));
			
		//	***** Business Partner - User *****
		String contact = fContact.getText();
		String email = fEMail.getText();
		if (m_user == null && (contact.length() > 0 || email.length() > 0))
			m_user = new MUser (m_partner);
		if (m_user != null)
		{
			if (contact.length() == 0)
				contact = fName.getText();
			m_user.setName(contact);
			m_user.setFirstName(fFirstName.getText());
			m_user.setLastNameKana(fLastNameKana.getText());
			m_user.setFirstNameKana(fFirstNameKana.getText());
			ValueNamePair sex = (ValueNamePair)fContactSex.getSelectedItem();
			m_user.setSex( sex.getValue() );
			m_user.setBirthday( fBirthDate.getTimestamp() );
			m_user.setEMail(email);
			m_user.setTitle(fTitle.getText());
			p = (KeyNamePair)fGreetingC.getSelectedItem();
			if (p != null && p.getKey() > 0)
				m_user.setC_Greeting_ID(p.getKey());
			else
				m_user.setC_Greeting_ID(0);
			//
			m_user.setPhone(fPhone.getText());
			m_user.setPhone2(fPhone2.getText());
			m_user.setPhone3(fPhone3.getText());
			m_user.setFax(fFax.getText());
			if (m_user.save())
				log.fine("AD_User_ID=" + m_user.getAD_User_ID());
			else
				ADialog.error(m_WindowNo, this, "BPartnerNotSaved", Msg.translate(Env.getCtx(), "AD_User_ID"));
		}
		return true;
	}	//	actionSave


	/**
	 *	Returns BPartner ID
	 *	@return C_BPartner_ID (0 = not saved)
	 */
	public int getC_BPartner_ID()
	{
		if (m_partner == null)
			return 0;
		return m_partner.getC_BPartner_ID();
	}	//	getBPartner_ID

	/**
	 * 	Get Sequence
	 *	@param ctx context
	 *	@param tableName table name
	 */
	private int getSequenceDocumentNo (String tableName)
	{
		String sql = "SELECT * FROM AD_Sequence "
			+ "WHERE UPPER(Name)=?"
			+ " AND AD_CLIENT_ID=" + Env.getAD_Client_ID( Env.getCtx());
		MSequence retValue = null;
		PreparedStatement pstmt = null;
		try
		{
			pstmt = DB.prepareStatement (sql, null);
			pstmt.setString (1, ("DocumentNo_" + tableName).toUpperCase());
			ResultSet rs = pstmt.executeQuery ();
			if (rs.next ())
				retValue = new MSequence (Env.getCtx(), rs, null);
			if (rs.next())
				log.log(Level.SEVERE, "More then one sequence for " + tableName);
			rs.close ();
			pstmt.close ();
			pstmt = null;
		}
		catch (Exception e)
		{
			log.log(Level.SEVERE, "get", e);
		}
		try
		{
			if (pstmt != null)
				pstmt.close ();
			pstmt = null;
		}
		catch (Exception e)
		{
			pstmt = null;
		}
		
		return retValue.getCurrentNext();
	}	//	get
	
	/**
	 * 	Focus Gained.
	 * 	Enabled with Obscure
	 *	@param e event
	 */
	public void focusGained (FocusEvent e)
	{
		if (e.getSource() == fAddressKana )
		{
			if ((fAddressKana.getText() == null || fAddressKana.getText().length() == 0 ) ){
				String addressKana = getAddressKana( fAddress.getC_Location_ID());
				fAddressKana.setText( addressKana );
			}
		}
	}	//	focusGained

	/**
	 * 	Focus Lost
	 * 	Enabled with Obscure
	 *	@param e event
	 */
	public void focusLost (FocusEvent e)
	{
		if( e.getSource() == fContact ){
			if ((fName.getText() == null || fName.getText().length() == 0 
					|| fName.getText().equals( fFirstName.getText())) && !fIsBPCompany.isSelected() ){
				if( fFirstName.getText()==null || fFirstName.getText().length()==0){
					fName.setText( fContact.getText() );
				} else {
					fName.setText( fContact.getText()+ " "+ fFirstName.getText() );
				}
			}
		}
		else if (e.getSource() == fFirstName)
		{
			if ((fName.getText() == null || fName.getText().length() == 0 
					|| fName.getText().equals( fContact.getText())) && !fIsBPCompany.isSelected() ){
				if( fContact.getText()==null || fContact.getText().length()==0 ){
					fName.setText( fFirstName.getText() );
				} else {
					fName.setText( fContact.getText()+ " "+ fFirstName.getText() );
				}
			}
		}
	}	//	focus Lost
	
	public String getAddressKana( int C_Location_ID ){
		
		String result = null;
		MLocation m_location = new MLocation(Env.getCtx(), C_Location_ID, null);
		
		PreparedStatement pstmt = null;
		StringBuffer sql = new StringBuffer ("SELECT X_ADDRESS, X_REGION_NAME_KANA, X_CITY_NAME_KANA, X_ADDRESS_KANA" 
		    + " FROM X_POSTAL"
			+ " WHERE IsActive='Y'" 
			+ " AND C_COUNTRY_ID=" + m_location.getC_Country_ID()
			+ " AND POSTAL='" + m_location.getPostal()+m_location.getPostal_Add()+"'"
			+ " AND C_REGION_ID=" + m_location.getRegion().getC_Region_ID()
			+ " AND X_CITY_NAME='" + m_location.getCity()+"'");
		try
		{
			pstmt = DB.prepareStatement (sql.toString(), null);
			ResultSet rs = pstmt.executeQuery ();
			while ( rs.next () )
			{
				String address1 = rs.getString( 1 );
				if( m_location.getAddress1()!= null && m_location.getAddress1().startsWith( address1 ) ){
					result = rs.getString( 2 ) + " " + rs.getString( 3 ) + " " + rs.getString( 4 );
					break;
				}
			}
			rs.close ();
			pstmt.close ();
			pstmt = null;
		}
		catch (Exception e)
		{
			log.log(Level.SEVERE, sql.toString(), e);
			try
			{
				if (pstmt != null)
					pstmt.close ();
			}
			catch (Exception pe)
			{
			}
			pstmt = null;
		}
	
		return result;
	}
	
	public String getSameAddressBPartner( int C_Location_ID ){
		
		String result = null;
		MLocation m_location = new MLocation(Env.getCtx(), C_Location_ID, null );
		
		PreparedStatement pstmt = null;
		StringBuffer sql = new StringBuffer ("SELECT C_BPARTNER.* FROM C_BPARTNER, C_BPARTNER_LOCATION, C_LOCATION"
			+ " WHERE C_BPARTNER.IsActive='Y' AND C_BPARTNER_LOCATION.IsActive='Y' AND C_LOCATION.IsActive='Y' " 
			+ " AND C_LOCATION.C_LOCATION_ID = C_BPARTNER_LOCATION.C_LOCATION_ID"
			+ " AND C_BPARTNER_LOCATION.C_BPARTNER_ID = C_BPARTNER.C_BPARTNER_ID"
			+ " AND C_LOCATION.C_COUNTRY_ID=" + m_location.getC_Country_ID()
			+ " AND C_LOCATION.POSTAL='" + m_location.getPostal()+"'" 
			+ " AND C_LOCATION.POSTAL_ADD='" + m_location.getPostal_Add()+"'"
			+ " AND C_LOCATION.C_REGION_ID=" + m_location.getRegion().getC_Region_ID()
			+ " AND C_LOCATION.CITY='" + m_location.getCity()+"'"
			+ " AND C_LOCATION.ADDRESS1='" + m_location.getAddress1()+"'"
			+ " AND C_LOCATION.C_LOCATION_ID<>" + m_location.getC_Location_ID());
		try
		{
			pstmt = DB.prepareStatement (sql.toString(), null);
			ResultSet rs = pstmt.executeQuery ();
			if( rs.next() ){
				MBPartner bpartner = new MBPartner(Env.getCtx(), rs, null );
				result = bpartner.getName();
			}
			rs.close ();
			pstmt.close ();
			pstmt = null;
		}
		catch (Exception e)
		{
			log.log(Level.SEVERE, sql.toString(), e);
			try
			{
				if (pstmt != null)
					pstmt.close ();
			}
			catch (Exception pe)
			{
			}
			pstmt = null;
		}
	
		return result;
	}

}	//	VBPartner

