/******************************************************************************
 * The contents of this file are subject to the   Compiere License  Version 1.1
 * ("License"); You may not use this file except in compliance with the License
 * You may obtain a copy of the License at http://www.compiere.org/license.html
 * Software distributed under the License is distributed on an  "AS IS"  basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 * The Original Code is Compiere ERP & CRM Smart Business Solution. The Initial
 * Developer of the Original Code is Jorg Janke. Portions created by Jorg Janke
 * are Copyright (C) 1999-2005 Jorg Janke.
 * All parts are Copyright (C) 1999-2005 ComPiere, Inc.  All Rights Reserved.
 * Contributor(s): ______________________________________.
 *****************************************************************************/
package org.compiere.grid.ed;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.VetoableChangeListener;
import java.util.ArrayList;
import java.util.EventObject;
import java.util.HashMap;
import java.util.logging.Level;

import javax.swing.AbstractCellEditor;
import javax.swing.JTable;
import javax.swing.UIManager;
import javax.swing.table.TableCellEditor;

import org.compiere.grid.VACPanel;
import org.compiere.model.GridField;
import org.compiere.model.GridTab;
import org.compiere.model.GridTable;
import org.compiere.model.MTreeNode;
import org.compiere.model.M_FieldFormat;
import org.compiere.model.M_TabFormat;
import org.compiere.plaf.CompierePLAF;
import org.compiere.util.CLogger;
import org.compiere.util.DisplayType;
import org.compiere.util.Env;

/**
 *  Cell Editor.
 *  <pre>
 *		Sequence of events:
 *			isCellEditable
 *			getTableCellEditor
 *			shouldSelectCell
 *			getCellEditorValue
 *  </pre>
 *  A new Editor is created for editable tab in columns.
 *  @author 	Jirimuto
 *  @version 	$Id: VFormatedCellEditor.java,v 1.1 2006/09/25 14:19:52 hans Exp $
 */
public final class VFormatedCellEditor extends AbstractCellEditor
	implements TableCellEditor, VetoableChangeListener, ActionListener
{
	/**
	 *	Constructor for Grid
	 *  @param mField
	 */
	public VFormatedCellEditor (GridTab mTab,  M_TabFormat mTabFormat)
	{
		super();
		m_mTab = mTab;
		m_mTabFormat = mTabFormat;
		createEditor(mTab, mTabFormat);
	}	//	VCellEditor

	/** The Tab               */
	private GridTab          m_mTab = null;

	private ArrayList<GridTab>      m_includedTabs = new ArrayList<GridTab>();
	
	private HashMap<String,VEditor> m_vEditors = new HashMap<String,VEditor>();
	/** The Tab Format              */
	private M_TabFormat      m_mTabFormat = null;
	
	/** The Table Editor        */
	private VACPanel vPanel = new VACPanel();
	/** Table                   */
	private JTable          m_table = null;

	/** Table                   */
	private Integer         m_mKeyValue = null;
	
	/** ClickCount              */
	private static int      CLICK_TO_START = 1;
	
	/**	Logger			*/
	private static CLogger log = CLogger.getCLogger(VFormatedCellEditor.class);

	/**
	 *  Create Editor
	 */
	private void createEditor(GridTab mTab,  M_TabFormat mTabFormat)
	{
		if( mTabFormat != null ) {
			vPanel.setRowAndCol( mTabFormat.getTab_RowCount(), mTabFormat.getTab_ColCount() );
		}
		//	Set Softcoded Mnemonic &x
		int size = mTab.getFields().length;
		for (int i = 0; i < size; i++)
		{
			GridField mField = mTab.getField(i);
			if (mField.isDisplayed())
				vPanel.setMnemonic(mField);
		}   //  for all fields
		
		//	Add Fields
		for (int i = 0; i < size; i++)
		{
			GridField mField = mTab.getField(i);
			if (mField.isDisplayed())
			{
				VEditor vEditor = VEditorFactory.getEditor(mTab, mField, false);
				if (vEditor == null)
				{
					log.severe("Editor not created for " + mField.getColumnName());
					continue;
				}
				m_vEditors.put( mField.getColumnName(), vEditor);
				if( mField.getDisplayType()==DisplayType.List || mField.getDisplayType()==DisplayType.TableDir ){
					((VLookup)vEditor).setComboEditable( true);
				}
				//  MField => VEditor - New Field value to be updated to editor
				mField.addPropertyChangeListener(vEditor);
				//  VEditor => this - New Editor value to be updated here (MTable)
				vEditor.addVetoableChangeListener(this);
				
				M_FieldFormat m_fieldFormat = M_FieldFormat.getM_FieldFormat(Env.getCtx(), mTab.getAD_Tab_ID(), mField.getAD_Column_ID() );
				//  Add to VPanel
				vPanel.addFieldWithFormat(vEditor, mField, m_fieldFormat);
				
			}
			
		}   //  for all fields
		
		if( mTabFormat!=null ){
			vPanel.prepareComplete( mTabFormat.getTab_ColCount() );
		} else {
			vPanel.prepareComplete( 0 );
		}
		
	}   //  createEditor

	/**
	 *  append Editor
	 */
	public void appendEditor( GridTab mTab )
	{
		m_includedTabs.add( mTab );
		
		//	Set Softcoded Mnemonic &x
		int size = mTab.getFields().length;
		for (int i = 0; i < size; i++)
		{
			GridField mField = mTab.getField(i);
			if (mField.isDisplayed())
				vPanel.setMnemonic(mField);
		}   //  for all fields
		
		//	Add Fields
		for (int i = 0; i < size; i++)
		{
			GridField mField = mTab.getField(i);
			if (mField.isDisplayed())
			{
				VEditor vEditor = VEditorFactory.getEditor(mTab, mField, false);
				if (vEditor == null)
				{
					log.severe("Editor not created for " + mField.getColumnName());
					continue;
				}
				m_vEditors.put( mField.getColumnName(), vEditor);
				if( mField.getDisplayType()==DisplayType.List || mField.getDisplayType()==DisplayType.TableDir ){
					((VLookup)vEditor).setComboEditable( true);
				}
				//  MField => VEditor - New Field value to be updated to editor
				mField.addPropertyChangeListener(vEditor);
				//  VEditor => this - New Editor value to be updated here (MTable)
				vEditor.addVetoableChangeListener(this);
				
				M_FieldFormat m_fieldFormat = M_FieldFormat.getM_FieldFormat(Env.getCtx(), mTab.getAD_Tab_ID(), mField.getAD_Column_ID() );
				//  Add to VPanel
				vPanel.addFieldWithFormat(vEditor, mField, m_fieldFormat);
				
			}
			
		}   //  for all fields
		
		if( m_mTabFormat!=null ){
			vPanel.prepareComplete( m_mTabFormat.getTab_ColCount() );
		} else {
			vPanel.prepareComplete( 0 );
		}
		
	}   //  createEditor

	/**
	 *	Ask the editor if it can start editing using anEvent.
	 *	If editing can be started this method returns true.
	 *	Previously called: MTable.isCellEditable
	 *  @param anEvent event
	 *  @return true if editable
	 */
	public boolean isCellEditable (EventObject anEvent)
	{
		log.fine(m_mTab.getName());		//	event may be null if CellEdit
			
		return true;
	}	//	isCellEditable

	/**
	 *	Sets an initial value for the editor. This will cause the editor to
	 *	stopEditing and lose any partially edited value if the editor is editing
	 *	when this method is called.
	 *	Returns the component that should be added to the client's Component hierarchy.
	 *	Once installed in the client's hierarchy this component
	 *	will then be able to draw and receive user input.
	 *
	 *  @param table
	 *  @param value
	 *  @param isSelected
	 *  @param row
	 *  @param col
	 *  @return component
	 */
	public Component getTableCellEditorComponent (JTable table, Object value, boolean isSelected, int row, int col)
	{
		log.finest(m_mTab.getName() + ": Value=" + value + ", row=" + row + ", col=" + col);
		if (row >= 0)
			table.setRowSelectionInterval(row,row);     //  force moving to new row

		m_table = table;
		m_table.requestFocus();
		if( value instanceof Integer){
			m_mKeyValue = (Integer)value;
		}
		for(int j=0; j<m_includedTabs.size(); j++ ){
			GridTab mTab = (GridTab)m_includedTabs.get(j);
			GridTable mTable = mTab.getTableModel();
			GridTab mParentTab = mTab.getParentTab();
			GridTable mParentTable = mParentTab.getTableModel();
			int parentCol = mParentTable.getFieldIndex( mTab.getLinkColumnName() );
			if( parentCol >= 0 ) {
				Object linkValueObject =  mParentTable.getValueAt( row, parentCol );
				int childRow=mTab.getRowCount();
				if( linkValueObject != null ){
					String linkValue =linkValueObject.toString();
					int colno = mTable.getFieldIndex( mTab.getLinkColumnName() );
					for( childRow=0; childRow<mTab.getRowCount(); childRow++){
						Object childLinkValue = mTable.getValueAt( childRow, colno );
						if( childLinkValue!=null && linkValue.equals( childLinkValue.toString() ) ){
							break;
						}
					}
				}
				if( childRow == mTab.getRowCount()){
					if( mTab.isOpen() ){
						if( !mTab.isInsertRecord() && mParentTab.getRowCount() != mTab.getRowCount() ){
							mTab.dataNew( false );
						}
						mTab.navigate( mParentTab.getCurrentRow() );
						mTab.setValue( mTab.getLinkColumnName(), mParentTable.getValueAt( row, parentCol ) );
						childRow = mTab.getCurrentRow();
					}
				} else if( childRow >= 0 ){
					mTab.navigate( childRow, false);
				}
				
			}
		}
		//  Other UI
		vPanel.setFont(table.getFont());
		vPanel.setBackground( m_table.getSelectionBackground() );		
		vPanel.setBorder(UIManager.getBorder("Table.focusCellHighlightBorder"));
		vPanel.requestFocus();
		
		return (Component)vPanel;
	}	//	getTableCellEditorComponent

	/**
	 *	The editing cell should be selected or not
	 *  @param e event
	 *  @return true (constant)
	 */
	public boolean shouldSelectCell(EventObject e)
	{
		log.finest(m_mTab.getName());
		return true;
	}	//	shouldSelectCell

	/**
	 *	Returns the value contained in the editor
	 *  @return value
	 */
	public Object getCellEditorValue()
	{
		return m_mKeyValue;
	}	//	getCellEditorValue

	/**
	 *	set the value to the editor
	 *  @return value
	 */
	public void setCellEditorValue(Integer value)
	{
		m_mKeyValue = value;
	}	//	setCellEditorValue

	/**
	 *  VEditor Change Listener (property name is columnName).
	 *  - indicate change  (for String/Text/..) <br>
	 *  When editing is complete the value is retrieved via getCellEditorValue
	 *  @param e event
	 */
	public void vetoableChange(PropertyChangeEvent e)
	{
		if (m_table == null)
			return;
		Object value = e.getNewValue();
		if (value == null)
			return;
		log.config(e.getPropertyName() + "=" + value + " - " + value.getClass().toString());

		//
		try {
			String columnName = e.getPropertyName();
			if( columnName != null ){
				int row = m_mTab.getCurrentRow();
				GridTable m_mTable = (GridTable)m_table.getModel();
				int col = m_mTable.getFieldIndex( columnName );
				if( col >= 0 ){
					String newValue = e.getNewValue().toString();
					Object oldValue = (Object)e.getOldValue();
					if( oldValue==null || !newValue.equals( oldValue.toString() )){
						m_mTable.setValueAt( e.getNewValue(), row, col);
						m_mTable.setChanged(true);
					}
					return;
				}
				for(int j=0; j<m_includedTabs.size(); j++ ){
					GridTab mTab = (GridTab)m_includedTabs.get(j);
					GridTable mTable = mTab.getTableModel();
					GridTab mParentTab = mTab.getParentTab();
					GridTable mParentTable = mParentTab.getTableModel();
					int parentCol = mParentTable.getFieldIndex( mTab.getLinkColumnName() );
					if( parentCol >= 0 ) {
						Object linkValueObject = mParentTable.getValueAt( row, parentCol );
						if( linkValueObject != null ){
							String linkValue = linkValueObject.toString();
							col = mTable.getFieldIndex( mTab.getLinkColumnName() );
							int childRow=-1;
							for( childRow=0; childRow<mTab.getRowCount(); childRow++){
								Object childLinkValue = mTable.getValueAt( childRow, col );
								if( childLinkValue == null && childRow==row ){
									mTab.setValue( mTab.getLinkColumnName(), mParentTable.getValueAt( row, parentCol ) );
								}
								if( childLinkValue!=null && linkValue.equals( childLinkValue.toString() ) ){
									break;
								}
							}
							if( childRow == mTab.getRowCount()){
								if( mTab.getRowCount() != mParentTab.getRowCount() ){
									if( mTab.isOpen() ){
										mTab.dataNew( false );
										mTab.setValue( mTab.getLinkColumnName(), mParentTable.getValueAt( row, parentCol ) );
										childRow = mTab.getCurrentRow();
									} else {
										childRow = 0;
									}
								}
							}
							
							mTab.navigate( childRow );
							int childCol = mTable.getFieldIndex( columnName );
							if( childCol >= 0 ){
								String newValue = e.getNewValue().toString();
								Object oldValue = (Object)e.getOldValue();
								if( oldValue==null || !newValue.equals( oldValue.toString() )){
									mTable.setValueAt( e.getNewValue(), childRow, childCol);
									mTable.setChanged(true);
									mParentTable.setChanged( true );								
									return;
								}
							}
							
						} else {
							
							int childRow = row;
							if( childRow >= mTab.getRowCount() ){
								if( mTab.isOpen() ){
									mTab.dataNew( false );
									mTab.setValue( mTab.getLinkColumnName(), mParentTable.getValueAt( row, parentCol ) );
									childRow = mTab.getCurrentRow();
								} else {
									childRow = 0;
								}
							}
							mTab.navigate( childRow );
							int childCol = mTable.getFieldIndex( columnName );
							if( childCol >= 0 ){
								String newValue = e.getNewValue().toString();
								Object oldValue = (Object)e.getOldValue();
								if( oldValue==null || !newValue.equals( oldValue.toString() )){
									mTable.setValueAt( e.getNewValue(), childRow, childCol);
									mTable.setChanged(true);
									mParentTable.setChanged( true );
									return;
								}
							}
						}
					}
				}
			}
			
		} catch ( Exception ex ){
			
			String columnName = e.getPropertyName();
			log.log(Level.SEVERE, "(ref) - Column=" + columnName, ex);
			
		}
	}   //  vetoableChange

	/**
	 *  Action Editor - Stop Editor
	 *  @param e event
	 */
	public void actionPerformed (ActionEvent e)
	{
		System.out.println("VFormatedCellEditor : Action="+e.getActionCommand());
		
		//		super.stopCellEditing();	//	causes VLookup.Search Text not to work
	}   //  actionPerformed

	/**
	 * 	Dispose
	 */
	public void dispose()
	{
		m_mTab = null;
		m_mTabFormat = null;
		m_table = null;
	}	//	dispose

	/**
	 *  Get Actual Editor.
	 *  Called from GridController to add ActionListener to Button
	 *  @return VEditor
	 */
	public VEditor getEditor()
	{
		return vPanel;
	}   //  getEditor

	public int getHeight(){
		return vPanel.getHeight();
	}
	
}	//	VCellEditor
