/******************************************************************************
 * The contents of this file are subject to the   Compiere License  Version 1.1
 * ("License"); You may not use this file except in compliance with the License
 * You may obtain a copy of the License at http://www.compiere.org/license.html
 * Software distributed under the License is distributed on an  "AS IS"  basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 * The Original Code is Compiere ERP & CRM Smart Business Solution. The Initial
 * Developer of the Original Code is Jorg Janke. Portions created by Jorg Janke
 * are Copyright (C) 1999-2005 Jorg Janke.
 * All parts are Copyright (C) 1999-2005 ComPiere, Inc.  All Rights Reserved.
 * Contributor(s): ______________________________________.
 *****************************************************************************/
package org.compiere.grid.ed;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.VetoableChangeListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.logging.Level;

import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;

import org.compiere.grid.VACPanel;
import org.compiere.model.GridField;
import org.compiere.model.GridTab;
import org.compiere.model.GridTable;
import org.compiere.model.M_FieldFormat;
import org.compiere.model.M_TabFormat;
import org.compiere.plaf.CompierePLAF;
import org.compiere.util.CLogger;
import org.compiere.util.Env;

/**
 *  Table Cell Renderer based on DisplayType
 *
 * 	@author 	Jirimuto
 * 	@version 	$Id: VFormatedCellRenderer.java,v 1.1 2006/09/25 14:19:56 hans Exp $
 */
public final class VFormatedCellRenderer extends DefaultTableCellRenderer 
		implements VetoableChangeListener, ActionListener
{
	/**
	 *	Constructor for Grid
	 *  @param mField field model
	 */
	public VFormatedCellRenderer(GridTab mTab, M_TabFormat mTabFormat)
	{
		super();
		m_columnName = mTab.getName();
		m_mTab = mTab;
		m_mTabFormat = mTabFormat;
		this.setName(m_columnName);
		
		if( m_mTabFormat != null ) {
			vPanel.setRowAndCol( m_mTabFormat.getTab_RowCount(), m_mTabFormat.getTab_ColCount() );
		}
		
		//	Set Softcoded Mnemonic &x
		int size = mTab.getFields().length;
		for (int i = 0; i < size; i++)
		{
			GridField mField = mTab.getField(i);
			if (mField.isDisplayed())
				vPanel.setMnemonic(mField);
		}   //  for all fields
		
		//	Add Fields
		for (int i = 0; i < size; i++)
		{
			GridField mField = mTab.getField(i);
			if (mField.isDisplayed())
			{
				VCellRendererEditor vEditor = new VCellRendererEditor( mField );
				// VEditor vEditor = VEditorFactory.getUnbindEditor(mTab, mField, false);
				if (vEditor == null)
				{
					log.severe("Editor not created for " + mField.getColumnName());
					continue;
				}
				
				vEditorMap.put(mTab.getTableName()+"-"+mField.getColumnName(), vEditor);
				
				M_FieldFormat m_fieldFormat = M_FieldFormat.getM_FieldFormat(Env.getCtx(), mTab.getAD_Tab_ID(), mField.getAD_Column_ID() );
				//  Add to VPanel
				vPanel.addFieldWithFormat(vEditor, mField, m_fieldFormat);
				
			}
			
		}   //  for all fields
		
		if( m_mTabFormat!=null ){
			vPanel.prepareComplete( m_mTabFormat.getTab_ColCount() );
		} else {
			vPanel.prepareComplete( 0 );
		}
		vPanel.setOpaque(true); 

		
	}	//	VCellRenderer

	/**
	 *	Constructor for Grid
	 *  @param mField field model
	 */
	public void appendFormatedCellRenderer( GridTab mTab )
	{
		m_includedTabs.add( mTab );
		this.setName(m_columnName);
		
		//	Set Softcoded Mnemonic &x
		int size = mTab.getFields().length;
		for (int i = 0; i < size; i++)
		{
			GridField mField = mTab.getField(i);
			if (mField.isDisplayed())
				vPanel.setMnemonic(mField);
		}   //  for all fields
		
		//	Add Fields
		for (int i = 0; i < size; i++)
		{
			GridField mField = mTab.getField(i);
			if (mField.isDisplayed())
			{
				VCellRendererEditor vEditor = new VCellRendererEditor( mField );
				if (vEditor == null)
				{
					log.severe("Editor not created for " + mField.getColumnName());
					continue;
				}
				vEditorMap.put( mTab.getTableName()+"-"+mField.getColumnName(), vEditor);
				
				M_FieldFormat m_fieldFormat = M_FieldFormat.getM_FieldFormat(Env.getCtx(), mTab.getAD_Tab_ID(), mField.getAD_Column_ID() );
				//  Add to VPanel
				vPanel.addFieldWithFormat(vEditor, mField, m_fieldFormat);
				
			}
			
		}   //  for all fields
		
		if( m_mTabFormat!=null ){
			vPanel.prepareComplete( m_mTabFormat.getTab_ColCount() );
		} else {
			vPanel.prepareComplete( 0 );
		}
		vPanel.setOpaque(true); 

		
	}	//	VCellRenderer
	
	private String	m_columnName = null;
	private GridTab m_mTab = null;
	private M_TabFormat m_mTabFormat = null;
	private ArrayList<GridTab>      m_includedTabs = new ArrayList<GridTab>();
	
	private HashMap<String, VEditor> vEditorMap = new HashMap<String, VEditor>();
	
	private HashMap<String, GridTab> mChildTableMap = new HashMap<String, GridTab>();

	private VACPanel vPanel = new VACPanel();
	
	/**	Logger			*/
	private static CLogger log = CLogger.getCLogger(VFormatedCellRenderer.class);
	
	/**
	 *	Get TableCell RendererComponent.
	 *  @param table table
	 *  @param value value
	 *  @param isSelected selected
	 *  @param hasFocus focus
	 *  @param row row
	 *  @param col col
	 *  @return component
	 */
	public Component getTableCellRendererComponent (JTable table, Object value,
		boolean isSelected, boolean hasFocus, int row, int col)
	{
	//	log.finest(m_columnName 
	//		+ ": Value=" + (value == null ? "null" : value.toString()) 
	//		+ ", Row=" + row + ", Col=" + col);
		Component c = null;
		c = vPanel;
		c.setFont(CompierePLAF.getFont_Field());

		if( isSelected ){
			vPanel.setBorder(javax.swing.BorderFactory.createLineBorder(java.awt.Color.GRAY,1) );
		} else {
			vPanel.setBorder( null );
		}
		
		//  Background & Foreground
		Color bg = CompierePLAF.getFieldBackground_Normal();
		Color fg = CompierePLAF.getTextColor_Normal();
		//  Inactive Background
		boolean ro = true; 
		if (ro)
			bg = CompierePLAF.getFieldBackground_Inactive();
		//  Selected is white on blue in Windows
		if (isSelected && !hasFocus)
		{
			if (!ro)
				bg = table.getSelectionBackground();
			fg = table.getSelectionBackground();
		}
		else 	//  row not selected or field has focus
		{
			//  Foreground
			int cCode = 0;
			//  Grid
			if (table instanceof org.compiere.grid.VTable)
				cCode = ((org.compiere.grid.VTable)table).getColorCode (row);
			//  MiniGrid
			else if (table instanceof org.compiere.minigrid.MiniTable)
				cCode = ((org.compiere.minigrid.MiniTable)table).getColorCode (row);
			//
			if (cCode == 0)
				;
			else if (cCode < 0)
				fg = CompierePLAF.getTextColor_Issue();
			else
				fg = CompierePLAF.getTextColor_OK();
		}
		//  Set Color
		c.setBackground(bg);
		c.setForeground(fg);
		//
	//	log.fine( "r=" + row + " c=" + col, // + " - " + c.getClass().getName(),
	//		"sel=" + isSelected + ", focus=" + hasFocus + ", edit=" + table.isCellEditable(row, col));
	//	Log.trace(7, "BG=" + (bg.equals(Color.white) ? "white" : bg.toString()), "FG=" + (fg.equals(Color.black) ? "black" : fg.toString()));

		//  Format it
		setValue(value, row);
		return c;
	}	//	getTableCellRendererComponent


	/**
	 *	Format Display Value.
	 *	Default is JLabel
	 *  @param value (key)value
	 */
	protected void setValue (Object value, int row)
	{
		try
		{
			super.setValue( value );
			if( value instanceof Integer ){
				int key_id = ((Integer)value).intValue();
				int tmp_key_id = m_mTab.getKeyID( row );
				if( tmp_key_id == key_id ){
					GridTable m_mTable = m_mTab.getTableModel();
					for(int i=0; i<m_mTable.getFieldCount(); i++ ){
						Object fieldValue = m_mTable.getValueAt(row, i);
						GridField tmp_mField = m_mTable.getFields()[i];
						if( tmp_mField!=null && tmp_mField.isDisplayed() ){
							VEditor vEditor = vEditorMap.get(m_mTab.getTableName()+"-"+tmp_mField.getColumnName());
							if( vEditor != null ){
								Component label = null;
								if( vEditor instanceof VCellRendererEditor){
									label = ((VCellRendererEditor)vEditor).getM_Label();
								}
								vEditor.setValue( fieldValue );
								if( tmp_mField.isDisplayed(true) ){
									vEditor.setVisible( true );
									if( label != null ){
										label.setVisible( true );
									}
								} else {
									vEditor.setVisible( false );
									if( label != null ){
										label.setVisible( false );
									}
								}
							}
						}
					}
					for( int j=0; j<m_includedTabs.size(); j++ ){
						GridTab mTab = (GridTab)m_includedTabs.get(j);
						GridTable mTable = mTab.getTableModel();
						GridTab mParentTab = mTab.getParentTab();
						GridTable mParentTable = mParentTab.getTableModel();
						int parentCol = mParentTable.getFieldIndex( mTab.getLinkColumnName() );
						if( parentCol >= 0 ) {
							String linkValue = mParentTable.getValueAt( row, parentCol ).toString();
							int col = mTable.getFieldIndex( mTab.getLinkColumnName() );
							int childRow=-1;
							for( childRow=0; childRow<mTab.getRowCount(); childRow++){
								Object childLinkObj = mTable.getValueAt( childRow, col );
								if( childLinkObj != null ){
									if( linkValue.equals( childLinkObj.toString() ) ){
										break;
									}
								}
							}
							if( childRow == mTab.getRowCount()){
								if( mTab.isOpen() ){
									mTab.childDataNew( false );
									mTab.setValue( mTab.getLinkColumnName(), mParentTable.getValueAt( row, parentCol ) );
									childRow = mTab.getCurrentRow();
								} else {
									childRow = 0;
								}
							}
							if( childRow >= 0 ){
								for(int i=0; i<mTable.getFieldCount(); i++ ){
									Object fieldValue = mTable.getValueAt(childRow, i);
									GridField tmp_mField = mTable.getFields()[i];
									if( tmp_mField!=null && tmp_mField.isDisplayed() ){
										VEditor vEditor = vEditorMap.get(mTab.getTableName()+"-"+tmp_mField.getColumnName());
										if( vEditor != null ){
											vEditor.setValue( fieldValue );
											Component label = null;
											if( vEditor instanceof VCellRendererEditor){
												label = ((VCellRendererEditor)vEditor).getM_Label();
											}
											vEditor.setValue( fieldValue );
											if( tmp_mField.isDisplayed(true) ){
												vEditor.setVisible( true );
												if( label != null ){
													label.setVisible( true );
												}
											} else {
												vEditor.setVisible( false );
												if( label != null ){
													label.setVisible( false );
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
			
		}
		catch (Exception e)
		{
			log.log(Level.SEVERE, "(" + value + ") " + value.getClass().getName() , e);
		}
	}	//	setValue

	/**
	 *  to String
	 *  @return String representation
	 */
	public String toString()
	{
		return "VFormatedCellRenderer[" + m_columnName + "]";
	}   //  toString

	/**
	 * 	Dispose
	 */
	public void dispose()
	{
		
		m_mTab = null;
		m_mTabFormat = null;
	}	//	dispose

	public int getHeight(){
		return vPanel.getHeight();
	}

	public void vetoableChange(PropertyChangeEvent e)
	{
		// do nothing
		log.config(e.getPropertyName() + "=" + e.getNewValue() );
		return;
	}   //  vetoableChange

	/**
	 *  Action Editor - Stop Editor
	 *  @param e event
	 */
	public void actionPerformed (ActionEvent e)
	{
		//System.out.println("VFormatedCellRenderer : Action="+e.getActionCommand());
		
		//		super.stopCellEditing();	//	causes VLookup.Search Text not to work
	}   //  actionPerformed

	
}	//	VFormatCellRenderer
