/******************************************************************************
 * The contents of this file are subject to the   Compiere License  Version 1.1
 * ("License"); You may not use this file except in compliance with the License
 * You may obtain a copy of the License at http://www.compiere.org/license.html
 * Software distributed under the License is distributed on an  "AS IS"  basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 * The Original Code is Compiere ERP & CRM Smart Business Solution. The Initial
 * Developer of the Original Code is Jorg Janke. Portions created by Jorg Janke
 * are Copyright (C) 1999-2005 Jorg Janke.
 * All parts are Copyright (C) 1999-2005 ComPiere, Inc.  All Rights Reserved.
 * Contributor(s): ______________________________________.
 *****************************************************************************/
package org.compiere.grid.ed;

import java.awt.Component;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;

import javax.swing.ComboBoxEditor;
import javax.swing.ComboBoxModel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import org.compiere.util.NamePair;

/**
 *  Lookup Visual Field.
 *  <p>
 *	    When r/o - display a Label
 *		When STABLE - display a ComboBox
 *		Otherwise show Selection Dialog
 *  <p>
 *  Sepecial handling of BPartner and Product
 *
 *  @author 	Jorg Janke
 *  @version 	$Id: VLookupEditor.java,v 1.1 2006/09/25 14:19:56 hans Exp $
 */
public class VLookupEditor  implements ComboBoxEditor, DocumentListener
{
	public JTextField m_Text;
	private volatile boolean filtering = false;
	private volatile boolean setting = false;
	public volatile boolean editing = false;
	private VComboBox m_Parent = null;
	
	public VLookupEditor(VComboBox parent) {
		super();
		m_Parent = parent;
		m_Text = new JTextField(15);
		m_Text.setBackground( parent.getBackground() );
		m_Text.getDocument().addDocumentListener(this);
		init();
	}

	/**
	 * Jirimuto Add For Focus Forward 
	 */
	private void init() {
		m_Text.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				if (e.getKeyCode() == KeyEvent.VK_ENTER)
				{
					((Component)e.getSource()).transferFocus();
				}
			}
		});
	}
	
	public Component getEditorComponent() { return m_Text; }

	public void setItem(Object item) {
	
		if(filtering)
			return;
		if(editing)
			return;
		setting = true;
		
		String newText = "";
		if( item instanceof NamePair ){
			newText = ((NamePair)item).getName();
		} else if( item != null ){
			newText = item.toString();
		}
		try {
			m_Text.setText(newText);
		} catch (Exception e) {
			System.err.println("set editer faild");
		}
		setting = false;
	}

	public Object getItem() {
		
		String value = m_Text.getText();
		
		ComboBoxModel m_model = m_Parent.getModel();
		if( m_model == null )
			return null;
		// Edit by Name
		for( int i=0; i<m_model.getSize(); i++ ){
			Object element = m_model.getElementAt(i);
			if( element instanceof NamePair ){
				String name = ((NamePair)element).getName();
				if( name.equals( value.toString() ) ){
					return element;
				}
			}
		}
		for( int i=0; i<m_model.getSize(); i++ ){
			Object element = m_model.getElementAt(i);
			if( element instanceof NamePair ){
				String name = ((NamePair)element).getName();
				if( name.equalsIgnoreCase( value.toString() ) ){
					return element;
				}
			}
		}
		for( int i=0; i<m_model.getSize(); i++ ){
			Object element = m_model.getElementAt(i);
			if( element instanceof NamePair ){
				String name = ((NamePair)element).getName();
				if( name.startsWith( value.toString() ) ){
					return element;
				}
			}
		}
		// Edit by ID
		for( int i=0; i<m_model.getSize(); i++ ){
			Object element = m_model.getElementAt(i);
			if( element instanceof NamePair ){
				String id = ((NamePair)element).getID();
				if( id.equals( value.toString() ) ){
					return element;
				}
			}
		}
		// Edit by ID
		for( int i=0; i<m_model.getSize(); i++ ){
			Object element = m_model.getElementAt(i);
			if( element instanceof NamePair ){
				String id = ((NamePair)element).getID();
				if( id.startsWith( value.toString() ) ){
					return element;
				}
			}
		}
		
		return null;
		
	}

	public void selectAll() { m_Text.selectAll(); }

	public void addActionListener(ActionListener l) {
		m_Text.addActionListener(l);
	}

	public void removeActionListener(ActionListener l) {
		m_Text.removeActionListener(l);
	}

	public void insertUpdate(DocumentEvent e) { handleChange(); }
	public void removeUpdate(DocumentEvent e) { handleChange(); }
	public void changedUpdate(DocumentEvent e) { }

	protected void handleChange() {
	
		if (setting)
			return;
	
		filtering = true;
	
	}

	public void setEditing(boolean editing) {
		this.editing = editing;
	}

	public VComboBox getParent() {
		return m_Parent;
	}

	public void setParent(VComboBox parent) {
		m_Parent = parent;
	}
	
	public void addFocusListener(FocusListener listener) {
		m_Text.addFocusListener(listener);
	}
	
}	// VLookupEditor

